/**
 *******************************************************************************
 * @file    tmpm066_tspi.h
 * @brief   This file provides all the functions prototypes for TSPI driver.
 * @version V2.0.2.1
 * @date    2015/09/08
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM066_TSPI_H
#define __TMPM066_TSPI_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "TMPM066.h"
#include "tx00_common.h"


/** @addtogroup TX00_Periph_Driver
  * @{
  */

/** @addtogroup TSPI
  * @{
  */

/** @defgroup TSPI_Exported_Types
  * @{
  */

#define IS_TSPI_UNIT(param)             ((param) == TSB_TSPI0)

#define TSPI_TX                         (uint8_t)0x00
#define TSPI_RX                         (uint8_t)0x01
#define IS_TSPI_TXRX(param)             ((param) <= TSPI_RX)

#define IS_TSPI_FIFOLEVELINT(param)     ((param) <= 8U)

/* for CR2 bit2, 4-7 : INT control */
#define TSPI_INT_ERR      (uint32_t)0x04        /* Parity error interrupt control */
#define TSPI_INT_RXEND     (uint32_t)0x10       /* Receive completation interrupt control */
#define TSPI_INT_RXFIFO    (uint32_t)0x20       /* Receive FIFO interrupt control */
#define TSPI_INT_TXEND     (uint32_t)0x40       /* Transmit completation interrupt control */
#define TSPI_INT_TXFIFO    (uint32_t)0x80       /* Transmit FIFO interrupt control */
#define TSPI_INT_ALL       (uint32_t)0xF4       /* All above interrupt control */
#define IS_TSPI_INTSRC(param)              (((param) & TSPI_INT_ALL) && \
                                            (!((param) & (uint32_t) (~TSPI_INT_ALL))))


    typedef enum {
        TSPI_MODE_SPI = 0U,
        TSPI_MODE_SIO = 1U
    } TSPI_Mode;
#define IS_TSPI_MODE(param)             ((param) <= TSPI_MODE_SIO)

    typedef enum {
        TSPI_MSTR_SLAVE = 0U,
        TSPI_MSTR_MASTER = 1U
    } TSPI_Mstr;
#define IS_TSPI_MSTR(param)             ((param) <= TSPI_MSTR_MASTER)

    typedef enum {
        TSPI_TRMODE_TX = 1U,    /* Half duplex (Transmit) mode */
        TSPI_TRMODE_RX = 2U,    /* Half duplex (Receive) mode  */
        TSPI_TRMODE_TXRX = 3U   /* Full duplex mode */
    } TSPI_TransferMode;
#define IS_TSPI_TRANSFERMODE(param)     (((param)>= TSPI_TRMODE_TX) && \
                                         ((param) <= TSPI_TRMODE_TXRX))

    typedef enum {
        TSPI_CS0 = 0U,
        TSPI_CS1 = 1U
    } TSPI_CSx;                 /* Chip select pin selection */
#define IS_TSPI_CSX(param)              ((param) <= TSPI_CS1)

#define IS_TSPI_TRANSFERNUM(param)              ((param) >= 1U)

    typedef enum {
        TSPI_TXOUTVALUE_HIZ = 0U,       /* Keep Hi-z */
        TSPI_TXOUTVALUE_LASTDATA = 1U,  /* Keep as last data in previous transfer */
        TSPI_TXOUTVALUE_KEEPLOW = 2U,   /* Fixed to low  */
        TSPI_TXOUTVALUE_KEEPHIGH = 3U   /* Fixed to high */
    } TSPI_TxOutValueInIdle;    /* Fixed output value function control when TSPIxTXD idles. */
#define IS_TSPI_TXOUTVALUEINIDLE(param)               ((param) <= TSPI_TXOUTVALUE_KEEPHIGH)
    typedef enum {
        TSPI_TXOUTVALUE_FIXEDTOLOW = 0U,        /* Fixed to low  */
        TSPI_TXOUTVALUE_FIXEDTOHIGH = 1U        /* Fixed to high */
    } TSPI_TxOutValueInAndorran;        /* Fixed output value function control when TSPIxTXD Andorrans. */
#define IS_TSPI_TXOUTVALUEINANDORRAN(param)     ((param) <= TSPI_TXOUTVALUE_FIXEDTOHIGH)

    typedef enum {
        TSPI_PHIT0_DIVIDE_2 = 0U,       /* phiT0/2 */
        TSPI_PHIT0_DIVIDE_4 = 1U,       /* phiT0/4 , the maximum clock */
        TSPI_PHIT0_DIVIDE_8 = 2U,       /* phiT0/8 */
        TSPI_PHIT0_DIVIDE_16 = 3U,      /* phiT0/16 */
        TSPI_PHIT0_DIVIDE_32 = 4U,      /* phiT0/32 */
        TSPI_PHIT0_DIVIDE_64 = 5U,      /* phiT0/64 */
        TSPI_PHIT0_DIVIDE_128 = 6U,     /* phiT0/128 */
        TSPI_PHIT0_DIVIDE_256 = 7U,     /* phiT0/256 */
        TSPI_PHIT0_DIVIDE_512 = 8U,     /* phiT0/512 */
        TSPI_PHIT0_DIVIDE_1024 = 9U     /* phiT0/1024 */
    } TSPI_BaudClock;
#define IS_TSPI_BAUDCLOCK(param)              ((param) <= TSPI_PHIT0_DIVIDE_1024)
#define IS_TSPI_DIVIDER(param)                (((param) >= 1U) && ((param) <= 16U))



#define TSPI_LSB_FIRST	    (uint8_t)0x00
#define TSPI_MSB_FIRST	    (uint8_t)0x01

#define TSPI_CLKPOL_LOW	    (uint8_t)0x00
#define TSPI_CLKPOL_HIGH	(uint8_t)0x01

#define TSPI_CS_POLARITY_NEGATIVE	(uint8_t)0x00
#define TSPI_CS_POLARITY_POSITIVE	(uint8_t)0x01

#define TSPI_PARITY_EVEN	(uint8_t)0x00
#define TSPI_PARITY_ODD	    (uint8_t)0x01

    typedef struct {
        uint8_t DataDirection;  /* LSB or MSB first */
        uint8_t FrameLength;    /* 8 to 32 bit */
        uint8_t CycleBetweenFrames;     /* 0 to 15 (Interval cycle between frames in burst mode) */
        uint8_t ClkPolority;    /* Level of serial clock during idle time */
        uint8_t ShortestIdleCycle;      /* 1 to 15 */
        uint8_t CS1Polarity;    /* Negative logic or Positive logic for pin CS1 */
        uint8_t CS0Polarity;    /* Negative logic or Positive logic for pin CS0 */
        uint8_t DelayCycleNum_CS_SCLK;  /* 1 to 16 (Delay cycle from asserting TSPIxCS to output serial clock) */
        uint8_t DelayCycleNum_Negate_CS;        /* 1 to 16 ( Delay cycle of negating TSPIxCS ) */
        uint8_t LastBit_Hold_Time;      /* 0 to 6 (bit holding time of TSPIxTXD pin in SIO slave mode) */
        FunctionalState ParityCheck;    /* ENABLE or DISABLE */
        uint8_t Parity;         /* TSPI_PARITY_ODD or TSPI_PARITY_ EVEN */
    } TSPI_InitTypeDef;

#define IS_TSPI_DATADIRECTION(param)        ((param) <= TSPI_MSB_FIRST)
#define IS_TSPI_FRAMELENGTH(param)          (((param)>= 8U) && ((param) <= 32U))
#define IS_TSPI_CYCLEBETWEENFRAMES(param)   ((param) <= 15U)
#define IS_TSPI_CLKPOLORITY(param)          ((param) <= TSPI_CLKPOL_HIGH)
#define IS_TSPI_SHORTESTIDLECYCLE(param)    (((param)>= 1U) && ((param) <= 15U))
#define IS_TSPI_CSPOLARITY(param)           ((param) <= TSPI_CS_POLARITY_POSITIVE)
#define IS_TSPI_DELAYCYCLENUM(param)        (((param)>= 1U) && ((param) <= 16U))
#define IS_TSPI_PARITYTYPE(param)           ((param) <= TSPI_PARITY_ODD)
#define IS_TSPI_HOLDTIME(param)             ((param) <= 6U)


/**
  * @brief Union to store TSPI status and store TSPI result
  */
    typedef union {
        uint32_t All;
        struct {
            uint32_t RxFIFOFillLevel:4; /* bit0--3 , Receive FIFO fill level */
            uint32_t RxFIFOFull:1;      /* bit4: '1': Receive FIFO is full  */
            uint32_t RxFIFO_INT:1;      /* bit5: '1': Receive FIFO interrupt generated */
            uint32_t RxCompleted:1;     /* bit6: '1': Reception completed */
            uint32_t RxRUN:1;   /* bit7: '1': Receive shift is operating */
            uint32_t Reserved1:8;
            uint32_t TxFIFOFillLevel:4; /* bit19--16 , Transmit FIFO fill level */
            uint32_t TxFIFOEmpty:1;     /* bit20: '1': Transmit FIFO is empty  */
            uint32_t TxFIFO_INT:1;      /* bit21: '1': Transmit FIFO interrupt generated */
            uint32_t TxCompleted:1;     /* bit22: '1': Transmission completed */
            uint32_t TxRUN:1;   /* bit23: '1': Transmit shift is operating */
            uint32_t Reserved2:7;
            uint32_t TSPI_RegUsing:1;   /* bit31: '1': TSPI registers must not be modified. */
        } Bit;
    } TSPI_StatusFlag;

    typedef union {
        uint32_t All;
        struct {
            uint32_t Reserved1:9;
            uint32_t ParityErr:1;       /* bit9 , Parity error flag */
            uint32_t OverrunErr:1;      /* bit10 , Overrun error flag */
            uint32_t UnderrunErr:1;     /* bit11 , Underrun error flag */
            uint32_t Reserved2:20;
        } Bit;
    } TSPI_Err;


/** @} */
/* End of group TSPI_Exported_Types */

/** @defgroup TSPI_Exporte                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       <input type="button"                                                                                                                  >d_FunctionPrototypes
  * @{
  */
    void TSPI_SWReset(TSB_TSPI_TypeDef * TSPIx);
    void TSPI_Enable(TSB_TSPI_TypeDef * TSPIx);
    void TSPI_Disable(TSB_TSPI_TypeDef * TSPIx);
    void TSPI_SetTxRxCtrl(TSB_TSPI_TypeDef * TSPIx, FunctionalState NewState);
    FunctionalState TSPI_GetTxRxCtrl(TSB_TSPI_TypeDef * TSPIx);
    void TSPI_SelectMode(TSB_TSPI_TypeDef * TSPIx, TSPI_Mode Mode);
    void TSPI_SelectMstr(TSB_TSPI_TypeDef * TSPIx, TSPI_Mstr Mstr);
    void TSPI_SelectTransferMode(TSB_TSPI_TypeDef * TSPIx, TSPI_TransferMode TrMode);
    void TSPI_SetCS(TSB_TSPI_TypeDef * TSPIx, TSPI_CSx CSx);
    void TSPI_SetTransferNum(TSB_TSPI_TypeDef * TSPIx, uint8_t Num);
    void TSPI_SetOutValueInIdle(TSB_TSPI_TypeDef * TSPIx, TSPI_TxOutValueInIdle OutputValue);
    void TSPI_SetOutValueInAndorran(TSB_TSPI_TypeDef * TSPIx,
                                    TSPI_TxOutValueInAndorran OutputValue);
    void TSPI_SetFIFOLevelINT(TSB_TSPI_TypeDef * TSPIx, uint8_t TxRx, uint8_t Level);
    void TSPI_SetDMA(TSB_TSPI_TypeDef * TSPIx, uint8_t TxRx, FunctionalState NewState);
    void TSPI_SetINT(TSB_TSPI_TypeDef * TSPIx, uint32_t IntSrc, FunctionalState NewState);
    void TSPI_InitFIFO(TSB_TSPI_TypeDef * TSPIx, uint8_t TxRx);
    void TSPI_SetBaudRate(TSB_TSPI_TypeDef * TSPIx, TSPI_BaudClock Clk, uint8_t Divider);
    void TSPI_Init(TSB_TSPI_TypeDef * TSPIx, TSPI_InitTypeDef * Init);
    void TSPI_SetTxData(TSB_TSPI_TypeDef * TSPIx, uint32_t Dat);
    uint32_t TSPI_GetRxData(TSB_TSPI_TypeDef * TSPIx);
    FunctionalState TSPI_IsRegisterModifiable(TSB_TSPI_TypeDef * TSPIx);
    TSPI_StatusFlag TSPI_GetStatus(TSB_TSPI_TypeDef * TSPIx);
    TSPI_Err TSPI_GetErrorState(TSB_TSPI_TypeDef * TSPIx);
    void TSPI_ClrAllErrFlag(TSB_TSPI_TypeDef * TSPIx);



/** @} */
/* End of group TSPI_Exported_FunctionPrototypes */

/** @} */
/* End of group TSPI */

/** @} */
/* End of group TX00_Periph_Driver */

#ifdef __cplusplus
}
#endif
#endif                          /* __TMPM066_TSPI_H */
