/**
 *******************************************************************************
 * @file    tmpm066_i2cs.c
 * @brief   This file provides API functions for I2CS driver.
 * @version V2.0.2.1
 * @date    2015/10/21
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm066_i2cs.h"

#if defined(__TMPM066_I2CS_H)

/** @addtogroup TX00_Periph_Driver
  * @{
  */

/** @defgroup I2CS 
  * @brief I2CS driver modules
  * @{
  */

/** @defgroup I2CS_Private_Defines
  * @{
  */

#define I2CWUPCR_ACK_SET                 ((uint8_t)0xDF)
#define I2CWUPCR_ACK_CLEAR               ((uint8_t)0x20)
#define I2CWUPCR_I2RES_RELEASE           ((uint8_t)0xEF)
#define I2CWUPCR_I2RES_DONE              ((uint8_t)0x10)
#define I2CWUPCR_INTEND_RELEASE          ((uint8_t)0x01)
#define I2CWUPCR_INTEND_CLEAR            ((uint8_t)0xFE)
#define I2CWUPCR_DATA_MASK               ((uint8_t)0xFB)
#define I2CS_SA_MASK                     ((uint8_t)0xFE)
#define I2CWUPSL_DATA_MASK               ((uint8_t)0x06)
#define I2CWUPCR3_SA2EN_USE              ((uint8_t)0x01)
#define I2CWUPCR3_SA2EN_NOUSE            ((uint8_t)0xFE)

/** @} */
/* End of group I2CS_Private_Defines */

/** @defgroup I2CS_Exported_Functions
  * @{
  */

/**
  * @brief  Enable or disable the generation of ACK clock. 
  * @param  NewState: New state of ACK clock.
  *   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void I2CS_SetACK(FunctionalState NewState)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    tmp = TSB_I2CS->WUPCR1;
    if (NewState == ENABLE) {
        /* Set I2CWUPCR1<ACK> to enable generation of ACK clock */
        tmp &= I2CWUPCR_ACK_SET;
    } else {
        /* Clear I2CWUPCR<ACK> to disable generation of ACK clock */
        tmp |= I2CWUPCR_ACK_CLEAR;        
    }
    TSB_I2CS->WUPCR1 = tmp;

}

/**
  * @brief  Reset the state of the specified I2CS channel.
  * @param  NewState: New state of I2CBUS state.
  *   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void I2CS_Reset(FunctionalState NewState)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));
        
    tmp = TSB_I2CS->WUPCR1;
    if (NewState == ENABLE) {
        /* Set I2CWUPCR1<I2RES> to reset done */
        tmp |= I2CWUPCR_I2RES_DONE;
    } else {
        /* Clear I2CWUPCR<I2RES> to reset release */
        tmp &= I2CWUPCR_I2RES_RELEASE;        
    }
    TSB_I2CS->WUPCR1 = tmp;    
}


/**
  * @brief  Release I2CS interrupt.
  * @param  NewState: New state of I2C interrupt release.
  *   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void I2CS_INTRelease(FunctionalState NewState)
{
    uint8_t tmp = 0U;
        
    tmp = TSB_I2CS->WUPCR1;
    if (NewState == ENABLE) {  
        /* Set I2CWUPCR1<INTEND> to release interrupt */    	  
        tmp |= I2CWUPCR_INTEND_RELEASE;
    } else {
        /* Clear I2CWUPCR1<INTEND> */
        tmp &= I2CWUPCR_INTEND_CLEAR;
    }                
    TSB_I2CS->WUPCR1 = tmp;    
}

/**
  * @brief  Get the I2CS state in I2CS mode
  * @retval The state of the I2CS in I2CS bus.
  */
I2CS_State I2CS_GetState(void)
{
    I2CS_State state;
    state.All = 0U;

    state.All = TSB_I2CS->WUPCR1;
    state.All &= I2CWUPCR_DATA_MASK;
    return state;
}


/**
  * @brief  Enable or disable using 2nd slave address of the I2CS channel.
  * @param  NewState: New state of free data mode.
  *   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void I2CS_Set2ndSlaveAddState(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        /* Set I2CWUPCR3<SA2EN> to using 2nd slave address */
        TSB_I2CS->WUPCR3 |= I2CWUPCR3_SA2EN_USE;
    } else {
        /* Clear I2CWUPCR3<SA2EN> to not using 2nd slave address */
        TSB_I2CS->WUPCR3 &= I2CWUPCR3_SA2EN_NOUSE;
    }
}


/**
  * @brief  Get the I2CS match slave address state in I2CS mode
  * @retval The state of the I2CS match slave address in I2CS bus.
  */
I2CS_MatchAdd I2CS_GetMatchAddState(void)
{
    I2CS_MatchAdd state;
    state.All = 0U;

    state.All = TSB_I2CS->WUPSL;
    state.All &= I2CWUPSL_DATA_MASK;
    return state;
}

/**
  * @brief  Set 1st slave address.
  * @param  Data: 1st slave address, max 0x7F.
  * @retval None
  */
void I2CS_Set1stSlaveAdd(uint8_t Data)
{
    /* Check the parameters */
    assert_param(IS_I2CS_DATA(Data));

    /* Write data into I2CWUPCR2 */
    TSB_I2CS->WUPCR2 |= (Data << 1);
}
/**
  * @brief  Get 1st slave address.
  * @retval 1st slave address.
  */
uint8_t I2CS_Get1stSlaveAdd(void)
{
    uint32_t retval = 0U;

    /* Save 1st slave address */
    retval = TSB_I2CS->WUPCR2;
    retval &= I2CS_SA_MASK;
    retval = retval >> 1U;
    return retval;
}

/**
  * @brief  Set 2nd slave address.
  * @param  Data: 2nd slave address, max 0x7F.
  * @retval None
  */
void I2CS_Set2ndSlaveAdd(uint8_t Data)
{
    /* Check the parameters */
    assert_param(IS_I2CS_DATA(Data));

    /* Write data into I2CWUPCR3 */
    TSB_I2CS->WUPCR3 |= (Data << 1);
}

/**
  * @brief  Get 2nd slave address.
  * @retval 2nd slave address.
  */
uint8_t I2CS_Get2ndSlaveAdd(void)
{
    uint32_t retval = 0U;

    /* Save 2nd slave address */
    retval = TSB_I2CS->WUPCR3;
    retval &= I2CS_SA_MASK;
    retval = retval >> 1U;
    return retval;
}

/** @} */
/* End of group I2CS_Exported_Functions */

/** @} */
/* End of group I2CS */

/** @} */
/* End of group TX00_Periph_Driver */

#endif                          /*   defined(__TMPM066_I2CS_H) */
