/**
 *******************************************************************************
 * @file    tmpm066_tmrd.c
 * @brief   This file provides API functions for TMRD driver.
 * @version V2.0.2.2
 * @date    2015/11/16
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 *
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm066_tmrd.h"

#if defined(__TMPM066_TMRD_H)
/** @addtogroup TX03_Periph_Driver
  * @{
  */

/** @defgroup TMRD
  * @brief TMRD driver modules
  * @{
  */

/** @defgroup TMRD_Private_Defines
  * @{
  */

#define EN_TDEN1_SET          (0x00000080UL)
#define EN_TDEN1_CLEAR        (0xFFFFFF7FUL)
#define EN_TDEN0_SET          (0x00000040UL)
#define EN_TDEN0_CLEAR        (0xFFFFFFBFUL)
#define EN_TDHALT_SET         (0x00000020UL)
#define EN_TDHALT_CLEAR       (0xFFFFFFDFUL)
#define CONF_TDI2TD1_SET      (0x00000080UL)
#define CONF_TDI2TD1_CLEAR    (0xFFFFFF7FUL)
#define CONF_TDI2TD0_SET      (0x00000040UL)
#define CONF_TDI2TD0_CLEAR    (0xFFFFFFBFUL)
#define CONF_TMRDMOD_MASK     (0xFFFFFFF8UL)
#define MOD_TDIV1_SET         (0x00000080UL)
#define MOD_TDIV1_CLEAR       (0xFFFFFF7FUL)
#define MOD_TDIV0_SET         (0x00000040UL)
#define MOD_TDIV0_CLEAR       (0xFFFFFFBFUL)
#define MOD_TDCLK_MASK        (0xFFFFFFF0UL)
#define MOD_TDCLE_SET         (0x00000010UL)
#define MOD_TDCLE_CLEAR       (0xFFFFFFEFUL)
#define CR_TDISO_MASK         (0xFFFFFFFCUL)
#define CR_TDMDCY_0_MASK      (0xFFFFFF1FUL)
#define CR_TDMDCY_1_MASK      (0xFFFFF1FFUL)
#define CR_TDMDPT_0_MASK      (0xFFFFFFEFUL)
#define CR_TDMDPT_1_MASK      (0xFFFFFEFFUL)
#define CR_TDRDE_SET          (0x00000004UL)
#define CR_TDRDE_CLEAR        (0xFFFFFFFBUL)
#define RG24_TIMING_MASK      (0xFFF0000FUL)
#define RG24_TIMING_TDMD_MASK (0xFFF00000UL)
#define RG24_DIR_SET          (0x80000000UL)
#define BCR_PHSCHG_SET        (0x00000010UL)
#define BCR_PHSCHG_CLEAR      (0xFFFFFFEFUL)
#define HSWB_HSWMD_CLEAR      (0xFFFFFFFCUL)
#define HSWB_OUTV_SET         (0x00000004UL)
#define HSWB_OUTV_CLEAR       (0xFFFFFFFBUL)
#define DMA_DMAEN_SET         (0x00000001UL)
#define DMA_DMAEN_CLEAR       (0xFFFFFFFEUL)
#define RUN_TDRUN_SET         (0x00000001UL)
#define RUN_TDRUN_CLEAR       (0xFFFFFFFEUL)

/** @} */
/* End of group TMRD_Private_Defines */

/** @defgroup TMRD_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group TMRD_Private_FunctionPrototypes */

/** @defgroup TMRD_Private_Functions
  * @{
  */

/** @} */
/* End of group TMRD_Private_Functions */

/** @defgroup TMRD_Exported_Functions
  * @{
  */

/**
  * @brief  Enable clock signal input to TMRD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @retval None
  */
void TMRD_Enable(TMRD_UNIT_Channel CHx)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));

    /* Set TDxEN<TDEN0, TDEN1> */
    TSB_TD->EN |= (CHx == TMRD_UNIT_CH_0) ? (EN_TDEN0_SET) : (EN_TDEN1_SET);
}

/**
  * @brief  Disable clock signal input to TMRD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @retval None
  */
void TMRD_Disable(TMRD_UNIT_Channel CHx)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));

    /* Clear TDxEN<TDEN0, TDEN1> */
    TSB_TD->EN &= (CHx == TMRD_UNIT_CH_0) ? (EN_TDEN0_CLEAR) : (EN_TDEN1_CLEAR);
}

/**
  * @brief  Set TMRD operation if a HALT instruction is 
  *   executed during debugging.
  * @param  RunState: TMRD operation status setting.
  *   This parameter can be one of the following values:
  *   TMRD_RUN, TMRD_STOP.
  * @retval None
  */
void TMRD_SetRunStateInHalt(uint8_t RunState)
{
    /* Check the parameters */

    assert_param(IS_TMRD_RUN_STATE(RunState));

    /* Set TDxEN<TDHALT> */
    TSB_TD->EN = (RunState == TMRD_RUN) ? (TSB_TD->EN | EN_TDHALT_SET) : (TSB_TD->EN & EN_TDHALT_CLEAR);
}

/**
  * @brief  Set TMRD operation during the IDLE mode. 
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  RunState: TMRD operation status setting.
  *   This parameter can be one of the following values:
  *   TMRD_RUN, TMRD_STOP.
  * @retval None
  */
void TMRD_SetRunStateInIdle(TMRD_UNIT_Channel CHx, uint8_t RunState)
{
    /* Check the parameters */

    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_RUN_STATE(RunState));

    /* Set TDxCONF<TDI2TD0, TDI2TD1> */
    if (RunState == TMRD_RUN) {
        TSB_TD->CONF |= (CHx == TMRD_UNIT_CH_0) ? (CONF_TDI2TD0_SET) : (CONF_TDI2TD1_SET);
    } else {
        TSB_TD->CONF &= (CHx == TMRD_UNIT_CH_0) ? (CONF_TDI2TD0_CLEAR) : (CONF_TDI2TD1_CLEAR);
    }
}

/**
  * @brief  Set the operation mode for TMRD. 
  * @param  Mode: Operation mode of TMRD.
  *   This parameter can be one of the following values:
  *   TMRD_MODE_BOTH_TMR, TMRD_MODE_0TMR_1PPG, 
  *   TMRD_MODE_0PPG_1TMR, TMRD_MODE_BOTH_PPG, 
  *   TMRD_MODE_INTERLOCK_TMR, TMRD_MODE_INTERLOCK_PPG_2CH,
  *   TMRD_MODE_INTERLOCK_PPG_3CH.
  * @retval None
  */
void TMRD_SetMode(uint8_t Mode)
{
    uint32_t tmp = 0U;
    /* Check the parameters */

    assert_param(IS_TMRD_MODE(Mode));

    /* Set TDxCONF<TMRDMOD> */
    tmp = TSB_TD->CONF & CONF_TMRDMOD_MASK;
    tmp |= Mode;
    TSB_TD->CONF = tmp;
}

/**
  * @brief  Set the clock prescaler of TMRD. 
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  ClkDiv: Clock prescaler factor.
  *   This parameter can be one of the following values:
  *   TMRD_CLK_DIV_1, TMRD_CLK_DIV_2, TMRD_CLK_DIV_4, 
  *   TMRD_CLK_DIV_8, TMRD_CLK_DIV_16.
  * @retval None
  */
void TMRD_SetClkDivision(TMRD_UNIT_Channel CHx, uint8_t ClkDiv)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_CLK_DIV(ClkDiv));

    if (CHx == TMRD_UNIT_CH_0) {
        /* Set TDnxMOD<TDCLK> */
        tmp = TSB_TD0->MOD & MOD_TDCLK_MASK;
        tmp |= ClkDiv;
        TSB_TD0->MOD = tmp;
    } else {
        /* Set TDnxMOD<TDCLK> */
        tmp = TSB_TD1->MOD & MOD_TDCLK_MASK;
        tmp |= ClkDiv;
        TSB_TD1->MOD = tmp;
    }
}

/**
  * @brief  Select the timer upcounter operation when match the cycle. 
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  UpCntCtrl: Up counter clear operation mode.
  *   This parameter can be one of the following values:
  *   TMRD_FREE_RUN, TMRD_AUTO_CLEAR. 
  * @retval None
  */
void TMRD_SetUpCntCtrl(TMRD_UNIT_Channel CHx, uint8_t UpCntCtrl)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_UC_CTRL(UpCntCtrl));

    /* Set TDnxMOD<TDCLE> */
    if (CHx == TMRD_UNIT_CH_0) {
        TSB_TD0->MOD = (UpCntCtrl == TMRD_AUTO_CLEAR) ? (TSB_TD0->MOD | MOD_TDCLE_SET) :
            (TSB_TD0->MOD & MOD_TDCLE_CLEAR);
    } else {
        TSB_TD1->MOD = (UpCntCtrl == TMRD_AUTO_CLEAR) ? (TSB_TD1->MOD | MOD_TDCLE_SET) :
            (TSB_TD1->MOD & MOD_TDCLE_CLEAR);
    }
}

/**
  * @brief  Set the initial leading/trailing edge of PPG channels.
  * @param  PPGChannel: PPG output channel.
  *   This parameter can be one of the following values:
  *   TMRD_PPG_CHANNEL_00, TMRD_PPG_CHANNEL_01, 
  *   TMRD_PPG_CHANNEL_10, TMRD_PPG_CHANNEL_11.
  * @param  WaveEdge: Wave edge of signal.
  *   This parameter can be one of the following values:
  *   TMRD_WAVE_EDGE_RISING, TMRD_WAVE_EDGE_FALLING. 
  * @retval None
  */
void TMRD_SetPPGInitLeadingEdge(uint8_t PPGChannel, uint8_t WaveEdge)
{
    /* Check the parameters */
    assert_param(IS_TMRD_PPG_CHANNEL(PPGChannel));
    assert_param(IS_TMRD_WAVE_EDGE(WaveEdge));

    /* Set TDnxMOD<TDIV0, TDIV1> */
    switch (PPGChannel) {
    case TMRD_PPG_CHANNEL_00:
        TSB_TD0->MOD = (WaveEdge == TMRD_WAVE_EDGE_FALLING) ? (TSB_TD0->MOD | MOD_TDIV0_SET) :
            (TSB_TD0->MOD & MOD_TDIV0_CLEAR);
        break;
    case TMRD_PPG_CHANNEL_01:
        TSB_TD0->MOD = (WaveEdge == TMRD_WAVE_EDGE_FALLING) ? (TSB_TD0->MOD | MOD_TDIV1_SET) :
            (TSB_TD0->MOD & MOD_TDIV1_CLEAR);
        break;
    case TMRD_PPG_CHANNEL_10:
        TSB_TD1->MOD = (WaveEdge == TMRD_WAVE_EDGE_FALLING) ? (TSB_TD1->MOD | MOD_TDIV0_SET) :
            (TSB_TD1->MOD & MOD_TDIV0_CLEAR);
        break;
    case TMRD_PPG_CHANNEL_11:
        TSB_TD1->MOD = (WaveEdge == TMRD_WAVE_EDGE_FALLING) ? (TSB_TD1->MOD | MOD_TDIV1_SET) :
            (TSB_TD1->MOD & MOD_TDIV1_CLEAR);
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set a write path to the compare register. 

  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  WritePath: The path to update compare register.
  *   This parameter can be one of the following values:
  *   TMRD_CMP_WRITE_DIRECT, TMRD_CMP_WRITE_INDIRECT. 
  * @retval None
  */
void TMRD_SetCMPRegWritePath(TMRD_UNIT_Channel CHx, uint8_t WritePath)
{

    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_CMP_WRITE_PATH(WritePath));

    /* Set TDnxCR<TDRDE> */
    if (CHx == TMRD_UNIT_CH_0) {
        TSB_TD0->CR = (WritePath == TMRD_CMP_WRITE_DIRECT) ? (TSB_TD0->CR & CR_TDRDE_CLEAR) :
            (TSB_TD0->CR | CR_TDRDE_SET);
    } else {
        TSB_TD1->CR = (WritePath == TMRD_CMP_WRITE_DIRECT) ? (TSB_TD1->CR & CR_TDRDE_CLEAR) :
            (TSB_TD1->CR | CR_TDRDE_SET);
    }
}

/**
  * @brief  Set the interrupt source of compare interrupt 0. 

  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  INTSrc: Select the interrupt factor.
  *   This parameter can be one of the following values:
  *   TMRD_INT_NONE, TMRD_INT_MATCH_CYCLE, 
  *   TMRD_INT_MATCH_PHASE (only TMRD0 has), TMRD_INT_UC_OVERFLOW. 
  * @retval None
  */
void TMRD_SetCMP0INTSrc(TMRD_UNIT_Channel CHx, uint8_t INTSrc)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    if (CHx == TMRD_UNIT_CH_0) {
        assert_param(IS_TMRD_TD0_INT_SRC(INTSrc));
    } else {
        assert_param(IS_TMRD_TD1_INT_SRC(INTSrc));
    }

    /* Set TDnxCR<TDISO> */
    if (CHx == TMRD_UNIT_CH_0) {
        tmp = TSB_TD0->CR & CR_TDISO_MASK;
        tmp |= INTSrc;
        TSB_TD0->CR = tmp;
    } else {
        tmp = TSB_TD1->CR & CR_TDISO_MASK;
        tmp |= INTSrc;
        TSB_TD1->CR = tmp;
    }
}

/**
  * @brief  Set the 1bit modulation cycle. 

  * @param  PPGChannel: PPG output channel.
  *   This parameter can be one of the following values:
  *   TMRD_PPG_CHANNEL_00, TMRD_PPG_CHANNEL_01, 
  *   TMRD_PPG_CHANNEL_10, TMRD_PPG_CHANNEL_11.
  * @param  BitModCycle: Select the 1bit modulation cycle.
  *   This parameter can be one of the following values:
  *   TMRD_1BITMOD_CYCLE_NONE, TMRD_1BITMOD_CYCLE_2TIMES, TMRD_1BITMOD_CYCLE_4TIMES, 
  *   TMRD_1BITMOD_CYCLE_8TIMES or TMRD_1BITMOD_CYCLE_16TIMES. 
  * @retval None
  */
void TMRD_SetBitModulationCycle(uint8_t PPGChannel, uint8_t BitModCycle)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRD_PPG_CHANNEL(PPGChannel));
    assert_param(IS_TMRD_1BITMOD_CYCLE(BitModCycle));

    /* Set TDnxCR */
    switch (PPGChannel) {
    case TMRD_PPG_CHANNEL_00:
        tmp = TSB_TD0->CR & CR_TDMDCY_0_MASK;
        tmp |= ((uint32_t) BitModCycle << 5U);
        TSB_TD0->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_01:
        tmp = TSB_TD0->CR & CR_TDMDCY_1_MASK;
        tmp |= ((uint32_t) BitModCycle << 9U);
        TSB_TD0->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_10:
        tmp = TSB_TD1->CR & CR_TDMDCY_0_MASK;
        tmp |= ((uint32_t) BitModCycle << 5U);
        TSB_TD1->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_11:
        tmp = TSB_TD1->CR & CR_TDMDCY_1_MASK;
        tmp |= ((uint32_t) BitModCycle << 9U);
        TSB_TD1->CR = tmp;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set the 1bit modulation update timing. 

  * @param  PPGChannel: PPG output channel.
  *   This parameter can be one of the following values:
  *   TMRD_PPG_CHANNEL_00, TMRD_PPG_CHANNEL_01, 
  *   TMRD_PPG_CHANNEL_10, TMRD_PPG_CHANNEL_11.
  * @param  NewState: The state of the 1bit modulation .
  *   This parameter can be one of the following values:
  *   DISABLE(1bit modulation), ENABLE(CPxx).
  * @retval None
  */
void TMRD_SetBitModUpdateTiming(uint8_t PPGChannel, FunctionalState NewState)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRD_PPG_CHANNEL(PPGChannel));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    /* Set TDnxCR */
    switch (PPGChannel) {
    case TMRD_PPG_CHANNEL_00:
        tmp = TSB_TD0->CR & CR_TDMDPT_0_MASK;
        tmp |= ((uint32_t) NewState << 4U);
        TSB_TD0->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_01:
        tmp = TSB_TD0->CR & CR_TDMDPT_1_MASK;
        tmp |= ((uint32_t) NewState << 8U);
        TSB_TD0->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_10:
        tmp = TSB_TD1->CR & CR_TDMDPT_0_MASK;
        tmp |= ((uint32_t) NewState << 4U);
        TSB_TD1->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_11:
        tmp = TSB_TD1->CR & CR_TDMDPT_1_MASK;
        tmp |= ((uint32_t) NewState << 8U);
        TSB_TD1->CR = tmp;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set the count operation of TMRD. 

  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  RunState: The counter operation of TMRD.
  *   This parameter can be one of the following values:
  *   TMRD_RUN, TMRD_STOP. 
  * @retval None
  */
void TMRD_SetRunState(TMRD_UNIT_Channel CHx, uint8_t RunState)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_RUN_STATE(RunState));

    /* Set TDnxRUN<TDRUN> */
    if (CHx == TMRD_UNIT_CH_0) {
        TSB_TD0->RUN = (RunState == TMRD_RUN) ? (TSB_TD0->RUN | RUN_TDRUN_SET) :
            (TSB_TD0->RUN & RUN_TDRUN_CLEAR);
    } else {
        TSB_TD1->RUN = (RunState == TMRD_RUN) ? (TSB_TD1->RUN | RUN_TDRUN_SET) :
            (TSB_TD1->RUN & RUN_TDRUN_CLEAR);
    }
}

/**
  * @brief  Set the phase relation of phase 1 to phase 0. 

  * @param  PhaseRelation: The phase relation of phase 1 to the phase 0.
  *   This parameter can be one of the following values:
  *   TMRD_PHASE_DELAY_OR_SAME, TMRD_PHASE_FAST_OR_SAME.
  * @retval None
  */
void TMRD_SetPhaseRelation(uint8_t PhaseRelation)
{
    uint32_t tmp = 0U;
    
    /* Check the parameters */
    assert_param(IS_TMRD_PHASE_RELATION(PhaseRelation));

    /* Set TDBCR<PHSCHG> */
    tmp = TSB_TD->BCR;
    if (PhaseRelation == TMRD_PHASE_FAST_OR_SAME) {
        tmp |= BCR_PHSCHG_SET;
    } else {
        tmp &= BCR_PHSCHG_CLEAR;
    }
    TSB_TD->BCR = tmp;
}

/**
  * @brief  Enable to update the compare register. 

  * @param  UpdateCHx: Select shift channel.
  *   This parameter can be one of the following values:
  *   TMRD_UPDATE_CH_00, TMRD_UPDATE_CH_01, 
  *   TMRD_UPDATE_CH_10, TMRD_UPDATE_CH_11,
  *   or combination of the channels above.
  * @retval None
  */
void TMRD_EnableUpdateCMPReg(uint8_t UpdateCHx)
{
    /* Check the parameters */
    assert_param(IS_TMRD_UPDATE_CH(UpdateCHx));

    TSB_TD->BCR |= UpdateCHx;
}

/**
  * @brief  Enable or disable the DMA request of INTTDxnCMP0 signal. 

  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  NewState: The state of DMA request.
  *   This parameter can be one of the following values:
  *   ENABLE, DISABLE. 
  * @retval None
  */
void TMRD_SetDMAReq(TMRD_UNIT_Channel CHx, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    /* Set TDnxDMA<DMAEN> */
    if (CHx == TMRD_UNIT_CH_0) {
        TSB_TD0->DMA = (NewState == ENABLE) ? (TSB_TD0->DMA | DMA_DMAEN_SET) :
            (TSB_TD0->DMA & DMA_DMAEN_CLEAR);
    } else {
        TSB_TD1->DMA = (NewState == ENABLE) ? (TSB_TD1->DMA | DMA_DMAEN_SET) :
            (TSB_TD1->DMA & DMA_DMAEN_CLEAR);
    }
}

/**
  * @brief  Initialize TMRD timing parameters. 

  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  TimingStruct: The pointer of TMRD timing 
  *   parameters structure.
  * @retval None
  */
void TMRD_SetInitTiming(TMRD_UNIT_Channel CHx,
                        TMRD_TimingTypeDef * TimingStruct)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_POINTER_NOT_NULL(TimingStruct));

    /* Set TDnxRGm */
    if (CHx == TMRD_UNIT_CH_0) {
        TSB_TD0->RG0 = (uint32_t) TimingStruct->Cycle;
        TSB_TD0->RG1 = TimingStruct->LeadingTiming0;
        TSB_TD0->RG2 &= RG24_TIMING_TDMD_MASK;
        TSB_TD0->RG2 |= ((uint32_t) TimingStruct->TrailingTiming0 << 4U) |
            ((uint32_t) TimingStruct->BitModulationRate0 & 0x0FU);
        TSB_TD0->RG3 = TimingStruct->LeadingTiming1;
        TSB_TD0->RG4 = ((uint32_t) TimingStruct->TrailingTiming1 << 4U) |
            ((uint32_t) TimingStruct->BitModulationRate1 & 0x0FU);
        TSB_TD0->RG5 = ((uint32_t) TimingStruct->PhaseShiftTiming << 1U);
    } else {
        TSB_TD1->RG0 = (uint32_t) TimingStruct->Cycle;
        TSB_TD1->RG1 = TimingStruct->LeadingTiming0;
        TSB_TD1->RG2 &= RG24_TIMING_TDMD_MASK;
        TSB_TD1->RG2 |= ((uint32_t) TimingStruct->TrailingTiming0 << 4U) |
            ((uint32_t) TimingStruct->BitModulationRate0 & 0x0FU);
        TSB_TD1->RG3 = TimingStruct->LeadingTiming1;
        TSB_TD1->RG4 = ((uint32_t) TimingStruct->TrailingTiming1 << 4U) |
            ((uint32_t) TimingStruct->BitModulationRate1 & 0x0FU);
    }

}

/**
  * @brief  Change the specified TMRD timing value. 

  * @param  TimingType: Timing parameters type.
  *   This parameter can be one of the following values:
  *   TMRD_TIMING_TD0_CYCLE, TMRD_TIMING_00_LEADING, 
  *   TMRD_TIMING_00_TRAILING, TMRD_TIMING_01_LEADING, 
  *   TMRD_TIMING_01_TRAILING, TMRD_TIMING_PHASE_SHIFT, 
  *   TMRD_TIMING_TD1_CYCLE, TMRD_TIMING_10_LEADING, 
  *   TMRD_TIMING_10_TRAILING, TMRD_TIMING_11_LEADING, 
  *   TMRD_TIMING_11_TRAILING.
  * @param  Timing: 16bits timing value.
  * @retval None
  */
void TMRD_ChangeTiming(uint8_t TimingType, uint16_t Timing)
{
    /* Check the parameters */
    assert_param(IS_TMRD_TIMING_TYPE(TimingType));

    /* Set TDnxRGm */
    switch (TimingType) {
    case TMRD_TIMING_TD0_CYCLE:
        TSB_TD0->RG0 = (uint32_t) Timing;
        break;
    case TMRD_TIMING_00_LEADING:
        TSB_TD0->RG1 = Timing;
        break;
    case TMRD_TIMING_00_TRAILING:
        TSB_TD0->RG2 &= RG24_TIMING_MASK;
        TSB_TD0->RG2 |= ((uint32_t) Timing << 4U);
        break;
    case TMRD_TIMING_01_LEADING:
        TSB_TD0->RG3 = Timing;
        break;
    case TMRD_TIMING_01_TRAILING:
        TSB_TD0->RG4 &= RG24_TIMING_MASK;
        TSB_TD0->RG4 |= ((uint32_t) Timing << 4U);
        break;
    case TMRD_TIMING_PHASE_SHIFT:
        TSB_TD0->RG5 = ((uint32_t) Timing << 1U);
        break;
    case TMRD_TIMING_TD1_CYCLE:
        TSB_TD1->RG0 = (uint32_t) Timing;
        break;
    case TMRD_TIMING_10_LEADING:
        TSB_TD1->RG1 = Timing;
        break;
    case TMRD_TIMING_10_TRAILING:
        TSB_TD1->RG2 &= RG24_TIMING_MASK;
        TSB_TD1->RG2 |= ((uint32_t) Timing << 4U);
        break;
    case TMRD_TIMING_11_LEADING:
        TSB_TD1->RG3 = Timing;
        break;
    case TMRD_TIMING_11_TRAILING:
        TSB_TD1->RG4 &= RG24_TIMING_MASK;
        TSB_TD1->RG4 |= ((uint32_t) Timing << 4U);
        break;
    default:
        /* Do nothing */
        break;
    }

}

/**
  * @brief  Get the specified TMRD timing value. 

  * @param  TimingType: Timing parameters type.
  *   This parameter can be one of the following values:
  *   TMRD_TIMING_TD0_CYCLE, TMRD_TIMING_00_LEADING, 
  *   TMRD_TIMING_00_TRAILING, TMRD_TIMING_01_LEADING, 
  *   TMRD_TIMING_01_TRAILING, TMRD_TIMING_PHASE_SHIFT, 
  *   TMRD_TIMING_TD1_CYCLE, TMRD_TIMING_10_LEADING, 
  *   TMRD_TIMING_10_TRAILING, TMRD_TIMING_11_LEADING, 
  *   TMRD_TIMING_11_TRAILING.
  * @retval Specified TMRD compare register timing.
  */
uint16_t TMRD_GetTiming(uint8_t TimingType)
{
    uint16_t retval = 0U;

    /* Check the parameters */
    assert_param(IS_TMRD_TIMING_TYPE(TimingType));

    /* Get TDnxCPm */
    switch (TimingType) {
    case TMRD_TIMING_TD0_CYCLE:
        retval = (uint16_t) TSB_TD0->CP0;
        break;
    case TMRD_TIMING_00_LEADING:
        retval = (uint16_t) TSB_TD0->CP1;
        break;
    case TMRD_TIMING_00_TRAILING:
        retval = (uint16_t) (TSB_TD0->CP2 >> 4U);
        break;
    case TMRD_TIMING_01_LEADING:
        retval = (uint16_t) TSB_TD0->CP3;
        break;
    case TMRD_TIMING_01_TRAILING:
        retval = (uint16_t) (TSB_TD0->CP4 >> 4U);
        break;
    case TMRD_TIMING_PHASE_SHIFT:
        retval = (uint16_t) TSB_TD0->CP5;
        break;
    case TMRD_TIMING_TD1_CYCLE:
        retval = (uint16_t) TSB_TD1->CP0;
        break;
    case TMRD_TIMING_10_LEADING:
        retval = (uint16_t) TSB_TD1->CP1;
        break;
    case TMRD_TIMING_10_TRAILING:
        retval = (uint16_t) (TSB_TD1->CP2 >> 4U);
        break;
    case TMRD_TIMING_11_LEADING:
        retval = (uint16_t) TSB_TD1->CP3;
        break;
    case TMRD_TIMING_11_TRAILING:
        retval = (uint16_t) (TSB_TD1->CP4 >> 4U);
        break;
    default:
        /* Do nothing */
        break;
    }
    return retval;
}

/**
  * @brief  Set the H-SW Mode when TDOUTIN is Low. 
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  PHx: Select the TMRD phase of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_PH_0, TMRD_UNIT_PH_1.
  * @param  LMode: H-SW Mode when TDOUTIN is low.
  *   This parameter can be one of the following values:
  *   TMRD_LOUT_HSW_CCW_CW, TMRD_LOUT_HSW_BRAKE, TMRD_LOUT_HSW_STOP.
  * @retval None
  */
void TMRD_SetLOutHSWMode(TMRD_UNIT_Channel CHx, TMRD_UNIT_Phase PHx,
                         uint8_t LMode)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_ALL_PHASE(PHx));
    assert_param(IS_TMRD_LOUT_HSW_MODE(LMode));

    if (CHx == TMRD_UNIT_CH_0) {
        if (PHx == TMRD_UNIT_PH_0) {
            tmp = TSB_TD0->HSWB0;
            tmp &= HSWB_HSWMD_CLEAR;
            tmp |= (uint32_t) LMode + 1U;
            TSB_TD0->HSWB0 = tmp;
        } else {
            tmp = TSB_TD0->HSWB1;
            tmp &= HSWB_HSWMD_CLEAR;
            tmp |= (uint32_t) LMode + 1U;
            TSB_TD0->HSWB1 = tmp;
        }
    } else {
        if (PHx == TMRD_UNIT_PH_0) {
            tmp = TSB_TD1->HSWB0;
            tmp &= HSWB_HSWMD_CLEAR;
            tmp |= (uint32_t) LMode + 1U;
            TSB_TD1->HSWB0 = tmp;
        } else {
            tmp = TSB_TD1->HSWB1;
            tmp &= HSWB_HSWMD_CLEAR;
            tmp |= (uint32_t) LMode + 1U;
            TSB_TD1->HSWB1 = tmp;
        }
    }
}

/**
  * @brief  Get the H-SW Mode when TDOUTIN is Low. 

  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  PHx: Select the TMRD phase of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_PH_0, TMRD_UNIT_PH_1.
  * @retval The H-SW mode.
  *   The value returned can be one of the following values:
  *   TMRD_LOUT_HSW_CCW_CW, TMRD_LOUT_HSW_BRAKE, TMRD_LOUT_HSW_STOP.
  */
uint8_t TMRD_GetLOutHSWMode(TMRD_UNIT_Channel CHx, TMRD_UNIT_Phase PHx)
{
    uint32_t tmp = 0U;
    uint8_t mode = TMRD_LOUT_HSW_CCW_CW;

    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_ALL_PHASE(PHx));

    if (CHx == TMRD_UNIT_CH_0) {
        if (PHx == TMRD_UNIT_PH_0) {
            tmp = TSB_TD0->HSW0;
        } else {
            tmp = TSB_TD0->HSW1;
        }
    } else {
        if (PHx == TMRD_UNIT_PH_0) {
            tmp = TSB_TD1->HSW0;
        } else {
            tmp = TSB_TD1->HSW1;
        }
    }
    tmp &= (~HSWB_HSWMD_CLEAR);
    (tmp == 0U) ? (mode = TMRD_LOUT_HSW_CCW_CW) : (mode = (uint8_t) (tmp - 1U));
    return mode;
}

/**
  * @brief  Enable H-SW mode. 
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  PHx: Select the TMRD phase of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_PH_0, TMRD_UNIT_PH_1.
  * @retval None
  */
void TMRD_EnableHSWMode(TMRD_UNIT_Channel CHx, TMRD_UNIT_Phase PHx)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_ALL_PHASE(PHx));

    if (CHx == TMRD_UNIT_CH_0) {
        if (PHx == TMRD_UNIT_PH_0) {
            TSB_TD0->HSWB0 |= HSWB_OUTV_SET;
        } else {
            TSB_TD0->HSWB1 |= HSWB_OUTV_SET;
        }
    } else {
        if (PHx == TMRD_UNIT_PH_0) {
            TSB_TD1->HSWB0 |= HSWB_OUTV_SET;
        } else {
            TSB_TD1->HSWB1 |= HSWB_OUTV_SET;
        }
    }
}

/**
  * @brief  Disable H-SW mode. 

  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  PHx: Select the TMRD phase of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_PH_0, TMRD_UNIT_PH_1.
  * @retval None
  */
void TMRD_DisableHSWMode(TMRD_UNIT_Channel CHx, TMRD_UNIT_Phase PHx)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_ALL_PHASE(PHx));

    if (CHx == TMRD_UNIT_CH_0) {
        if (PHx == TMRD_UNIT_PH_0) {
            TSB_TD0->HSWB0 &= HSWB_OUTV_CLEAR;
        } else {
            TSB_TD0->HSWB1 &= HSWB_OUTV_CLEAR;
        }
    } else {
        if (PHx == TMRD_UNIT_PH_0) {
            TSB_TD1->HSWB0 &= HSWB_OUTV_CLEAR;
        } else {
            TSB_TD1->HSWB1 &= HSWB_OUTV_CLEAR;
        }
    }
}

/**
  * @brief  Get H-SW mode state. 
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  PHx: Select the TMRD phase of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_PH_0, TMRD_UNIT_PH_1.
  * @retval state of H-SW mode.
  *   The value returned can be one of the following values:
  *   ENABLE, DISABLE.
  */
FunctionalState TMRD_GetHSWModeState(TMRD_UNIT_Channel CHx,
                                     TMRD_UNIT_Phase PHx)
{
    FunctionalState state = DISABLE;

    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_ALL_PHASE(PHx));

    if (CHx == TMRD_UNIT_CH_0) {
        if (PHx == TMRD_UNIT_PH_0) {
            (TSB_TD0->HSW0 & HSWB_OUTV_SET) ? (state = ENABLE) : (state = DISABLE);
        } else {
            (TSB_TD0->HSW1 & HSWB_OUTV_SET) ? (state = ENABLE) : (state = DISABLE);
        }
    } else {
        if (PHx == TMRD_UNIT_PH_0) {
            (TSB_TD1->HSW0 & HSWB_OUTV_SET) ? (state = ENABLE) : (state = DISABLE);
        } else {
            (TSB_TD1->HSW1 & HSWB_OUTV_SET) ? (state = ENABLE) : (state = DISABLE);
        }
    }
    return state;
}

/**
  * @brief  Set the level of TDDIR. 
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  PHx: Select the TMRD phase of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_PH_0, TMRD_UNIT_PH_1.
  * @param  Level: Select the setting level.
  *   This parameter can be one of the following values:
  *   TMRD_LOW_LEVEL, TMRD_HIGH_LEVEL.  
  * @retval None
  */
void TMRD_SetTDDIRLevel(TMRD_UNIT_Channel CHx, TMRD_UNIT_Phase PHx,
                        uint8_t Level)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_LEVEL(Level));
    assert_param(IS_TMRD_ALL_PHASE(PHx));

    if (CHx == TMRD_UNIT_CH_0) {
        if (PHx == TMRD_UNIT_PH_0) {
            (Level == TMRD_LOW_LEVEL) ? (TSB_TD0->RG2 &= (~RG24_DIR_SET)) :
                (TSB_TD0->RG2 |= RG24_DIR_SET);
        } else {
            (Level == TMRD_LOW_LEVEL) ? (TSB_TD0->RG4 &= (~RG24_DIR_SET)) :
                (TSB_TD0->RG4 |= RG24_DIR_SET);
        }
    } else {
        if (PHx == TMRD_UNIT_PH_0) {
            (Level == TMRD_LOW_LEVEL) ? (TSB_TD1->RG2 &= (~RG24_DIR_SET)) :
                (TSB_TD1->RG2 |= RG24_DIR_SET);
        } else {
            (Level == TMRD_LOW_LEVEL) ? (TSB_TD1->RG4 &= (~RG24_DIR_SET)) :
                (TSB_TD1->RG4 |= RG24_DIR_SET);
        }
    }
}

/**
  * @brief  Get the level of TDDIR. 
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  PHx: Select the TMRD phase of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_PH_0, TMRD_UNIT_PH_1.
  * @retval The level of TDDIR.
  *   The value returned can be one of the following values:
  *   TMRD_LOW_LEVEL, TMRD_HIGH_LEVEL.
  */
uint8_t TMRD_GetTDDIRLevel(TMRD_UNIT_Channel CHx, TMRD_UNIT_Phase PHx)
{
    uint8_t level = 0U;

    /* Check the parameters */
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_ALL_PHASE(PHx));

    if (CHx == TMRD_UNIT_CH_0) {
        if (PHx == TMRD_UNIT_PH_0) {
            (TSB_TD0->RG2 & RG24_DIR_SET) ? (level = TMRD_HIGH_LEVEL) : (level = TMRD_LOW_LEVEL);
        } else {
            (TSB_TD0->RG4 & RG24_DIR_SET) ? (level = TMRD_HIGH_LEVEL) : (level = TMRD_LOW_LEVEL);
        }
    } else {
        if (PHx == TMRD_UNIT_PH_0) {
            (TSB_TD1->RG2 & RG24_DIR_SET) ? (level = TMRD_HIGH_LEVEL) : (level = TMRD_LOW_LEVEL);
        } else {
            (TSB_TD1->RG4 & RG24_DIR_SET) ? (level = TMRD_HIGH_LEVEL) : (level = TMRD_LOW_LEVEL);
        }
    }
    return level;
}

/** @} */
/* End of group TMRD_Exported_Functions */

/** @} */
/* End of group TMRD */

/** @} */
/* End of group TX03_Periph_Driver */

#endif                          /* defined(__TMPM066_TMRD_H) */
