/**
 *******************************************************************************
 * @file    tmpm066_tspi.c
 * @brief   This file provides API functions for TSPI driver.
 * @version V2.0.2.1
 * @date    2015/09/08
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm066_tspi.h"

#if defined(__TMPM066_TSPI_H)

#define TSPI_RESET_CLEAR     ((uint32_t)0xFFFFFF3F)

#define CR1_TRXE_MASK        ((uint32_t)0xFFFFBFFF)
#define CR1_TSPIMS_MASK      ((uint32_t)0xFFFFDFFF)
#define CR1_MSTR_MASK        ((uint32_t)0xFFFFEFFF)
#define CR1_TMMD_MASK        ((uint32_t)0xFFFFF3FF)
#define CR1_CSSEL_MASK       ((uint32_t)0xFFFFFEFF)
#define CR1_FC_MASK          ((uint32_t)0xFFFFFF00)

#define CR2_TIDLE_MASK       ((uint32_t)0xFF3EFFFF)
#define CR2_TXDEMP_MASK      ((uint32_t)0xFFDEFFFF)
#define CR2_TIL_MASK         ((uint32_t)0xFFFE0FFF)
#define CR2_RIL_MASK         ((uint32_t)0xFFFEF0FF)
#define CR2_INT_MASK         ((uint32_t)0xFFFEFF0B)
#define CR2_DMATE_MASK       ((uint32_t)0xFFFEFFFD)
#define CR2_DMARE_MASK       ((uint32_t)0xFFFEFFFE)
/** @addtogroup TX00_Periph_Driver
  * @{
  */

/** @defgroup TSPI 
  * @brief TSPI driver modules
  * @{
  */

/** @defgroup TSPI_Private_Defines
  * @{
  */


/** @} */
/* End of group TSPI_Private_Defines */

/** @defgroup TSPI_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group TSPI_Private_FunctionPrototypes */

/** @defgroup TSPI_Private_Functions
  * @{
  */

/** @} */
/* End of group TSPI_Private_Functions */

/** @defgroup TSPI_Exported_Functions
  * @{
  */

/**
  * @brief Software reset TSPI module.
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @retval None.
  */
void TSPI_SWReset(TSB_TSPI_TypeDef * TSPIx)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));

    /* write '10' then '01' to bit7-6 */
    tmp = TSPIx->CR0;
    tmp &= TSPI_RESET_CLEAR;
    tmp |= (uint32_t) (0x02U << 6U);
    TSPIx->CR0 = tmp;
    tmp &= TSPI_RESET_CLEAR;
    tmp |= (uint32_t) (0x01U << 6U);
    TSPIx->CR0 = tmp;
}

/**
  * @brief Enable TSPI module, it must be called at first.
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @retval None.
  */
void TSPI_Enable(TSB_TSPI_TypeDef * TSPIx)
{
    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));

    /* write '1' to bit 0, no need to care about bit7-6 */
    TSPIx->CR0 = 0x01U;
}


/**
  * @brief Disable TSPI module.
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @retval None.
  */
void TSPI_Disable(TSB_TSPI_TypeDef * TSPIx)
{
    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));

    /* write '0' to bit 0 */
    TSPIx->CR0 = 0x0U;
}

/**
  * @brief Enable/Disable TSPI Transfer.
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param NewState: Specify the state for transfer.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.  
  * @retval None.
  */
void TSPI_SetTxRxCtrl(TSB_TSPI_TypeDef * TSPIx, FunctionalState NewState)
{
    /* CR1 used: bit14, 13, 12, 11-10, 8, 7-0, clear bit14 here */
    uint32_t tmp = TSPIx->CR1 & (CR1_TRXE_MASK);

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    /* write '1' to bit 14 to enable, write '0' to disable */
    tmp |= (uint32_t) (NewState) << 14U;
    TSPIx->CR1 = tmp;
}

/**
  * @brief Get the TSPI transfer control status.
  * @param TSPIx:  Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @retval The transfer control status of specify TSPI unit.
  *   This value can be one of the following values:
  *   ENABLE or DISABLE.   
  */
FunctionalState TSPI_GetTxRxCtrl(TSB_TSPI_TypeDef * TSPIx)
{
    /* CR1 used: bit14, 13, 12, 11-10, 8, 7-0, only keep bit14 here */
    uint32_t tmp = TSPIx->CR1 & (0x01U << 14U);
    FunctionalState state = DISABLE;
    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));

    if (tmp >> 14U) {
        state = ENABLE;
    } else {
        /* Do nothing */
    }
    return state;
}


/**
  * @brief Select TSPI mode: as SIO or SPI.
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param Mode: Specify the mode for TSPI.
  *   This parameter can be one of the following values:
  *   TSPI_MODE_SPI or TSPI_MODE_SIO.  
  * @retval None.  
  */
void TSPI_SelectMode(TSB_TSPI_TypeDef * TSPIx, TSPI_Mode Mode)
{
    /* CR1 used: bit14, 13, 12, 11-10, 8, 7-0, clear bit13 here */
    uint32_t tmp = TSPIx->CR1 & (CR1_TSPIMS_MASK);

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_MODE(Mode));

    /* write '0' to bit 13 for SPI, write '1' for SIO */
    tmp |= (uint32_t) (Mode) << 13U;
    TSPIx->CR1 = tmp;
}

/**
  * @brief Select master/slave: as slave or master.
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param Mstr: Specify the master/slave.
  *   This parameter can be one of the following values:
  *   TSPI_MSTR_SLAVE or TSPI_MSTR_MASTER.  
  * @retval None.  
  */
void TSPI_SelectMstr(TSB_TSPI_TypeDef * TSPIx, TSPI_Mstr Mstr)
{
    /* CR1 used: bit14, 13, 12, 11-10, 8, 7-0, clear bit12 here */
    uint32_t tmp = TSPIx->CR1 & (CR1_MSTR_MASK);

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_MSTR(Mstr));

    /* write '0' to bit 12 for slave, write '1' for master */
    tmp |= (uint32_t) (Mstr) << 12U;
    TSPIx->CR1 = tmp;
}

/**
  * @brief Select TSPI transfer mode
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param TrMode: Specify the transfer mode for TSPI.
  *   This parameter can be one of the following values:
  *   TSPI_TRMODE_TX, TSPI_TRMODE_RX or TSPI_TRMODE_TXRX.  
  * @retval None.  
  */
void TSPI_SelectTransferMode(TSB_TSPI_TypeDef * TSPIx, TSPI_TransferMode TrMode)
{
    /* CR1 used: bit14, 13, 12, 11-10, 8, 7-0, clear bit11-10 here */
    uint32_t tmp = TSPIx->CR1 & (CR1_TMMD_MASK);

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_TRANSFERMODE(TrMode));

    /* write 'TrMode' to bit11-10 */
    tmp |= (uint32_t) (TrMode) << 10U;
    TSPIx->CR1 = tmp;
}


/**
  * @brief Select chip selection pin.
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param CSx: Specify the chip selection pin.
  *   This parameter can be one of the following values:
  *   TSPI_CS0 or TSPI_CS1.  
  * @retval None.  
  */
void TSPI_SetCS(TSB_TSPI_TypeDef * TSPIx, TSPI_CSx CSx)
{
    /* CR1 used: bit14, 13, 12, 11-10, 8, 7-0, clear bit8 here */
    uint32_t tmp = TSPIx->CR1 & (CR1_CSSEL_MASK);

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_CSX(CSx));

    /* write 'CSx' to bit8 */
    tmp |= (uint32_t) (CSx) << 8U;
    TSPIx->CR1 = tmp;
}

/**
  * @brief Set the number of transfer frame. 
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param Num: Specify the number of transferring.
  *   This parameter can be one of the following values:
  *   1: One frame (Single transfer)  
  *   2-255: 2 to 255 frames (Burst transfer)  
  * @retval None.  
  */
void TSPI_SetTransferNum(TSB_TSPI_TypeDef * TSPIx, uint8_t Num)
{
    /* CR1 used: bit14, 13, 12, 11-10, 8, 7-0, clear bit7-0 here */
    uint32_t tmp = TSPIx->CR1 & (CR1_FC_MASK);

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_TRANSFERNUM(Num));

    /* write 'Num' to bit7-0 */
    tmp |= (uint32_t) (Num);
    TSPIx->CR1 = tmp;
}

/**
  * @brief Fixed output value function control when TSPIxTXD idles. 
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param OutputValue: Specify output value when TSPIxTXD idles.
  *   This parameter can be one of the following values:
  *   TSPI_TXOUTVALUE_HIZ, TSPI_TXOUTVALUE_LASTDATA,
  *   TSPI_TXOUTVALUE_KEEPLOW, TSPI_TXOUTVALUE_KEEPHIGH  
  * @retval None.  
  */
void TSPI_SetOutValueInIdle(TSB_TSPI_TypeDef * TSPIx, TSPI_TxOutValueInIdle OutputValue)
{
    /* CR2 used: bit23-22, 21, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit 23, 22 here */
    uint32_t tmp = TSPIx->CR2 & (CR2_TIDLE_MASK);

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_TXOUTVALUEINIDLE(OutputValue));

    /* write 'OutputValue' to bit23-22 */
    tmp |= (uint32_t) (OutputValue) << 22U;
    TSPIx->CR2 = tmp;
}

/**
  * @brief Fixed output value function control when TSPIxTXD Andorrans. 
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param OutputValue: Specify output value when TSPIxTXD Andorrans.
  *   This parameter can be one of the following values:
  *   TSPI_TXOUTVALUE_FIXEDTOLOW, TSPI_TXOUTVALUE_FIXEDTOHIGH
  * @retval None.  
  */
void TSPI_SetOutValueInAndorran(TSB_TSPI_TypeDef * TSPIx, TSPI_TxOutValueInAndorran OutputValue)
{
    /* CR2 used: bit23-22, 21, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit 21 here */
    uint32_t tmp = TSPIx->CR2 & (CR2_TXDEMP_MASK);

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_TXOUTVALUEINANDORRAN(OutputValue));

    /* write 'OutputValue' to bit21 */
    tmp |= (uint32_t) (OutputValue) << 21U;
    TSPIx->CR2 = tmp;
}

/**
  * @brief Set the FIFO fill level to generate transfer(Tx or Rx) interrupt. 
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param TxRx: Specify tranmit or receive.
  *   This parameter can be one of the following values:
  *   TSPI_TX, TSPI_RX
  * @param Level: Specify the FIFO fill level to generate interrupt.
  *   This parameter can be one of the following values:
  *   0 to 8  (depended on the Data format)
  * @retval None.  
  */
void TSPI_SetFIFOLevelINT(TSB_TSPI_TypeDef * TSPIx, uint8_t TxRx, uint8_t Level)
{
    /* read CR2 */
    uint32_t tmp = TSPIx->CR2;

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_TXRX(TxRx));
    assert_param(IS_TSPI_FIFOLEVELINT(Level));

    if (TxRx == TSPI_TX) {
        /* CR2 used: bit23-22, 21, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit15-12 here */
        tmp &= CR2_TIL_MASK;
        tmp |= (uint32_t) Level << 12U;
    } else {                    /* is TSPI_RX */
        /* CR2 used: bit23-22, 21, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit11-8 here */
        tmp &= CR2_RIL_MASK;
        tmp |= (uint32_t) Level << 8U;
    }

    TSPIx->CR2 = tmp;
}

/**
  * @brief Set Transfer DMA request control. 
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param TxRx: Specify tranmit or receive.
  *   This parameter can be one of the following values:
  *   TSPI_TX, TSPI_RX
  * @param NewState: Specify the state of transfer DMA request.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE. 
  * @retval None.  
  */
void TSPI_SetDMA(TSB_TSPI_TypeDef * TSPIx, uint8_t TxRx, FunctionalState NewState)
{
    /* read CR2 */
    uint32_t tmp = TSPIx->CR2;

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_TXRX(TxRx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (TxRx == TSPI_TX) {
        /* CR2 used: bit23-22, 21, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit1 here */
        tmp &= CR2_DMATE_MASK;
        tmp |= (uint32_t) (NewState) << 1U;
    } else {                    /* is TSPI_RX */
        /* CR2 used: bit23-22, 21, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit0 here */
        tmp &= CR2_DMARE_MASK;
        tmp |= (uint32_t) (NewState);
    }

    TSPIx->CR2 = tmp;
}


/**
  * @brief Enable/Disable the interrupt source
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param IntSrc: Specify the interrupt source to be set
  *   This parameter can be one of the following values or their combination:
  *   TSPI_INT_ERR, 
  *   TSPI_INT_RXEND, TSPI_INT_RXFIFO,
  *   TSPI_INT_TXEND, TSPI_INT_TXFIFO, 
  *   TSPI_INT_ALL (all above interrupt source)    
  * @param NewState: Specify the state for interrupt control.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.   
  * @retval None.  
  */
void TSPI_SetINT(TSB_TSPI_TypeDef * TSPIx, uint32_t IntSrc, FunctionalState NewState)
{
    /* CR2 used: bit23-22, 21, 15-12, 11-8, 7,6,5,4, 2,1,0. */
    uint32_t tmp = TSPIx->CR2 & (CR2_INT_MASK);

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_INTSRC(IntSrc));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    if (NewState == ENABLE) {
        /* write 'IntSrc' to bit7,6,5,4, 2 */
        tmp |= IntSrc;
    } else {
        tmp &= ~IntSrc;
    }
    TSPIx->CR2 = tmp;
}


/**
  * @brief Initialize transfer FIFO pointer. 
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param TxRx: Specify tranmit or receive.
  *   This parameter can be one of the following values:
  *   TSPI_TX, TSPI_RX
  * @retval None.  
  */
void TSPI_InitFIFO(TSB_TSPI_TypeDef * TSPIx, uint8_t TxRx)
{
    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_TXRX(TxRx));

    if (TxRx == TSPI_TX) {
        TSPIx->CR3 = 0x02U;
    } else {
        TSPIx->CR3 = 0x01U;
    }
}


/**
  * @brief Set the clock and divider for baud rate generator
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param Clk: Specify the input clock to the baud rate generator.
  *   This parameter can be one of the following values:
  *   TSPI_PHIT0_DIVIDE_2,     TSPI_PHIT0_DIVIDE_4
  *   TSPI_PHIT0_DIVIDE_8,     TSPI_PHIT0_DIVIDE_16  
  *   TSPI_PHIT0_DIVIDE_32,    TSPI_PHIT0_DIVIDE_64
  *   TSPI_PHIT0_DIVIDE_128,   TSPI_PHIT0_DIVIDE_256 
  *   TSPI_PHIT0_DIVIDE_512,   TSPI_PHIT0_DIVIDE_1024
  * @param Divider: Specify the division ratio "N" 
  *                 (Note: clock output to pin TSPIxSCK = Clk / N ).
  *   This parameter can be one of the following values:
  *   1 to 16
  * @retval None.  
  */
void TSPI_SetBaudRate(TSB_TSPI_TypeDef * TSPIx, TSPI_BaudClock Clk, uint8_t Divider)
{
    uint32_t tmp = Divider;

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_TSPI_BAUDCLOCK(Clk));
    assert_param(IS_TSPI_DIVIDER(Divider));

    if (tmp == 16U) {
        tmp = 0U;
    } else {
        /* Do nothing */
    }
    TSPIx->BR = (((uint32_t) Clk) << 4U) | (tmp);
}

/**
  * @brief Initialize the specified TSPI channel 
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param Init: The structure containing basic TSPI configuration.
  * @retval None.  
  * NOTE: call TSPI_SetTransferNum() at first.
  */
void TSPI_Init(TSB_TSPI_TypeDef * TSPIx, TSPI_InitTypeDef * Init)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));
    assert_param(IS_POINTER_NOT_NULL(Init));
    assert_param(IS_TSPI_DATADIRECTION(Init->DataDirection));
    assert_param(IS_TSPI_FRAMELENGTH(Init->FrameLength));
    assert_param(IS_TSPI_CYCLEBETWEENFRAMES(Init->CycleBetweenFrames));
    assert_param(IS_TSPI_CLKPOLORITY(Init->ClkPolority));
    assert_param(IS_TSPI_SHORTESTIDLECYCLE(Init->ShortestIdleCycle));
    assert_param(IS_TSPI_CSPOLARITY(Init->CS1Polarity));
    assert_param(IS_TSPI_CSPOLARITY(Init->CS0Polarity));
    assert_param(IS_TSPI_DELAYCYCLENUM(Init->DelayCycleNum_CS_SCLK));
    assert_param(IS_TSPI_DELAYCYCLENUM(Init->DelayCycleNum_Negate_CS));
    assert_param(IS_TSPI_HOLDTIME(Init->LastBit_Hold_Time));
    assert_param(IS_FUNCTIONAL_STATE(Init->ParityCheck));
    assert_param(IS_TSPI_PARITYTYPE(Init->Parity));


    tmp = (uint32_t) Init->DataDirection << 31U;
    tmp |= (uint32_t) Init->FrameLength << 24U;
    tmp |= (uint32_t) Init->CycleBetweenFrames << 20U;
    tmp |= (uint32_t) Init->ClkPolority << 14U;
    tmp |= (uint32_t) Init->ShortestIdleCycle << 10U;
    tmp |= (uint32_t) Init->CS1Polarity << 9U;
    tmp |= (uint32_t) Init->CS0Polarity << 8U;
    tmp |= ((uint32_t) Init->DelayCycleNum_CS_SCLK - 1U) << 4U;
    tmp |= ((uint32_t) Init->DelayCycleNum_Negate_CS - 1U) << 0U;
    TSPIx->FMTR0 = tmp;

    tmp = (uint32_t) Init->LastBit_Hold_Time << 4U;
    tmp |= (uint32_t) Init->ParityCheck << 1U;
    tmp |= (uint32_t) Init->Parity << 0U;
    TSPIx->FMTR1 = tmp;

}

/**
  * @brief Access to transmit FIFO 
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @param dat: The data which will be sent.
  * @retval None.  
  */
void TSPI_SetTxData(TSB_TSPI_TypeDef * TSPIx, uint32_t Dat)
{
    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));

    /* Do not write data to 'DR' register when transmit FIFO is full */
    TSPIx->DR = Dat;
}

/**
  * @brief Access to receive FIFO 
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @retval The received data.  
  */
uint32_t TSPI_GetRxData(TSB_TSPI_TypeDef * TSPIx)
{
    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));

    /* Do not read data from 'DR' register when receive FIFO is empty */
    return TSPIx->DR;
}


/**
  * @brief Check if TSPI registers is modifiable.
  * @param TSPIx: Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @retval The modifiable status of TSPI register. 
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.    
  */
FunctionalState TSPI_IsRegisterModifiable(TSB_TSPI_TypeDef * TSPIx)
{
    FunctionalState tmp = ENABLE;

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));

    if (TSPIx->SR & (0x01U << 31U)) {   /* bit31 == 1: cann't be modified */
        tmp = DISABLE;
    } else {
        /* Do nothing */
    }
    return tmp;

}


/**
  * @brief Get the TSPI status.
  * @param TSPIx:  Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @retval The Tx/Rx FIFO, Run, Interrupt status of TSPI unit.
  */
TSPI_StatusFlag TSPI_GetStatus(TSB_TSPI_TypeDef * TSPIx)
{
    TSPI_StatusFlag tmp = { 0U };

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));

    /* the unused bit is read as '0', no need to mask them */
    tmp.All = TSPIx->SR;
    return tmp;
}


/**
  * @brief Get error flag of TSPI transfer.
  * @param TSPIx:  Select TSPI unit.
  *   This parameter can be one of the following values:
  *   TSB_TSPI0
  * @retval The state of transfer buffer.
  */
TSPI_Err TSPI_GetErrorState(TSB_TSPI_TypeDef * TSPIx)
{
    TSPI_Err tmp = { 0U };

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));

    tmp.All = TSPIx->ERR;
    return tmp;
}

/**
  * @brief Clear all TSPI error flag.
  * @param TSPIx:  Select TSPI unit.
  *   This parameter can be one of the following values:
  *    TSB_TSPI0
  * @retval None
  */
void TSPI_ClrAllErrFlag(TSB_TSPI_TypeDef * TSPIx)
{

    /* Check the parameters */
    assert_param(IS_TSPI_UNIT(TSPIx));

    TSPIx->ERR = (uint32_t) 0xE00;
}




/** @} */
/* End of group TSPI_Exported_Functions */

/** @} */
/* End of group TSPI */

/** @} */
/* End of group TX00_Periph_Driver */

#endif                          /* defined(__TMPM066_TSPI_H) */
