/**
 *******************************************************************************
 * @file    tmpm066_udmac.c
 * @brief   This file provides API functions for uDMAC driver.
 * @version V2.0.2.1
 * @date    2015/09/11
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */


/* Includes ------------------------------------------------------------------*/
#include "tmpm066_udmac.h"

#if defined(__TMPM066_uDMAC_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup uDMAC
  * @brief uDMAC driver modules
  * @{
  */

/** @defgroup uDMAC_Private_Defines
  * @{
  */


/** @} */
/* End of group uDMAC_Private_Defines */

/** @defgroup uDMAC_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group uDMAC_Private_FunctionPrototypes */

/** @defgroup uDMAC_Private_Functions
  * @{
  */

/** @} */
/* End of group uDMAC_Private_Functions */

/** @defgroup uDMAC_Exported_Functions
  * @{
  */

/**
  * @brief  Get the state of the DMA circuit.
  * @param  None
  * @retval The state of DMAC operation.
  *   This parameter can be one of the following values:
  *   DISABLE : The DMA circuit is disabled.
  *   ENABLE  : The DMA circuit is enabled.
  */
FunctionalState DMAC_GetDMACState(void)
{
    FunctionalState state = DISABLE;

    /* read TSB_DMA->STATUS.b0, 0 is disable, 1 is enable */
    if (TSB_DMA->STATUS & (uint32_t) 0x00000001) {
        state = ENABLE;
    } else {
        /* Do nothing */
    }

    return state;
}


/**
  * @brief  Enable the DMA circuit.
  * @param  None
  * @retval None
  */
void DMAC_Enable(void)
{
    /* set TSB_DMA->CFG.b0 */
    TSB_DMA->CFG = (uint32_t) 0x00000001;
}


/**
  * @brief  Disable the DMA circuit.
  * @param  None
  * @retval None
  */
void DMAC_Disable(void)
{
    /* clr TSB_DMA->CFG.b0 */
    TSB_DMA->CFG = (uint32_t) 0x00000000;
}



/** 
  * @brief  Set the base address of the primary data of the DMA circuit.
  * @param  None
  * @param  Addr: The base address of the primary data, bit0 to bit9 must be 0.
  * @retval None
  */
void DMAC_SetPrimaryBaseAddr(uint32_t Addr)
{

    assert_param(IS_DMAC_CTRL_BASE(Addr));

    TSB_DMA->CTRLBASEPTR = Addr;
}


/**
  * @brief  Get the primary/alternative base address of the DMA circuit.
  * @param  None
  * @param  PriAlt: Select base address type.
  *   This parameter can be one of the following values:
  *   DMAC_PRIMARY :  Get primary base address
  *   DMAC_ALTERNATE:  Get alternative base address
  * @retval The base address of primary/alternative data.
  */
uint32_t DMAC_GetBaseAddr(DMAC_PrimaryAlt PriAlt)
{
    uint32_t tmp = 0U;


    assert_param(IS_DMAC_PRIMARYALT(PriAlt));

    if (PriAlt == DMAC_PRIMARY) {
        tmp = TSB_DMA->CTRLBASEPTR;
    } else {
        tmp = TSB_DMA->ALTCTRLBASEPTR;
    }

    return tmp;
}

/**
  * @brief  Set software transfer request to the specified channel of the DMA circuit.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION 
  * @retval None
  */
void DMAC_SetSWReq(uint8_t Channel)
{
    uint32_t tmp = 0U;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));

    /* write '1' to right bit position to requests a transfer */
    tmp = TSB_DMA->CHNLSWREQUEST;
    tmp |= (0x01UL << ((uint32_t) Channel));
    TSB_DMA->CHNLSWREQUEST = tmp;
}

/**
  * @brief  Set transfer type to the DMA circuit.
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION 
  * @retval None.
  */
void DMAC_SetTransferType(uint8_t Channel, DMAC_TransferType Type)
{
    uint32_t tmp = 0U;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));
    assert_param(IS_DMAC_TRANSFERTYPE(Type));

    if (Type == DMAC_BURST) {
        /* Write '1' to _SET to disables the single transfer to the channel, and only burst transfer request is valid */
        tmp = TSB_DMA->CHNLUSEBURSTSET;
        tmp |= 0x01UL << ((uint32_t) Channel);
        TSB_DMA->CHNLUSEBURSTSET = tmp;
    } else {
        /* Do nothing */
    }
}

/**
  * @brief  Get transfer type setting for the DMA circuit.
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @retval The transfer type.
  *   DMAC_BURST  : Only burst transfer request can be used.
  *   DMAC_SINGLE : Single transfer is enabled.
  */
DMAC_TransferType DMAC_GetTransferType(uint8_t Channel)
{
    uint32_t tmp = 0U;
    DMAC_TransferType Type = DMAC_SINGLE;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));

    tmp = TSB_DMA->CHNLUSEBURSTSET & (0x01UL << ((uint32_t) Channel));
    if (tmp) {
        /* read _SET, if bit is not 0, Only burst transfer request can be used */
        Type = DMAC_BURST;
    } else {
        Type = DMAC_SINGLE;
    }
    return Type;
}


/**
  * @brief  Set mask for the specified channel of the DMA circuit.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @param  NewState: Clear or set the mask to enable or disable the DMA channel
  *   This parameter can be one of the following values:
  *   ENABLE  : The DMA channel mask is cleared, DMA request is enable(valid)
  *   DISABLE : The DMA channel is masked, DMA request is disable(invalid)
  * @retval None
  */
void DMAC_SetMask(uint8_t Channel, FunctionalState NewState)
{
    uint32_t tmp = 0U;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        /* ENABLE: clear mask to enable DMA channel,(valid). write '1' to _CLR */
        tmp = TSB_DMA->CHNLREQMASKCLR;
        tmp |= (0x01UL << ((uint32_t) Channel));
        TSB_DMA->CHNLREQMASKCLR = tmp;
    } else {
        /* DISABLE: set mask to disable DMA channel, (invalid), write '1' to _SET */
        tmp = TSB_DMA->CHNLREQMASKSET;
        tmp |= (0x01UL << ((uint32_t) Channel));
        TSB_DMA->CHNLREQMASKSET = tmp;
    }
}

/**
  * @brief  Get mask setting for the specified channel of the DMA circuit.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @retval The inverted mask setting.
  *   ENABLE  : The DMA channel mask is cleared, DMA request is enable(valid)
  *   DISABLE : The DMA channel is masked, DMA request is disable(invalid)
  */
FunctionalState DMAC_GetMask(uint8_t Channel)
{
    uint32_t tmp = 0U;
    FunctionalState state = DISABLE;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));

    /* read _SET, mask with '1 << Channel' */
    tmp = TSB_DMA->CHNLREQMASKSET & (uint32_t) (0x01UL << ((uint32_t) Channel));
    if (tmp) {                  /* if result is not 0, return DISABLE */
        state = DISABLE;
    } else {                    /* if result is 0, return ENABLE */
        state = ENABLE;
    }

    return state;
}

/**
  * @brief  Enable or disable the specified channel of the DMA circuit.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:  
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @param  NewState: Enable or disable the DMA channel
  *   This parameter can be one of the following values:
  *   ENABLE  : The DMA channel will be enabled
  *   DISABLE : The DMA channel will be disabled
  * @retval None
  */
void DMAC_SetChannel(uint8_t Channel, FunctionalState NewState)
{
    uint32_t tmp = 0U;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        /* ENABLE: write '1' to _SET */
        tmp = TSB_DMA->CHNLENABLESET;
        tmp |= (0x01UL << ((uint32_t) Channel));
        TSB_DMA->CHNLENABLESET = tmp;
    } else {
        /* DISABLE: write '1' to _CLR */
        tmp = TSB_DMA->CHNLENABLECLR;
        tmp |= (0x01UL << ((uint32_t) Channel));
        TSB_DMA->CHNLENABLECLR = tmp;
    }
}

/**
  * @brief  Get the enable/disable setting for specified channel of the DMA circuit.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:  
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @retval  The enable/disable setting for channel
  *   ENABLE  : The DMA channel is enabled
  *   DISABLE : The DMA channel is disabled  
  */
FunctionalState DMAC_GetChannelState(uint8_t Channel)
{
    uint32_t tmp = 0U;
    FunctionalState state = DISABLE;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));

    /* read _SET, mask with '1 << Channel' */
    tmp = TSB_DMA->CHNLENABLESET & (uint32_t) (0x01UL << ((uint32_t) Channel));
    if (tmp) {                  /* if result is not 0, return ENABLE */
        state = ENABLE;
    } else {                    /* if result is 0, return DISABLE */
        state = DISABLE;
    }
    return state;
}

/**
  * @brief  Set to use primary data or alternative data for specified channel of the DMA circuit.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:  
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @param  PriAlt: Select primary data or alternative data for channel specified by 'ChannelA' above
  *   This parameter can be one of the following values:
  *   DMAC_PRIMARY   : Channel will use primary data
  *   DMAC_ALTERNATE : Channel will use alternative data
  * @retval None
  */
void DMAC_SetPrimaryAlt(uint8_t Channel, DMAC_PrimaryAlt PriAlt)
{
    uint32_t tmp = 0U;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));
    assert_param(IS_DMAC_PRIMARYALT(PriAlt));

    if (PriAlt == DMAC_PRIMARY) {
        /* PRIMARY: write '1' to _CLR */
        tmp = TSB_DMA->CHNLPRIALTCLR;
        tmp |= (0x01UL << ((uint32_t) Channel));
        TSB_DMA->CHNLPRIALTCLR = tmp;
    } else {
        /* ALTERNATE: write '1' to _SET */
        tmp = TSB_DMA->CHNLPRIALTSET;
        tmp |= (0x01UL << ((uint32_t) Channel));
        TSB_DMA->CHNLPRIALTSET = tmp;
    }
}

/**
  * @brief  Get the setting of the using of primary data or alternative data for specified channel of the DMA circuit.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:  
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @retval The setting of the using of primary data or alternative data
  *   DMAC_PRIMARY   : Channel is using primary data 
  *   DMAC_ALTERNATE : Channel is using alternative data
  */
DMAC_PrimaryAlt DMAC_GetPrimaryAlt(uint8_t Channel)
{
    uint32_t tmp = 0U;
    DMAC_PrimaryAlt PriAlt = DMAC_PRIMARY;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));

    /* read _SET, mask with '1 << Channel' */
    tmp = TSB_DMA->CHNLPRIALTSET & (uint32_t) (0x01UL << (uint32_t) Channel);
    if (tmp) {                  /* if result is not 0, return _ALTERNATE */
        PriAlt = DMAC_ALTERNATE;
    } else {                    /* if result is 0, return _PRIMARY */
        PriAlt = DMAC_PRIMARY;
    }
    return PriAlt;
}

/**
  * @brief  Set the priority for specified channel of the DMA circuit.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:  
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @param  Priority: Select Priority.
  *   This parameter can be one of the following values:
  *   DMAC_PRIOTIRY_NORMAL : Normal priority
  *   DMAC_PRIOTIRY_HIGH   : High priority
  * @retval None
  */
void DMAC_SetChannelPriority(uint8_t Channel, DMAC_Priority Priority)
{
    uint32_t tmp = 0U;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));
    assert_param(IS_DMAC_PRIORITY(Priority));

    if (Priority == DMAC_PRIOTIRY_NORMAL) {
        /* DMAC_PRIOTIRY_NORMAL: write '1' to _CLR */
        tmp = TSB_DMA->CHNLPRIORITYCLR;
        tmp |= (0x01UL << ((uint32_t) Channel));
        TSB_DMA->CHNLPRIORITYCLR = tmp;
    } else {
        /* DMAC_PRIOTIRY_HIGH: write '1' to _SET */
        tmp = TSB_DMA->CHNLPRIORITYSET;
        tmp |= (0x01UL << ((uint32_t) Channel));
        TSB_DMA->CHNLPRIORITYSET = tmp;
    }
}

/**
  * @brief  Get the priority setting for specified channel of the DMA circuit.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:  
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @retval  The priority setting of channel
  *   DMAC_PRIOTIRY_NORMAL : Normal priority
  *   DMAC_PRIOTIRY_HIGH   : High priority
  */
DMAC_Priority DMAC_GetChannelPriority(uint8_t Channel)
{
    uint32_t tmp = 0U;
    DMAC_Priority Priority = DMAC_PRIOTIRY_NORMAL;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));

    /* read _SET, mask with '1 << Channel' */
    tmp = TSB_DMA->CHNLPRIORITYSET & (uint32_t) (0x01UL << (uint32_t) Channel);
    if (tmp) {                  /* if result is not 0, return _PRIOTIRY_HIGH */
        Priority = DMAC_PRIOTIRY_HIGH;
    } else {                    /* if result is 0, return _PRIOTIRY_NORMAL */
        Priority = DMAC_PRIOTIRY_NORMAL;
    }
    return Priority;
}

/**
  * @brief  Clear the bus error of the DMA circuit.
  * @param  None
  * @retval None
  */
void DMAC_ClearBusErr(void)
{


    /* write '1' to bit0 */
    TSB_DMA->ERRCLR = (uint32_t) 0x01U;
}

/**
  * @brief  Get the bus error state of the DMA circuit.
  * @param  None
  * @retval The bus error state
  *   SUCCESS :  No bus error
  *   ERROR   :  There is error in bus
  */
Result DMAC_GetBusErrState(void)
{
    Result tmp = SUCCESS;


    /* read bit0, '0' is no error, '1' is error */
    if (TSB_DMA->ERRCLR & (uint32_t) 0x01U) {
        tmp = ERROR;
    } else {
        /* Do nothing */
    }
    return tmp;
}

/**
  * @brief  Fill the DMA setting data of specified channel of the DMA circuit to RAM.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:  
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @param  InitStruct: the structure contain the DMA setting values.
  * @retval None
  */
void DMAC_FillInitData(uint8_t Channel, DMAC_InitTypeDef * InitStruct)
{
    DMAC_PrimaryAlt PriAlt = DMAC_PRIMARY;
    uint32_t Addr = 0U;
    uint32_t tmpCtrl = 0U;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));

    assert_param(IS_POINTER_NOT_NULL(InitStruct));

    assert_param(IS_DMAC_CYCLECTRL(InitStruct->Mode));
    assert_param(IS_DMAC_NEXT_USEBURST(InitStruct->NextUseBurst));
    assert_param(IS_DMAC_TXNUM(InitStruct->TxNum));
    assert_param(IS_DMAC_ARBITRATION(InitStruct->ArbitrationMoment));
    assert_param(IS_DMAC_BIT_WIDTH(InitStruct->SrcWidth));
    assert_param(IS_DMAC_INCWIDTH(InitStruct->SrcInc));
    assert_param(IS_DMAC_BIT_WIDTH(InitStruct->DstWidth));
    assert_param(IS_DMAC_INCWIDTH(InitStruct->DstInc));

    assert_param(IS_DMAC_INCWIDTH_VALID
                 ((uint32_t) InitStruct->SrcWidth, (uint32_t) InitStruct->SrcInc));
    assert_param(IS_DMAC_INCWIDTH_VALID
                 ((uint32_t) InitStruct->DstWidth, (uint32_t) InitStruct->DstInc));

    PriAlt = DMAC_GetPrimaryAlt(Channel);

    if (PriAlt == DMAC_PRIMARY) {
        Addr = DMAC_GetBaseAddr(DMAC_PRIMARY);
    } else {
        Addr = DMAC_GetBaseAddr(DMAC_ALTERNATE);
    }

    /* add offset for channel, each channel use (4 x uint32_t) RAM */
    Addr += (16U * (uint32_t) Channel);

    /* make setting data for 'control data' */
    tmpCtrl |= (uint32_t) InitStruct->Mode;
    tmpCtrl |= (uint32_t) InitStruct->NextUseBurst << 3U;
    tmpCtrl |= (uint32_t) (InitStruct->TxNum - 1U) << 4U;
    tmpCtrl |= (uint32_t) InitStruct->ArbitrationMoment << 14U;
    tmpCtrl |= (uint32_t) InitStruct->SrcWidth << 24U;
    tmpCtrl |= (uint32_t) InitStruct->SrcInc << 26U;
    tmpCtrl |= (uint32_t) InitStruct->DstWidth << 28U;
    tmpCtrl |= (uint32_t) InitStruct->DstInc << 30U;

    /* write setting data to right address */
    *((uint32_t *) (Addr)) = InitStruct->SrcEndPointer;
    Addr += 4U;
    *((uint32_t *) (Addr)) = InitStruct->DstEndPointer;
    Addr += 4U;
    *((uint32_t *) (Addr)) = tmpCtrl;
    Addr += 4U;
    *((uint32_t *) (Addr)) = (uint32_t) 0U;
}


/** @} */
/* End of group uDMAC_Exported_Functions */

/** @} */
/* End of group uDMAC */

/** @} */
/* End of group TX00_Periph_Driver */

#endif                          /* defined(__TMPM066_uDMAC_H) */
