/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of CG demo for the TOSHIBA
 *          'TMPM066' Device Series 
 * @version V0.100
 * @date    2015/10/23
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

#include "main.h"

/**
  * @brief  The main function of demo
  * @param  None
  * @retval None
  */

int main(void)
{
    SW_Init();  
    LED_Init();

    GPIO_ExtIntSrc();

    /* after release standby mode */
    if (is_StanbyRelease()) {
        LED_On(LED_ALL);
        INTIFAO_SetSTBYReleaseINTSrc(CG_ExtINTSrc,INTIFAO_INT_ACTIVE_STATE_H,ENABLE);
        NVIC_EnableIRQ(ExtINTSrc_IRQn);

    } else {
        INTIFAO_ClearINTReq(CG_ExtINTSrc);
        NVIC_ClearPendingIRQ(ExtINTSrc_IRQn);
        NVIC_EnableIRQ(ExtINTSrc_IRQn);
    }

    LED_On(LED_ALL);
    while (1U) {
        if (SW_Get(CG_SW) == 0) {

            LED_Off(LED_ALL);   /* LED  is off before enter stop1 */
            Send_To_LCD(FIRST_LINE, LCD_COMMAND);
            Send_LCD_Text("STOP MODE\r\n");
            enter_STOP1();
        } else {
            LED_On(LED_ALL);
			switchFromIHOSCtoEHOSC();
            Send_To_LCD(FIRST_LINE, LCD_COMMAND);
            Send_LCD_Text("NORMAL MODE\r\n");
        }
    }
}

/**
  * @brief  Enter stop1 low power mode
  * @param  None
  * @retval None
  */
void enter_STOP1(void)
{
    INTIFAO_SetSTBYReleaseINTSrc(CG_ExtINTSrc,INTIFAO_INT_ACTIVE_STATE_RISING,ENABLE);

    config_STOP1();

    __DSB();
    __WFI();
}

/**
  * @brief  Set GPIO of External interrupt source
  * @param  None
  * @retval None
  */
void GPIO_ExtIntSrc(void)
{
    GPIO_SetInput(GPIO_ExtINTSrc, Bit_ExtINTSrc);
    GPIO_DisableFuncReg(GPIO_ExtINTSrc, GPIO_FUNC_REG_1, Bit_ExtINTSrc);
    GPIO_DisableFuncReg(GPIO_ExtINTSrc, GPIO_FUNC_REG_2, Bit_ExtINTSrc);
}

/**
  * @brief  Check the reset factor
  * @param  None
  * @retval 0 or 1
  */

int is_StanbyRelease(void)
{
    volatile AO_ResetFlag tmp = AO_GetResetFlag();
    if (tmp.Bit.PinReset != 1) {
        return 1U;
    } else {
        return 0U;
    }
}

/**
  * @brief  Config the stop1 mode
  * @param  None
  * @retval None
  */
void config_STOP1(void)
{
    uint32_t regval;
    WorkState st = BUSY;

	TSB_WD->MOD = WDT_DISABLE;
	TSB_WD->CR = WDT_DISABLE_CODE;
	while((TSB_FC->SR & SR_BUSY_MASK) == SR_BUSY_FLAG){
	}
    do {
    	st = CG_GetWarmUpState();
	} while (st != DONE);
    /* Set CG module: Normal ->Stop mode */
    CG_SetWarmUpTime(CG_WARM_UP_SRC_OSC_INT_HIGH, CG_WUODR_INT);
    /* Set low power consumption mode stop1 */
    CG_SetSTBYMode(CG_STBY_MODE_STOP1);
	CG_SetFcSrc(CG_FC_SRC_FOSC);
	while(CG_GetFcSrc() == CG_FC_SRC_FPLL){
	}
	CG_SetPLL(DISABLE);
    /* Set fosc source */
    CG_SetFoscSrc(CG_FOSC_OSC_INT);
    /* Wait for <OSCSEL> to become "0" */
    while (CG_GetFoscSrc() == CG_FOSC_OSC_EXT) {
    }
	regval = (TSB_CG->OSCCR & CG_OSCCR_EOSCEN_UNUSE);
	TSB_CG->OSCCR = regval;
	while(TSB_CG->OSCCR != regval){
	}
}

/**
  * @brief  Use the clock gear switching
  * @param  None
  * @retval None
  */
void switchFromIHOSCtoEHOSC(void)
{
	uint32_t regval = TSB_CG->OSCCR;

    CG_SetWarmUpTime(CG_WARM_UP_SRC_OSC_EXT_HIGH, CG_WUODR_EXT);
    regval |= CG_OSCCR_EOSCEN_EHOSC;
    TSB_CG->OSCCR = regval;
    /* Start warm-up */
    CG_StartWarmUp();
    /* Wait until EHOSC become stable */
    while (CG_GetWarmUpState() != DONE) {
    }
    /* Set fosc source */
    CG_SetFoscSrc(CG_FOSC_OSC_EXT);
    /* Wait for <OSCSEL> to become "1" */
    while (CG_GetFoscSrc() != CG_FOSC_OSC_EXT) {
    }
    /* Set up pll and wait for pll to warm up, set fc source to fpll */
    CG_EnableClkMulCircuit();
    /* Set fgear = fc */
    CG_SetFgearLevel(CG_DIVIDE_1);
    /* Set T0 = fc . modify this case by case !*/
    CG_SetPhiT0Level(CG_DIVIDE_1);
}

/**
  * @brief  PLL configuration changes
  * @param  None
  * @retval SUCCESS: PLL operation setting success
  */
Result CG_EnableClkMulCircuit(void)
{
    Result retval = ERROR;
    WorkState st = BUSY;

	CG_SetFcSrc(CG_FC_SRC_FOSC);
	while(CG_GetFcSrc() == CG_FC_SRC_FPLL){
	}
    CG_SetPLL(DISABLE);
    CG_SetFPLLValue(CG_12M_MUL_8_FPLL);
    /* Set warm up time */
    CG_SetWarmUpTime(CG_WARM_UP_SRC_OSC_EXT_HIGH, CG_WUODR_PLL);
    CG_StartWarmUp();
    do {
		st = CG_GetWarmUpState();
    } while (st != DONE);
    retval = CG_SetPLL(ENABLE);
    if (retval == SUCCESS) {
        /* Set warm up time */
        CG_SetWarmUpTime(CG_WARM_UP_SRC_OSC_EXT_HIGH, CG_WUODR_PLL);
        CG_StartWarmUp();
        do {
            st = CG_GetWarmUpState();
        } while (st != DONE);
       CG_SetFcSrc(CG_FC_SRC_FPLL);
    } else {
        /*Do nothing */
    }
    return retval;
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1U) {
        __NOP();
    }
}
#endif
