/** 
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of TMRD interlock PPG demo for the TOSHIBA
 *          'TMPM066' Device 
 * @version V0.100
 * @date    2015/09/16
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

#include "tmpm066_tmrd.h"
#include "tmpm066_gpio.h"
#include "tmpm066_cg.h"

/*
 * Cycle is 500us:
 * fosc = 16MHz(external high-speed oscillator)
 * fc = 6 * fosc = 96MHz(in this demo, PLL multiplying value is set to 6 multiplying)
 * ftmrd = fpll  = 96MHz(in this demo, ftmrd = fpll)
 * So the value is 500us*ftmrd = 48000 = 0xBB80U
 *
 * Duty is 50%:
 * Set LeadingTiming0 to 0x0000, then TrailingTiming0 is Cycle/2 = 0x5DC0U(250us)
 * LeadingTiming1 and TrailingTiming1 are not used, so keep their value to 0x0000.
 *
 * Phase shift is 120 degree:
 * The phase shift can be calculated by using the formula:
 *
 * Fai = 360 degree*(PhaseShiftTiming/(Cycle+1))
 *
 * so in this demo, PhaseShiftTiming should be set to Cycle/3 = 0x208DU(167us)
 */
#define TIME_CYCLE               ((uint16_t)0xBB80U)    /* 500us */
#define TIME_DUTY                ((uint16_t)0x5DC0U)    /* 250us */
#define TIME_PHASE_SHIFT         ((uint16_t)0x3E80U)    /* 500/3= 167us */

int main(void)
{
    TMRD_TimingTypeDef timestruct = { 0U };

    timestruct.Cycle = TIME_CYCLE;
    timestruct.TrailingTiming1 = TIME_DUTY;
    timestruct.PhaseShiftTiming = TIME_PHASE_SHIFT;

    /* TDA0OUT1, TDA1OUT1 port setting */
    GPIO_SetOutput(GPIO_PD, GPIO_BIT_1 | GPIO_BIT_3);
    GPIO_EnableFuncReg(GPIO_PD, GPIO_FUNC_REG_2, GPIO_BIT_1 | GPIO_BIT_3/* | GPIO_BIT_2 | GPIO_BIT_3*/);

    /* TMRD CG setting */
    CG_SetFcOptional(CG_FPLL_PERIPH_TMRD,ENABLE);
    CG_SetFcPeriphA(CG_FC_PERIPH_TMRD,ENABLE);

    /* TMRD mode setting */
    TMRD_SetMode(TMRD_MODE_INTERLOCK_PPG_3CH);

    /* TMRD IDLE mode */
    TMRD_SetRunStateInIdle(TMRD_UNIT_CH_0, TMRD_RUN);
    TMRD_SetRunStateInIdle(TMRD_UNIT_CH_1, TMRD_RUN);

    /* TMRD enable clock signal */
    TMRD_Enable(TMRD_UNIT_CH_0);
    TMRD_Enable(TMRD_UNIT_CH_1);

    /* Set the TMRD prescaler */
    TMRD_SetClkDivision(TMRD_UNIT_CH_0, TMRD_CLK_DIV_1);

    /* Set the PPG output edge */
    TMRD_SetPPGInitLeadingEdge(TMRD_PPG_CHANNEL_01, TMRD_WAVE_EDGE_RISING);
    TMRD_SetPPGInitLeadingEdge(TMRD_PPG_CHANNEL_11, TMRD_WAVE_EDGE_RISING);

    /* Direct update the compare register */
    TMRD_SetCMPRegWritePath(TMRD_UNIT_CH_0, TMRD_CMP_WRITE_DIRECT);
    TMRD_SetCMPRegWritePath(TMRD_UNIT_CH_1, TMRD_CMP_WRITE_DIRECT);

    /* Set the value to timer register */
    TMRD_SetInitTiming(TMRD_UNIT_CH_0, &timestruct);
    TMRD_SetInitTiming(TMRD_UNIT_CH_1, &timestruct);

    /* Set the relation of phase 0 and 1 */
    TMRD_SetPhaseRelation(TMRD_PHASE_DELAY_OR_SAME);

    /* Indirct update the compare register */
    TMRD_SetCMPRegWritePath(TMRD_UNIT_CH_0, TMRD_CMP_WRITE_INDIRECT);
    TMRD_SetCMPRegWritePath(TMRD_UNIT_CH_1, TMRD_CMP_WRITE_INDIRECT);

    /* Start to run TMRD */
    TMRD_SetRunState(TMRD_UNIT_CH_0, TMRD_RUN);
    while (1) {
      /* Do nothing */
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
