/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of TSPI demo for the
 *          TOSHIBA 'TMPM066' Device Series
 * @version V0.100
 * @date    2015/09/08
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

/*--------------------------------------------------------------------
 This demo will show basic usage of TSPI module in M066.
 
 Connect pin 21(PF1, TSPI_TXD) to pin 20(PF2, TSPI_RXD),
 the LEDs will blink to show the data is being received.
  
 User can define BITRATE_MIN to see the blink effect.
 
 This demo is compiled by EWARM v6.40 and run on Toshiba M066 Ev board. 
---------------------------------------------------------------------*/


/* include file */
#include "tmpm066_tspi.h"
#include "led.h"
#include "tmpm066_cg.h"

/* define the TSPI unit used in this demo */
#define TSPI_x   TSB_TSPI0
/* to check the effect due to too low speed, uncomment the line below */
#define BITRATE_MIN

#define MAX_BUFSIZE  64U

void ConfigTSPI(void);
void ConfigGPIOForTSPI(void);

typedef enum { SAME = 1U, NOT_SAME = !SAME } TestResult;
TestResult result = NOT_SAME;
TestResult Buffercompare(const uint8_t * Src, const uint8_t * Dst, uint32_t Len);


uint16_t datTx = 1U;            /* its size is the 'FrameLength' */

uint32_t cntTx = 1U;
uint32_t cntRx = 0U;

uint32_t receive = 0U;
uint8_t Rx_Buf[MAX_BUFSIZE] = { 0U };
uint8_t Tx_Buf[MAX_BUFSIZE] = { 0U };

/* main function */
int main(void)
{
    uint8_t CompOnce = 0U;

    /* enable clock supply to TSPI */
    CG_SetFcPeriphA(CG_FC_PERIPH_TSPI, ENABLE);
    
    ConfigGPIOForTSPI();
    ConfigTSPI();

    /* initialize LEDs before display something */
    LED_Init();
   
    /* send the 1st data */
    Tx_Buf[0] = datTx;
    TSPI_SetTxData(TSPI_x, datTx);

    while (1) {
        if (CompOnce == 0U) {
            if (cntRx == MAX_BUFSIZE) {
                CompOnce = 1U;
                result = Buffercompare(Tx_Buf, Rx_Buf, MAX_BUFSIZE);
            } else {
                /* Do nothing */
            }
        } else {
            /* Do nothing */
        }
    }
}

/**
  * @brief  To configure the pins as for TSPI function
  * @param  None
  * @retval None
  */
void ConfigGPIOForTSPI(void)
{
    if (TSPI_x == TSB_TSPI0) {
        TSB_PF->CR = 0x02U;
        TSB_PF->FR1 = 0x06U;
        TSB_PF->FR2 = 0x00U;
        TSB_PF->IE = 0x04U;
    } else {
        /* Do nothing */
    }
}


/**
  * @brief  To configure TSPI by using functions from TSPI driver
  * @param  None
  * @retval None
  */
void ConfigTSPI(void)
{
    /* Single transfer */
    uint8_t transfer_num = 1U;

    /* maximum clock: clk = phiT0/2, divider=1 */
    TSPI_BaudClock clk = TSPI_PHIT0_DIVIDE_2;
    uint8_t divider = 1U;

    TSPI_InitTypeDef init = { 0U };

    TSPI_Enable(TSPI_x);
    TSPI_SWReset(TSPI_x);

    TSPI_SetTxRxCtrl(TSPI_x, ENABLE);
    TSPI_SelectMode(TSPI_x, TSPI_MODE_SIO);
    TSPI_SelectTransferMode(TSPI_x, TSPI_TRMODE_TXRX);

    TSPI_SetTransferNum(TSPI_x, transfer_num);

    TSPI_SetFIFOLevelINT(TSPI_x, TSPI_TX, 0U);
    TSPI_SetFIFOLevelINT(TSPI_x, TSPI_RX, 0U);

    /* enable TSPI interrupt */
    TSPI_SetINT(TSPI_x, TSPI_INT_ALL, ENABLE);

#ifdef BITRATE_MIN
    clk = TSPI_PHIT0_DIVIDE_256;
    divider = 16U;
#endif
    TSPI_SetBaudRate(TSPI_x, clk, divider);

    init.DataDirection = TSPI_LSB_FIRST;
    init.FrameLength = sizeof(datTx) * 8U;      /* size of datTx in main() */
    init.CycleBetweenFrames = 1U;
    init.ClkPolority = TSPI_CLKPOL_LOW;
    init.ShortestIdleCycle = 1U;
    init.CS0Polarity = TSPI_CS_POLARITY_NEGATIVE;
    init.CS1Polarity = TSPI_CS_POLARITY_NEGATIVE;
    init.DelayCycleNum_CS_SCLK = 1U;
    init.DelayCycleNum_Negate_CS = 1U;
    init.ParityCheck = DISABLE;
    init.Parity = TSPI_PARITY_ODD;
    init.LastBit_Hold_Time = 0U;

    TSPI_Init(TSPI_x, &init);

    NVIC_ClearPendingIRQ(INTSPIRX_IRQn); 
    NVIC_EnableIRQ(INTSPIRX_IRQn);

    NVIC_ClearPendingIRQ(INTSPITX_IRQn);
    NVIC_EnableIRQ(INTSPITX_IRQn);

    NVIC_ClearPendingIRQ(INTSPIERR_IRQn);
    NVIC_EnableIRQ(INTSPIERR_IRQn);

    __enable_irq();


}

/**
  * @brief  ISR for TSPI Rx
  * @param  None
  * @retval None
  */
void INTSPIRX_IRQHandler(void)
{
    uint8_t tmp = 0U;

    receive = TSPI_GetRxData(TSPI_x);
    if (cntRx < MAX_BUFSIZE) {
        Rx_Buf[cntRx] = (uint8_t) receive;
        cntRx++;
    }
    /* adjust the loop blinking speed */
    tmp = (uint8_t) (receive >> 4U);
    LED_Off(~tmp);
    LED_On(tmp);
}

/**
  * @brief  ISR for TSPI Tx
  * @param  None
  * @retval None
  */
void INTSPITX_IRQHandler(void)
{
    datTx++;
    TSPI_SetTxData(TSPI_x, datTx);
    if (cntTx < MAX_BUFSIZE) {
        Tx_Buf[cntTx] = datTx;
        cntTx++;
    }
}

/**
  * @brief  ISR for TSPI Error
  * @param  None
  * @retval None
  */
void INTSPIERR_IRQHandler(void)
{
    __NOP();
}


/**
  * @brief  To compare two half word buffer
  * @param  Src : pointer to point source buffer 
  * @param  Dst : pointer to point destination buffer   
  * @param  Len : how many data will be compared 
  * @retval SAME: two buffer contain same data
            NOT_SAME: two buffer contain different data  
  */
TestResult Buffercompare(const uint8_t * Src, const uint8_t * Dst, uint32_t Len)
{
    uint32_t tmp = 0U;
    TestResult result = NOT_SAME;

    for (tmp = 0U; tmp < Len; tmp++) {
        if (Src[tmp] != Dst[tmp]) {
            break;
        } else {
            result = SAME;
        }
    }
    return result;
}



#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
