/**
 *******************************************************************************
 * @file    cdc_loopback.c
 * @brief   CDC(ACM) Loopback Sample Code for TOSHIBA 'TMPM366' Device Series
 * @version V0.202
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 *
 * (C)Copyright TOSHIBA CORPORATION 2014 All rights reserved
 *******************************************************************************
 */

#include <string.h>
#include "usbd_hw.h"
#include "usbd_hw_com.h"
#include "usbd_var.h"
#include "usbd_device_request.h"
#include "usbd_typedefs.h"
#include "usbd_descriptor_cdc.h"
#include "usbd_cdc.h"
#include "usbd_trans.h"

/**
  * @brief Callback information
  */
CDC_CB_t gSample_CDC_CB;
/**
  * @brief Don't care status
  */
bool gbStatDC;

/**
  * @brief Data buffer size
  */
#define SAMPLE_BUF_SIZE     (256U)
/**
  * @brief Data buffer
  */
static uint8_t l_aucSampleBuf[SAMPLE_BUF_SIZE];
/**
  * @brief Line coding buffer
  */
static uint8_t l_aucSampleLineCoding[USBD_CDC_LINE_CODING_SIZE];
/**
  * @brief Line state buffer
  */
static uint16_t l_usSampleControlLineState;

/* Prototypes */
static void _Sample_ConnStat(bool);
static void _Sample_SendData(uint32_t, uint8_t*, uint16_t);
static void _Sample_RecvData(uint32_t, uint8_t*, uint16_t);
static void _Sample_SendEncapsulatedCommad(uint32_t, uint8_t*);
static void _Sample_GetEncapsulatedResponse(uint16_t);
static void _Sample_SetLineCoding(uint32_t, uint8_t*);
static void _Sample_GetLineCoding(uint16_t);
static void _Sample_SetControlLineState(uint16_t);
static void _Sample_ResponseAvailable(void);

/******************************************************************************/

/**
  * @brief  CDC(ACM) sample main routine
  * @param  None
  * @retval None
  */
int main(void)
{
    /* Sample line coding */
    l_aucSampleLineCoding[0] = 0x80U;
    l_aucSampleLineCoding[1] = 0x25U;
    l_aucSampleLineCoding[2] = 0x00U;
    l_aucSampleLineCoding[3] = 0x00U;
    l_aucSampleLineCoding[4] = 0x00U;
    l_aucSampleLineCoding[5] = 0x00U;
    l_aucSampleLineCoding[6] = 0x08U;

    /* Clear line state */
    l_usSampleControlLineState = 0x0000U;

    /* Set CDC(ACM) initialize parameters */
    gSample_CDC_CB.pfnConnStat            = _Sample_ConnStat;
    gSample_CDC_CB.pfnSendEncapsulatedCmd = _Sample_SendEncapsulatedCommad;
    gSample_CDC_CB.pfnGetEncapsulatedRes  = _Sample_GetEncapsulatedResponse;
    gSample_CDC_CB.pfnSetCommFeature      = NULL;
    gSample_CDC_CB.pfnGetCommFeature      = NULL;
    gSample_CDC_CB.pfnClearCommFeature    = NULL;
    gSample_CDC_CB.pfnSetLineCoding       = _Sample_SetLineCoding;
    gSample_CDC_CB.pfnGetLineCoding       = _Sample_GetLineCoding;
    gSample_CDC_CB.pfnSetControlLineState = _Sample_SetControlLineState;
    gSample_CDC_CB.pfnSendBreak           = NULL;
    gSample_CDC_CB.pfnNetworkConnection   = NULL;
    gSample_CDC_CB.pfnResponseAvailable   = _Sample_ResponseAvailable;
    gSample_CDC_CB.pfnSerialState         = NULL;
    gSample_CDC_CB.pfnSendData            = _Sample_SendData;
    gSample_CDC_CB.pfnReciveData          = _Sample_RecvData;

    /* Initialize CDC(ACM) driver */
    usbd_cdc_init(&gSample_CDC_CB);

    /* Initialize driver works */
    usbd_initialize_standard_class_work_data();

    /* Initialize USB controller */
    Config_USB();

    /* Forever loop (Interrupt driven sample) */
    while(true);
}

/**
  * @brief  Callback for connect/disconnect
  * @param  bStat : Connect status.
  *   This parameter can be one of the following values :
  *      true(Connect), false(Disconnect)
  * @retval None
  */
static void _Sample_ConnStat(bool bStat)
{
    if (bStat == true) {
        /* Connect */
        /* Request for data receive */
        gbStatDC = USBD_CDC_RecvData(SAMPLE_BUF_SIZE, l_aucSampleBuf);
        /* No care status in sample */
    } else {
        /* Disconnect */
        /* Do nothing in sample */
    }

    return;
}

/**
  * @brief  Callback for send data
  * @param  ulSize : Send data size.
  * @param  pucData : Send data buffer pointer.
  * @param  usStat : Send status.
  *   This parameter can be one of the following values :
  *      EPSTAT_COMPLETE, EPSTAT_ABORT
  * @retval None
  */
static void _Sample_SendData(uint32_t ulSize, uint8_t *pucData, uint16_t usStat)
{
    if (usStat == EPSTAT_COMPLETE) {
        /* Request for data receive */
        gbStatDC = USBD_CDC_RecvData(SAMPLE_BUF_SIZE, l_aucSampleBuf);
        /* No care status in sample */
    }
    
    return;
}

/**
  * @brief  Callback for receive data
  * @param  ulSize : Receive data size.
  * @param  pucData : Receive data buffer pointer.
  * @param  usStat : Receive status.
  *   This parameter can be one of the following values :
  *      EPSTAT_COMPLETE, EPSTAT_ABORT
  * @retval None
  */
static void _Sample_RecvData(uint32_t ulSize, uint8_t *pucData, uint16_t usStat)
{
    if (usStat == EPSTAT_COMPLETE) {
        /* Request for data send */
        gbStatDC = USBD_CDC_SendData(ulSize, l_aucSampleBuf);
        /* No care status in sample */
    }

    return;
}

/**
  * @brief  Callback for SEND_ENCAPSULATED_COMMAND
  * @param  ulSize : Data size.
  * @param  pucData : Data buffer pointer.
  * @retval None
  */
static void _Sample_SendEncapsulatedCommad(uint32_t ulSize, uint8_t *pucData)
{
    /* SEND_ENCAPSULATED_COMMAND received */
    /* Do nothing in sample */

    return;
}

/**
  * @brief  Callback for GET_ENCAPSULATED_RESPONSE
  * @param  usLength : Request length.
  * @retval None
  */
static void _Sample_GetEncapsulatedResponse(uint16_t usLength)
{
    /* GET_ENCAPSULATED_RESPONSE received */
    /* Do nothing in sample */

    return;
}

/**
  * @brief  Callback for SET_LINE_CODING
  * @param  ulSize : Data size.
  * @param  pucData : Data buffer pointer.
  * @retval None
  */
static void _Sample_SetLineCoding(uint32_t ulSize, uint8_t *pucData)
{
    /* SET_LINE_CODING received */

    /* Keep line coding */
    memcpy(l_aucSampleLineCoding, pucData, ulSize);

    return;
}

/**
  * @brief  Callback for GET_LINE_CODING
  * @param  usLength : Request length.
  * @retval None
  */
static void _Sample_GetLineCoding(uint16_t usLength)
{
    /* GET_LINE_CODING received */

    /* Reply line coding */
    gbStatDC = USBD_CDC_Send_GetLineCoding(usLength, l_aucSampleLineCoding);
    /* No care status in sample */

    return;
}

/**
  * @brief  Callback for SET_CONTROL_LINE_STATE
  * @param  usValue : Request value.
  * @retval None
  */
static void _Sample_SetControlLineState(uint16_t usValue)
{
    /* SET_CONTROL_LINE_STATE received */

    /* Keep line state */
    l_usSampleControlLineState = usValue;

    return;
}

/**
  * @brief  Callback for RESPONSE_AVAILABLE
  * @param  None
  * @retval None
  */
static void _Sample_ResponseAvailable(void)
{
    /* RESPONSE_AVAILABLE received */
    /* Do nothing in sample */

    return;
}
#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/* EOC */
