/**
 *******************************************************************************
 * @file    usbd_cdc.c
 * @brief   CDC(ACM) Function Code for TOSHIBA 'TMPM366' Device Series
 * @version V0.202
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 *
 * (C)Copyright TOSHIBA CORPORATION 2014 All rights reserved
 *******************************************************************************
 */

#include <string.h>
#include "usbd_hw.h"
#include "usbd_device_request.h"
#include "usbd_var.h"
#include "usbd_cdc.h"
#include "usbd_trans.h"
#include "usbd_descriptor_cdc.h"

/**
  * @brief Callback information
  */
CDC_CB_t gCDC_CB;

/**
  * @brief USB connect status
  */
static bool l_bUsbStat;
/**
  * @brief Request data buffer
  */
static uint8_t                      l_aucReqDataBuf[USBD_CDC_REQ_DATA_BUF_SIZE];
/**
  * @brief Encapsulated response size
  */
static uint16_t                     l_wEncapsulatedResponseSize;
/**
  * @brief NETWORK_CONNECTION data
  */
static Notification_t               l_tNetworkConnection;
/**
  * @brief RESPONSE_AVAILABLE data
  */
static Notification_t               l_tResponseAvailable;
/**
  * @brief SERIAL_STATE data
  */
static NotificationSerialState_t    l_tSerialState;

/* Prototypes */
static void _USBD_CDC_CB_SendData(USBD_EPx);
static void _USBD_CDC_CB_RecvData(USBD_EPx);
static bool _USBD_CDC_SendEncapsulatedCommand(void);
static void _USBD_CDC_CB_SendEncapsulatedCommand(USBD_EPx);
static bool _USBD_CDC_GetEncapsulatedResponse(void);
static bool _USBD_CDC_SetCommFeature(void);
static void _USBD_CDC_CB_SetCommFeature(USBD_EPx);
static bool _USBD_CDC_GetCommFeature(void);
static bool _USBD_CDC_ClearCommFeature(void);
static bool _USBD_CDC_SetLineCoding(void);
static void _USBD_CDC_CB_SetLineCoding(USBD_EPx);
static bool _USBD_CDC_GetLineCoding(void);
static bool _USBD_CDC_SetControlLineState(void);
static bool _USBD_CDC_SendBreak(void);
static void _USBD_CDC_CB_Notification_NetworkConnection(USBD_EPx);
static void _USBD_CDC_CB_Notification_ResponseAvailable(USBD_EPx);
static void _USBD_CDC_CB_Notification_SerialState(USBD_EPx);

/**
  * @brief Endpoint information
  */
extern EpTransInfo_t gEpTransInfo[CNumEndpointSupport];

/******************************************************************************/

/**
  * @brief  Initialize CDC/ACM driver
  * @param  p_cdc_cb : CDC/ACM callback information pointer.
  * @retval None
  */
void usbd_cdc_init(CDC_CB_t *p_cdc_cb)
{
    /* Set of callback functions */
    gCDC_CB.pfnConnStat            = p_cdc_cb->pfnConnStat;
    gCDC_CB.pfnSendEncapsulatedCmd = p_cdc_cb->pfnSendEncapsulatedCmd;
    gCDC_CB.pfnGetEncapsulatedRes  = p_cdc_cb->pfnGetEncapsulatedRes;
    gCDC_CB.pfnSetCommFeature      = p_cdc_cb->pfnSetCommFeature;
    gCDC_CB.pfnGetCommFeature      = p_cdc_cb->pfnGetCommFeature;
    gCDC_CB.pfnClearCommFeature    = p_cdc_cb->pfnClearCommFeature;
    gCDC_CB.pfnSetLineCoding       = p_cdc_cb->pfnSetLineCoding;
    gCDC_CB.pfnGetLineCoding       = p_cdc_cb->pfnGetLineCoding;
    gCDC_CB.pfnSetControlLineState = p_cdc_cb->pfnSetControlLineState;
    gCDC_CB.pfnSendBreak           = p_cdc_cb->pfnSendBreak;
    gCDC_CB.pfnNetworkConnection   = p_cdc_cb->pfnNetworkConnection;
    gCDC_CB.pfnResponseAvailable   = p_cdc_cb->pfnResponseAvailable;
    gCDC_CB.pfnSerialState         = p_cdc_cb->pfnSerialState;
    gCDC_CB.pfnSendData            = p_cdc_cb->pfnSendData;
    gCDC_CB.pfnReciveData          = p_cdc_cb->pfnReciveData;

    /* Initialize buffers */
    memset(l_aucReqDataBuf, 0x00U, sizeof(l_aucReqDataBuf));
    memset(&l_tNetworkConnection, 0x00U, sizeof(l_tNetworkConnection));
    memset(&l_tResponseAvailable, 0x00U, sizeof(l_tResponseAvailable));
    memset(&l_tSerialState, 0x00U, sizeof(l_tSerialState));
    l_wEncapsulatedResponseSize = 0U;

    /* Initialization of an internal variable */
    l_bUsbStat = false;

    return;
}

/**
  * @brief  Class request process of CDC/ACM
  * @param  request : Index of request.
  * @retval bStat : true or false.
  */
bool usbd_interrupt_CDC_req(uint32_t request)
{
    bool bStat;

    if ((f_bmDirection == CbmRequestType_Direction_DtoH)
     && (g_RequestType == CbmRequestType_Type_Class)
     && (g_Recipient == CbmRequestType_Recipient_Interface)) {
        switch (request) {
        case USBD_CDC_GET_ENCAPSULATED_RESPONSE:
            /* GET_ENCAPSULATED_RESPONSE */
            bStat = _USBD_CDC_GetEncapsulatedResponse();
            break;
        case USBD_CDC_GET_COMM_FEATURE:
            /* GET_COMM_FEATURE */
            bStat = _USBD_CDC_GetCommFeature();
            break;
        case USBD_CDC_GET_LINE_CODING:
            /* GET_LINE_CODING */
            bStat = _USBD_CDC_GetLineCoding();
            break;
        default:
            /* Not supported request */
            bStat = false;
        }
    } else if ((f_bmDirection == CbmRequestType_Direction_HtoD)
     && (g_RequestType == CbmRequestType_Type_Class)
     && (g_Recipient == CbmRequestType_Recipient_Interface)) {
        switch (request) {
        case USBD_CDC_CLEAR_COMM_FEATURE:
            /* CLEAR_COMM_FEATURE */
            bStat = _USBD_CDC_ClearCommFeature();
            break;
        case USBD_CDC_SET_CONTROL_LINE_STATE:
            /* SET_CONTROL_LINE_STATE */
            bStat = _USBD_CDC_SetControlLineState();
            break;
        case USBD_CDC_SEND_BREAK:
            /* SEND_BREAK */
            bStat = _USBD_CDC_SendBreak();
            break;
        case USBD_CDC_SEND_ENCAPSULATED_COMMAND:
            /* SEND_ENCAPSULATED_COMMAND */
            bStat = _USBD_CDC_SendEncapsulatedCommand();
            break;
        case USBD_CDC_SET_COMM_FEATURE:
            /* SET_COMM_FEATURE */
            bStat = _USBD_CDC_SetCommFeature();
            break;
        case USBD_CDC_SET_LINE_CODING:
            /* SET_LINE_CODING */
            bStat = _USBD_CDC_SetLineCoding();
            break;
        default:
            /* Not supported request */
            bStat = false;
        }
    } else {
        /* Not supported request */
        bStat = false;
    }

    return bStat;
}

/**
  * @brief  Send data on bulk IN endpoint
  * @param  ulSize : Send data size.
  * @param  pucData : Send data buffer pointer.
  * @retval bStat : true or false.
  */
bool USBD_CDC_SendData(uint32_t ulSize, uint8_t *pucData)
{
    bool bStat = false;

    if ((gCallManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_DATA_CLASS) != 0U) {
        /* Send data on EP1 (Bulk-IN) */
        bStat = usbd_Epx_Send(USBD_EP1, ulSize, pucData, NULL, _USBD_CDC_CB_SendData);
    }

    return bStat;
}

/**
  * @brief  Callback for USBD_CDC_SendData
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP1.
  * @retval None
  */
static void _USBD_CDC_CB_SendData(USBD_EPx epNum)
{
    if (gCDC_CB.pfnSendData != NULL) {
        /* Callback */
        (*gCDC_CB.pfnSendData)(gEpTransInfo[epNum].ulTransSize, gEpTransInfo[epNum].pucBuf, gEpTransInfo[epNum].usEpStat);
    }
}

/**
  * @brief  Receive data on bulk OUT endpoint
  * @param  ulSize : Receive data size.
  * @param  pucData : Receive data buffer pointer.
  * @retval bStat : true or false.
  */
bool USBD_CDC_RecvData(uint32_t ulSize, uint8_t *pucData)
{
    bool bStat = false;

    if ((gCallManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_DATA_CLASS) != 0U) {
        /* Receive data on EP2 (Bulk-OUT) */
        bStat = usbd_Epx_Recv(USBD_EP2, ulSize, pucData, NULL, _USBD_CDC_CB_RecvData);
    }

    return bStat;
}

/**
  * @brief  Callback routine for USBD_CDC_RecvData
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP2.
  * @retval None
  */
static void _USBD_CDC_CB_RecvData(USBD_EPx epNum)
{
    if (gCDC_CB.pfnReciveData != NULL) {
        /* Callback */
        (*gCDC_CB.pfnReciveData)(gEpTransInfo[epNum].ulTransSize, gEpTransInfo[epNum].pucBuf, gEpTransInfo[epNum].usEpStat);
    }
}

/**
  * @brief  Received SEND_ENCAPSULATED_COMMAND
  * @param  None
  * @retval bStat : true or false.
  */
static bool _USBD_CDC_SendEncapsulatedCommand(void)
{
    bool bStat = false;

    if ((g_wValue == 0U)
     && (g_wIndex == USBD_CDC_MANAGEMENT_IF)
     && (g_wLength > 0U)) {
        /* Receive data on EP0 (Control-OUT) */
        bStat = usbd_Epx_Recv(USBD_EP0, g_wLength, l_aucReqDataBuf, NULL, _USBD_CDC_CB_SendEncapsulatedCommand);
    }

    return bStat;
}

/**
  * @brief  Callback for _USBD_CDC_SendEncapsulatedCommand
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP0.
  * @retval None
  */
static void _USBD_CDC_CB_SendEncapsulatedCommand(USBD_EPx epNum)
{
    if (gCDC_CB.pfnSendEncapsulatedCmd != NULL) {
        /* Callback */
        (*gCDC_CB.pfnSendEncapsulatedCmd)(gEpTransInfo[epNum].ulTransSize, gEpTransInfo[epNum].pucBuf);
    }
}

/**
  * @brief  Received GET_ENCAPSULATED_RESPONSE
  * @param  None
  * @retval bStat : true or false.
  */
static bool _USBD_CDC_GetEncapsulatedResponse(void)
{
    bool bStat = false;

    if ((g_wValue == 0U)
     && (g_wIndex == USBD_CDC_MANAGEMENT_IF)) {
        /* Keep length */
        l_wEncapsulatedResponseSize = g_wLength;
        
        if (gCDC_CB.pfnGetEncapsulatedRes != NULL) {
            /* Callback */
            (*gCDC_CB.pfnGetEncapsulatedRes)(g_wLength);
        }
        bStat = true;
    }

    return bStat;
}

/**
  * @brief  Received SET_COMM_FEATURE
  * @param  None
  * @retval bStat : true or false.
  */
static bool _USBD_CDC_SetCommFeature(void)
{
    bool bStat = false;

    if ((gAbstractControlManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_FEATURE) != 0U) {
        if ((g_wValue  == USBD_CDC_ABSTRACT_STATE)
         && (g_wIndex  == USBD_CDC_MANAGEMENT_IF)
         && (g_wLength == USBD_CDC_FEATURE_STATUS_SIZE)) {
            /* Receive data on EP0 (Control-OUT) */
            bStat = usbd_Epx_Recv(USBD_EP0, g_wLength, l_aucReqDataBuf, NULL, _USBD_CDC_CB_SetCommFeature);
        }
    }

    return bStat;
}

/**
  * @brief  Callback for _USBD_CDC_SetCommFeature
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP0.
  * @retval None
  */
static void _USBD_CDC_CB_SetCommFeature(USBD_EPx epNum)
{
    if (gCDC_CB.pfnSetCommFeature != NULL) {
        /* Callback */
        (*gCDC_CB.pfnSetCommFeature)(gEpTransInfo[epNum].ulTransSize, gEpTransInfo[epNum].pucBuf);
    }
}

/**
  * @brief  Received GET_COMM_FEATURE
  * @param  None
  * @retval bStat : true or false.
  */
static bool _USBD_CDC_GetCommFeature(void)
{
    bool bStat = false;

    if ((gAbstractControlManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_FEATURE) != 0U) {
        if ((g_wValue  == USBD_CDC_ABSTRACT_STATE)
         && (g_wIndex  == USBD_CDC_MANAGEMENT_IF)
         && (g_wLength == USBD_CDC_FEATURE_STATUS_SIZE)) {
            if (gCDC_CB.pfnGetCommFeature != NULL) {
                /* Callback */
                (*gCDC_CB.pfnGetCommFeature)(g_wLength);
            }
            bStat = true;
        }
    }

    return bStat;
}

/**
  * @brief  Received CLEAR_COMM_FEATURE
  * @param  None
  * @retval bStat : true or false.
  */
static bool _USBD_CDC_ClearCommFeature(void)
{
    bool bStat = false;

    if ((gAbstractControlManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_FEATURE) != 0U) {
        if ((g_wValue  == USBD_CDC_ABSTRACT_STATE)
         && (g_wIndex  == USBD_CDC_MANAGEMENT_IF)
         && (g_wLength == 0U)) {
            if (gCDC_CB.pfnClearCommFeature != NULL) {
                /* Callback */
                (*gCDC_CB.pfnClearCommFeature)();
            }
            bStat = true;
        }
    }

    return bStat;
}

/**
  * @brief  Received SET_LINE_CODING
  * @param  None
  * @retval bStat : true or false.
  */
static bool _USBD_CDC_SetLineCoding(void)
{
    bool bStat = false;

    if ((gAbstractControlManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_LINE_SERIAL) != 0U) {
        if ((g_wValue  == 0U)
         && (g_wIndex  == USBD_CDC_MANAGEMENT_IF)
         && (g_wLength == USBD_CDC_LINE_CODING_SIZE)) {
            /* Receive data on EP0 (Control-OUT) */
            bStat = usbd_Epx_Recv(USBD_EP0, g_wLength, l_aucReqDataBuf, NULL, _USBD_CDC_CB_SetLineCoding);
        }
    }

    return bStat;
}

/**
  * @brief  Callback for _USBD_CDC_SetLineCoding
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP0.
  * @retval None
  */
static void _USBD_CDC_CB_SetLineCoding(USBD_EPx epNum)
{
    if (gCDC_CB.pfnSetLineCoding != NULL) {
        /* Callback */
        (*gCDC_CB.pfnSetLineCoding)(gEpTransInfo[epNum].ulTransSize, gEpTransInfo[epNum].pucBuf);
    }
}

/**
  * @brief  Received GET_LINE_CODING
  * @param  None
  * @retval bStat : true or false.
  */
static bool _USBD_CDC_GetLineCoding(void)
{
    bool bStat = false;

    if ((gAbstractControlManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_LINE_SERIAL) != 0U) {
        if ((g_wValue  == 0U)
         && (g_wIndex  == USBD_CDC_MANAGEMENT_IF)
         && (g_wLength == USBD_CDC_LINE_CODING_SIZE)) {
            if (gCDC_CB.pfnGetLineCoding != NULL) {
                /* Callback */
                (*gCDC_CB.pfnGetLineCoding)(g_wLength);
            }
            bStat = true;
        }
    }

    return bStat;
}

/**
  * @brief  Received SET_CONTROL_LINE_STATE
  * @param  None
  * @retval bStat : true or false.
  */
static bool _USBD_CDC_SetControlLineState(void)
{
    bool bStat = false;

    if ((gAbstractControlManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_LINE_SERIAL) != 0U) {
        if ((g_wIndex  == USBD_CDC_MANAGEMENT_IF)
         && (g_wLength == 0U)) {
            if (gCDC_CB.pfnSetControlLineState != NULL) {
                /* Callback */
                (*gCDC_CB.pfnSetControlLineState)(g_wValue);
            }
            bStat = true;
        }
    }

    return bStat;
}

/**
  * @brief  Received SEND_BREAK
  * @param  None
  * @retval bStat : true or false.
  */
static bool _USBD_CDC_SendBreak(void)
{
    bool bStat = false;

    if ((gAbstractControlManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_BREAK) != 0U) {
        if ((g_wIndex  == USBD_CDC_MANAGEMENT_IF)
         && (g_wLength == 0U)) {
            if (gCDC_CB.pfnSendBreak != NULL) {
                /* Callback */
                (*gCDC_CB.pfnSendBreak)(g_wValue);
            }
            bStat = true;
        }
    }

    return bStat;
}

/**
  * @brief  Send notification for NETWORK_CONNECTION
  * @param  usConn : Network connection status.
  * @retval bStat : true or false.
  */
bool USBD_CDC_Notification_NetworkConnection(uint16_t usConn)
{
    bool            bStat = false;
    bmRequestType_t *p_bmRequestType;

    if ((gAbstractControlManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_NETWORK) != 0U) {
        /* Parameter setup */
        p_bmRequestType                    = (bmRequestType_t *)&l_tNetworkConnection.bmRequestType;
        p_bmRequestType->Bit.Direction     = CbmRequestType_Direction_DtoH;
        p_bmRequestType->Bit.Req_Type      = CbmRequestType_Type_Class;
        p_bmRequestType->Bit.Recipient     = CbmRequestType_Recipient_Interface;
        l_tNetworkConnection.bNotification = USBD_CDC_NETWORK_CONNECTION;
        l_tNetworkConnection.wValue        = usConn;
        l_tNetworkConnection.wIndex        = USBD_CDC_NOTIFICATION_IF;
        l_tNetworkConnection.wLength       = 0U;

        /* Send notification on EP3 (Interrupt-IN) */
        bStat = usbd_Epx_Send(USBD_EP3, sizeof(l_tNetworkConnection), (uint8_t *)&l_tNetworkConnection, NULL, _USBD_CDC_CB_Notification_NetworkConnection);
    }

    return bStat;
}

/**
  * @brief  Callback for USBD_CDC_Notification_NetworkConnection
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP3.
  * @retval None
  */
static void _USBD_CDC_CB_Notification_NetworkConnection(USBD_EPx epNum)
{
    if (gEpTransInfo[epNum].pfnCallback != NULL) {
        /* Callback */
        (*gEpTransInfo[epNum].pfnCallback)(epNum);
    }
}

/**
  * @brief  Send notification for RESPONSE_AVAILABLE
  * @param  None
  * @retval bStat : true or false.
  */
bool USBD_CDC_Notification_ResponseAvailable(void)
{
    bool            bStat;
    bmRequestType_t *p_bmRequestType;

    /* Parameter setup */
    p_bmRequestType                    = (bmRequestType_t *)&l_tResponseAvailable.bmRequestType;
    p_bmRequestType->Bit.Direction     = CbmRequestType_Direction_DtoH;
    p_bmRequestType->Bit.Req_Type      = CbmRequestType_Type_Class;
    p_bmRequestType->Bit.Recipient     = CbmRequestType_Recipient_Interface;
    l_tResponseAvailable.bNotification = USBD_CDC_RESPONSE_AVAILABLE;
    l_tResponseAvailable.wValue        = 0U;
    l_tResponseAvailable.wIndex        = USBD_CDC_NOTIFICATION_IF;
    l_tResponseAvailable.wLength       = 0U;

    /* Send notification on EP3 (Interrupt-IN) */
    bStat = usbd_Epx_Send(USBD_EP3, sizeof(l_tResponseAvailable), (uint8_t *)&l_tResponseAvailable, NULL, _USBD_CDC_CB_Notification_ResponseAvailable);

    return bStat;
}

/**
  * @brief  Callback for USBD_CDC_Notification_ResponseAvailable
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP3.
  * @retval None
  */
static void _USBD_CDC_CB_Notification_ResponseAvailable(USBD_EPx epNum)
{
    if (gEpTransInfo[epNum].pfnCallback != NULL) {
        /* Callback */
        (*gEpTransInfo[epNum].pfnCallback)(epNum);
    }
}

/**
  * @brief  Send notification for SERIAL_STATE
  * @param  usStatBmap : SerialState bitmap.
  * @retval bStat : true or false.
  */
bool USBD_CDC_Notification_SerialState(uint16_t usStatBmap)
{
    bool            bStat = false;
    bmRequestType_t *p_bmRequestType;

    if ((gAbstractControlManagementFunctionalDescriptor1_0.bmCapabilities & USBD_CDC_BM_CAPA_LINE_SERIAL ) != 0U) {
        /* Parameter setup */
        p_bmRequestType                      = (bmRequestType_t *)&l_tSerialState.tNotice.bmRequestType;
        p_bmRequestType->Bit.Direction       = CbmRequestType_Direction_DtoH;
        p_bmRequestType->Bit.Req_Type        = CbmRequestType_Type_Class;
        p_bmRequestType->Bit.Recipient       = CbmRequestType_Recipient_Interface;
        l_tSerialState.tNotice.bNotification = USBD_CDC_RESPONSE_AVAILABLE;
        l_tSerialState.tNotice.wValue        = 0U;
        l_tSerialState.tNotice.wIndex        = USBD_CDC_NOTIFICATION_IF;
        l_tSerialState.tNotice.wLength       = 2U;
        l_tSerialState.usState               = usStatBmap;

        /* Send notification on EP3 (Interrupt-IN) */
        bStat = usbd_Epx_Send(USBD_EP3, sizeof(l_tSerialState), (uint8_t *)&l_tSerialState, NULL, _USBD_CDC_CB_Notification_SerialState);
    }

    return bStat;
}

/**
  * @brief  Callback for USBD_CDC_Notification_SerialState
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP3.
  * @retval None
  */
static void _USBD_CDC_CB_Notification_SerialState(USBD_EPx epNum)
{
    if (gEpTransInfo[epNum].pfnCallback != NULL) {
        /* Callback */
        (*gEpTransInfo[epNum].pfnCallback)(epNum);
    }
}

/**
  * @brief  Send GET_COMM_FEATURE information
  * @param  ulSize : Send data size.
  * @param  pucData : Send data buffer pointer.
  * @retval bStat : true or false.
  */
bool USBD_CDC_Send_GetCommFeature(uint32_t ulSize, uint8_t *pucData)
{
    bool bStat = false;

    if (ulSize == USBD_CDC_FEATURE_STATUS_SIZE) {
        /* Send data on EP0 (Control-IN) */
        bStat = usbd_Epx_Send(USBD_EP0, ulSize, pucData, NULL, NULL);
    }

    return bStat;
}

/**
  * @brief  Send GET_ENCAPSULATED_RESPONSE information
  * @param  ulSize : Send data size.
  * @param  pucData : Send data buffer pointer.
  * @retval bStat : true or false.
  */
bool USBD_CDC_Send_GetEncapsulatedResponse(uint32_t ulSize, uint8_t *pucData)
{
    bool bStat = false;

    if (ulSize == l_wEncapsulatedResponseSize) {
        /* Send data on EP0 (Control-IN) */
        bStat = usbd_Epx_Send(USBD_EP0, ulSize, pucData, NULL, NULL);
    }

    return bStat;
}

/**
  * @brief  Send GET_LINE_CODING information
  * @param  ulSize : Send data size.
  * @param  pucData : Send data buffer pointer.
  * @retval bStat : true or false.
  */
bool USBD_CDC_Send_GetLineCoding(uint32_t ulSize, uint8_t *pucData)
{
    bool bStat = false;

    if (ulSize == USBD_CDC_LINE_CODING_SIZE) {
        /* Send data on EP0 (Control-IN) */
        bStat = usbd_Epx_Send(USBD_EP0, ulSize, pucData, NULL, NULL);
    }

    return bStat;
}

/**
  * @brief  USB connect (Call at CONFIGURED state)
  * @param  None
  * @retval None
  */
void USBD_CDC_Connect(void)
{
    /* Check local USB status */
    if (l_bUsbStat == false) {
        l_bUsbStat = true;

        if (gCDC_CB.pfnConnStat != NULL) {
            /* Callback */
            (*gCDC_CB.pfnConnStat)(l_bUsbStat);
        }
    }
}

/**
  * @brief  USB disconnect (Call at Non-CONFIGURED State)
  * @param  None
  * @retval None
  */
void USBD_CDC_DisConnect(void)
{
    /* Check local USB status */
    if (l_bUsbStat == true) {
        l_bUsbStat = false;

        /* Cancellation to all transmission demand */
        usbd_Epx_Abort(USBD_EP1);
        usbd_Epx_Abort(USBD_EP2);
        usbd_Epx_Abort(USBD_EP3);

        if (gCDC_CB.pfnConnStat != NULL) {
            /* Callback */
            (*gCDC_CB.pfnConnStat)(l_bUsbStat);
        }
    }
}

/* EOC */
