/**
 *******************************************************************************
 * @file    hid_mouse.c
 * @brief   HID Mouse Sample Code for TOSHIBA Cortex M0/M3/M4  Device Series
 * @version V1.000
 * @date    2015/09/11
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 *
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

#include <string.h>
#include "usbd_hw.h"
#include "usbd_hw_com.h"
#include "usbd_var.h"
#include "usbd_device_request.h"
#include "usbd_typedefs.h"
#include "usbd_descriptor_hid.h"
#include "usbd_hid.h"
#include "usbd_trans.h"

/**
  * @brief Mouse demo structure of input report
  */
typedef struct {
    uint8_t     ucButton;           /**< Button       */
    uint8_t     ucX;                /**< X            */
    uint8_t     ucY;                /**< Y            */
    uint8_t     ucZ;                /**< Z (Not Used) */
} SampleInputReport_t;

/**
  * @brief Callback information
  */
HID_CB_t gSample_HID_CB;
/**
  * @brief Don't care status
  */
bool gbStatDC;

/**
  * @brief Mouse demo action pattern
  */
#define SAMPLEINPUTREPORT_NUM    (9U)
/**
  * @brief Repeat count
  */
static uint8_t l_ucSampleRepeatCnt;
/**
  * @brief Table index number
  */
static uint8_t l_ucSampleIndexNum;
/**
  * @brief Mouse demo repeat count table
  */
static const uint8_t l_aucSampleInputReportCount[SAMPLEINPUTREPORT_NUM] = {
    1U, 1U, 1U, 10U, 9U, 1U, 1U, 10U, 10U
};
/**
  * @brief Mouse demo input report table
  */
static const SampleInputReport_t l_tSampleInputReport[SAMPLEINPUTREPORT_NUM] = {
    {0x02U, 0x00U, 0x00U, 0x00U},   /* right key press     */
    {0x00U, 0x00U, 0x00U, 0x00U},   /* right key release   */
    {0x00U, 0xF6U, 0x00U, 0x00U},   /* left move 10 pixel  */
    {0x00U, 0x00U, 0x0AU, 0x00U},   /* down move 10 pixel  */
    {0x00U, 0xF6U, 0x00U, 0x00U},   /* left move 10 pixel  */
    {0x01U, 0x00U, 0x00U, 0x00U},   /* left key press      */
    {0x00U, 0x00U, 0x00U, 0x00U},   /* left key release    */
    {0x00U, 0x00U, 0xF6U, 0x00U},   /* up move 10 pixel    */
    {0x00U, 0x0AU, 0x00U, 0x00U}    /* right move 10 pixel */
};

/* Prototypes */
static void _Sample_ConnStat(bool);
static void _Sample_SendData(uint32_t, uint8_t*, uint16_t);

/******************************************************************************/

/**
  * @brief  HID sample main routine
  * @param  None
  * @retval None
  */
int main(void)
{
    /* Initialize repeat count */
    l_ucSampleRepeatCnt = 0U;
    /* Initialize table index number */
    l_ucSampleIndexNum  = 0U;
    
    /* Set HID initialize parameters */
    gSample_HID_CB.pfnConnStat     = _Sample_ConnStat;
    gSample_HID_CB.pfnSuspend      = NULL;
    gSample_HID_CB.pfnResume       = NULL;
    gSample_HID_CB.pfnGetReport    = NULL;
    gSample_HID_CB.pfnSetReport    = NULL;
    gSample_HID_CB.pfnGetIdle      = NULL;
    gSample_HID_CB.pfnSetIdle      = NULL;
    gSample_HID_CB.pfnGetProtocol  = NULL;
    gSample_HID_CB.pfnSetProtocol  = NULL;
    gSample_HID_CB.pfnSendData     = _Sample_SendData;
    gSample_HID_CB.pfnCtrlSendData = NULL;
    gSample_HID_CB.pfnCtrlRecvData = NULL;

    /* Initialize HID driver */
    usbd_hid_init(&gSample_HID_CB);

    /* Initialize driver works */
    usbd_initialize_standard_class_work_data();

    /* Initialize USB controller */
    Config_USB();
    
    /* Forever loop (Interrupt driven sample) */
    while(true);
}

/**
  * @brief  Callback for connect/disconnect
  * @param  bStat : Connect status.
  *   This parameter can be one of the following values :
  *      true(Connect), false(Disconnect)
  * @retval None
  */
static void _Sample_ConnStat(bool bStat)
{
    if (bStat == true) {
        /* Connect */
        /* Send input report */
        gbStatDC = USBD_HID_SendData(sizeof(SampleInputReport_t), (uint8_t *)&l_tSampleInputReport[l_ucSampleIndexNum]);
        /* No care status in sample */
    } else {
        /* Disconnect */
        /* Do nothing in sample */
    }
}

/**
  * @brief  Callback for send data
  * @param  ulSize : Send data size.
  * @param  pucData : Send data buffer pointer.
  * @param  usStat : Send status.
  *   This parameter can be one of the following values :
  *      EPSTAT_COMPLETE, EPSTAT_ABORT
  * @retval None
  */
static void _Sample_SendData(uint32_t ulSize, uint8_t *pucData, uint16_t usStat)
{
    uint16_t    i;
    
    if (usStat == EPSTAT_COMPLETE) {
        /* Update repeat count */
        if (++l_ucSampleRepeatCnt == l_aucSampleInputReportCount[l_ucSampleIndexNum]) {
            l_ucSampleRepeatCnt = 0U;
            
            /* Update table index number */
            if (++l_ucSampleIndexNum == SAMPLEINPUTREPORT_NUM) {
                l_ucSampleIndexNum = 0U;
            }
        }
        
        /* Wait for a while */
        for (i=0; i<5U; i++) {
            usbd_wait_1ms();
        }
        
        /* Send input report */
        gbStatDC = USBD_HID_SendData(sizeof(SampleInputReport_t), (uint8_t *)&l_tSampleInputReport[l_ucSampleIndexNum]);
        /* No care status in sample */
    }
    
    return;
}
#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/* EOC */
