/**
 *******************************************************************************
 * @file    usbd_descriptor_hid.c
 * @brief   HID Descriptor Code for TOSHIBA Cortex M0/M3/M4 Device Series
 * @version V1.000
 * @date    2015/09/11
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 *
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

#include "usbd_hw.h"
#include "usbd_var.h"
#include "usbd_device_request.h"
#include "usbd_typedefs.h"
#include "usbd_descriptor.h"
#include "usbd_descriptor_hid.h"

/* xxxxxA_B_C means A:Config B:Interface C:Endpoint */

/**
  * @brief Device Descriptor
  */
const DeviceDescriptor_t gDeviceDescriptor = {
    CbLength_Device,                /* bLength                      */
    CbDescripterType_Device,        /* bDescriptorType              */
    0x0200U,                        /* bcdUSB                       */
    0x00U,                          /* bDeviceClass                 */
    0x00U,                          /* bDeviceSubClass              */
    0x00U,                          /* bDeviceProtocol              */
    0x40U,                          /* bMaxPacketSize0              */
    0x0930U,                        /* idVendor                     */
    0x6514U,                        /* idProduct                    */
    0x0001U,                        /* bcdDevice(Low)               */
    0x01U,                          /* iManufacturer                */
    0x02U,                          /* iProduct                     */
    0x03U,                          /* iSerialNumber                */
    CbNumConfigrations              /* bNumConfigrations            */
};

/**
  * @brief Configuration Descriptor (Config:1)
  */
const ConfigDescriptor_t gConfigDescriptor1 = {
    CbLength_Configuration,         /* bLength                      */
    CbDescripterType_Configuration, /* bDescriptorType              */
    CwTotalLengthConf1,             /* wTotalLength                 */
    CbNumInterfacesConf1,           /* bNumInterfaces               */
    CbConfigurationValue1,          /* bConfigurationValue          */
    0x00U,                          /* iConfiguration               */
    0xC0U,                          /* bmAttributes                 */
    0x31U                           /* bMaxPower                    */
};

/**
  * @brief Interface Descriptor (Config:1, Interface:0)
  */
const InterfaceDescriptor_t gInterfaceDescriptor1_0 = {
    CbLength_Interface,             /* bLength                      */
    CbDescripterType_Interface,     /* bDescriptorType              */
    CbInterfaceNumber1_0,           /* bInterfaceNumber             */
    CbAlternateSetting1_0,          /* bAlternateSetting            */
    CbNumEndPoints1_0,              /* bNumEndpoints                */
    0x03U,                          /* bInterfaceClass              */
    0x00U,                          /* bInterfaceSubClass           */
    0x02U,                          /* bInterfaceProtocol           */
    0x00U                           /* iInterface                   */
};

/**
  * @brief Endpoint Descriptor (Config:1, Interface:0, Endpoint:3)
  */
const EndpointDescriptor_t gEndpointDescriptor1_0_3 = {
    CbLength_Endpoint,              /* bLength                      */
    CbDescripterType_Endpoint,      /* bDescriptorType              */
    0x83U,                          /* bEndPointAddress             */
    0x03U,                          /* bmAttributes                 */
    CwMaxPacketSize_EP3_INIT,       /* wMaxPacketSize               */
    0x0AU                           /* bInterval                    */
};

/**
  * @brief Device Qualifier Descriptor
  */
const DeviceQualifierDescriptor_t gDeviceQualifierDescriptor = {
    CbLength_DeviceQualifer,        /* bLength(10byte)              */
    CbDescripterType_DeviceQualifer,/* bDescriptorType              */
    0x0200U,                        /* bcdUSB                       */
    0x00U,                          /* bDeviceClass                 */
    0x00U,                          /* bDeviceSubClass              */
    0x00U,                          /* bDeviceProtocol              */
    0x40U,                          /* bMaxPacketSize0(64byte)      */
    CbNumConfigrations,             /* bNumConfiguration            */
    0x00U                           /* Reserve                      */
};

/**
  * @brief HID Descriptor
  */
const uint8_t gHIDDescriptor[CbLength_HIDDescriptor] = {
    CbLength_HIDDescriptor,         /* bLength                                        */
    CbDescripterType_HID,           /* bDescriptorType, HID = 0x21                    */
    0x10U,                          /* bcdHID, low byte                               */
    0x01U,                          /* bcdHID, high byte                              */
    0x00U,                          /* bCountryCode                                   */
    0x01U,                          /* bNumDescriptors                                */
    CbDescripterType_HIDReport,     /* bDescriptorType, HID Report = 0x22             */
    0x34U,                          /* wItemLength, low byte(total length of report)  */
    0x00U                           /* wItemLength, high byte(total length of report) */
};

/**
  * @brief HID Report Descriptor
  */
const uint8_t gHIDReportDescriptor[CbLength_HIDReportDescriptor] = {
    0x05U, 0x01U,                   /* Usage Page (Generic Desktop)         */
    0x09U, 0x02U,                   /* Usage (Mouse)                        */
    0xA1U, 0x01U,                   /* Collection (Application)             */
    0x09U, 0x01U,                   /*   Usage (Pointer)                    */
    0xA1U, 0x00U,                   /*   Collection (Physical)              */
    0x05U, 0x09U,                   /*     Usage Page (Buttons)             */
    0x19U, 0x01U,                   /*     Usage Minimum (01)               */
    0x29U, 0x03U,                   /*     Usage Maximum (03)               */
    0x15U, 0x00U,                   /*     Logical Minimum (0)              */
    0x25U, 0x01U,                   /*     Logical Maximum (1)              */
    0x95U, 0x03U,                   /*     Report Count (1)                 */
    0x75U, 0x01U,                   /*     Report Size (1)                  */
    0x81U, 0x02U,                   /*     Input (Data, Variable, Absolute) */
    0x95U, 0x01U,                   /*     Report Count (1)                 */
    0x75U, 0x05U,                   /*     Report Size (5)                  */
    0x81U, 0x01U,                   /*     Input (Constant) for padding     */
    0x05U, 0x01U,                   /*     Usage Page (Generic Desktop)     */
    0x09U, 0x30U,                   /*     Usage (X)                        */
    0x09U, 0x31U,                   /*     Usage (Y)                        */
    0x09U, 0x38U,                   /*     Usage (Z)                        */
    0x15U, 0x81U,                   /*     Logical Minimum (-127)           */
    0x25U, 0x7FU,                   /*     Logical Maximum (127)            */
    0x75U, 0x08U,                   /*     Report Size (8)                  */
    0x95U, 0x03U,                   /*     Report Count (2)                 */
    0x81U, 0x06U,                   /*     Input (Data, Variable, Relative) */
    0xC0U,                          /*   End Collection (Physical)          */
    0xC0U                           /* End Collection (Application)         */
};

/**
  * @brief Endpoint (Interface:0)
  */
const EndpointDescriptor_t *gEndpointDescriptor1_0[CbNumEndPoints1_0] = {
    &gEndpointDescriptor1_0_3,      /* Endpoint:3               */
};

/**
  * @brief Interface:0 (Config:1)
  */
const StructInterfaceDesc_t gStructInterfaceDesc1_0 = {
    &gInterfaceDescriptor1_0,       /* Interface:0              */
    gEndpointDescriptor1_0          /* Endpoint                 */
};

/**
  * @brief Configuration:1
  */
const StructConfigDesc_t gStructConfigDesc[CbNumConfigrations] = {
    &gConfigDescriptor1,            /* Configuration:1          */
    &gStructInterfaceDesc1_0,       /* Interface:0              */
    gStringDescMicro                /* String(Micro controller) */
};

/**
  * @brief Device
  */
const UserDescriptor_t gUserDescriptor = {
    &gDeviceDescriptor,             /* Device                   */
    gStructConfigDesc               /* Configuration:1          */
};

/**
  * @brief String Descriptor (LANGID)
  */
const StringDescriptorLanguageSupport_t gStringLanguageSupported = {
    CStringSize_LanguageSupport,    /* Size of Descriptor(byte) */
    CbDescripterType_String,        /* Descriptor Type          */
    CStringLaguageID_0
};

/**
  * @brief String Descriptor (Manufacturer)
  */
const uint8_t gStringDescManu[CStringSize_Manufacturer] = {
    CStringSize_Manufacturer,       /* Size of Descriptor(byte) */
    CbDescripterType_String,        /* Descriptor Type          */
    'T', 0x00U,
    'O', 0x00U,
    'S', 0x00U,
    'H', 0x00U,
    'I', 0x00U,
    'B', 0x00U,
    'A', 0x00U,
    ' ', 0x00U
};

/**
  * @brief String Descriptor (Product)
  */
const uint8_t gStringDescProd[CStringSize_Product] = {
    CStringSize_Product,            /* Size of Descriptor(byte) */
    CbDescripterType_String,        /* Descriptor Type          */
    'T', 0x00U,
    'S', 0x00U,
    'B', 0x00U,
    '-', 0x00U,
    'M', 0x00U,
    'C', 0x00U,
    'U', 0x00U,
    ' ', 0x00U
};

/**
  * @brief String Descriptor (Serial number)
  */
const uint8_t gStringDescSN[CStringSize_SerialNumber] = {
    CStringSize_SerialNumber,       /* Size of Descriptor(byte) */
    CbDescripterType_String,        /* Descriptor Type          */
    '0', 0x00U,
    '1', 0x00U,
    '2', 0x00U,
    '3', 0x00U,
    '4', 0x00U,
    '5', 0x00U,
    '6', 0x00U,
    '7', 0x00U
};

/**
  * @brief String Descriptor (Micro controller)
  * @note If support iConfiguration or iInterface, use this data.
  */
const uint8_t gStringDescMicro[CStringSize_MicroController] = {
    CStringSize_MicroController,    /* Size of Descriptor(byte) */
    CbDescripterType_String,        /* Descriptor Type          */
    'T', 0x00U,
    'S', 0x00U,
    'B', 0x00U,
    '-', 0x00U,
    'C', 0x00U,
    'o', 0x00U,
    'r', 0x00U,
    't', 0x00U,
    'e', 0x00U,
    'x', 0x00U,
    '-', 0x00U,
    '-', 0x00U,
    'U', 0x00U,
    'S', 0x00U,
    'B', 0x00U
};

/**
  * @brief String Descriptor(s)
  */
const uint8_t *gStringDescriptor[CbNumStringDescriptor] = {
    (const uint8_t *) &gStringLanguageSupported,    /* LANGID           */
    gStringDescManu,                                /* Manufacturer     */
    gStringDescProd,                                /* Product          */
    gStringDescSN,                                  /* Serial number    */
    gStringDescMicro                                /* Micro controller */
};

/* EOC */