/**
 *******************************************************************************
 * @file    usbd_hid.c
 * @brief   HID Function Code for TOSHIBA Cortex M0/M3/M4 Device Series
 * @version V1.000
 * @date    2015/09/11
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 *
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

#include <string.h>
#include "usbd_hw.h"
#include "usbd_device_request.h"
#include "usbd_var.h"
#include "usbd_hid.h"
#include "usbd_trans.h"
#include "usbd_descriptor_hid.h"

/**
  * @brief Callback information
  */
HID_CB_t gHID_CB;

/**
  * @brief USB connect status
  */
static bool l_bUsbStat;
/**
  * @brief Protocol buffer
  */
static uint32_t l_ulProtocol;
/**
  * @brief Protocol buffer pointer
  */
static uint8_t *l_pucProtocol = (uint8_t *)&l_ulProtocol;
/**
  * @brief Idle rate buffer
  */
static uint8_t l_aucIdleRate[USBD_HID_REPORT_ID_NUM];

/* Prototypes */
static void _USBD_HID_CB_SendData(USBD_EPx);
static void _USBD_HID_CB_CtrlSendData(USBD_EPx);
static void _USBD_HID_CB_CtrlRecvData(USBD_EPx);
static bool _USBD_HID_GetReport(uint16_t, uint16_t);
static bool _USBD_HID_SetReport(uint16_t, uint16_t);
static bool _USBD_HID_GetIdle(uint16_t);
static bool _USBD_HID_SetIdle(uint16_t);
static bool _USBD_HID_GetProtocol(void);
static bool _USBD_HID_SetProtocol(uint16_t);

/**
  * @brief Endpoint information
  */
extern EpTransInfo_t gEpTransInfo[CNumEndpointSupport];

/******************************************************************************/

/**
  * @brief  Initialize HID driver
  * @param  p_hid_cb : HID callback information pointer.
  * @retval None
  */
void usbd_hid_init(HID_CB_t *p_hid_cb)
{
    uint8_t ucCnt;

    /* Set of callback functions */
    gHID_CB.pfnConnStat     = p_hid_cb->pfnConnStat;
    gHID_CB.pfnResume       = p_hid_cb->pfnResume;
    gHID_CB.pfnSuspend      = p_hid_cb->pfnSuspend;
    gHID_CB.pfnGetReport    = p_hid_cb->pfnGetReport;
    gHID_CB.pfnSetReport    = p_hid_cb->pfnSetReport;
    gHID_CB.pfnGetIdle      = p_hid_cb->pfnGetIdle;
    gHID_CB.pfnSetIdle      = p_hid_cb->pfnSetIdle;
    gHID_CB.pfnGetProtocol  = p_hid_cb->pfnGetProtocol;
    gHID_CB.pfnSetProtocol  = p_hid_cb->pfnSetProtocol;
    gHID_CB.pfnSendData     = p_hid_cb->pfnSendData;
    gHID_CB.pfnCtrlSendData = p_hid_cb->pfnCtrlSendData;
    gHID_CB.pfnCtrlRecvData = p_hid_cb->pfnCtrlRecvData;

    /* Initialization of an internal variable */
    l_bUsbStat     = false;
    *l_pucProtocol = (uint8_t)USBD_HID_REPORT_PROTOCOL;

    for(ucCnt=0U; ucCnt<USBD_HID_REPORT_ID_NUM; ucCnt++)
    {
        l_aucIdleRate[ucCnt] = 0U;
    }

    return;
}

/**
  * @brief  Class request process of HID
  * @param  request : Index of request.
  * @retval bStat : true or false.
  */
bool usbd_interrupt_HID_req(uint32_t request)
{
    bool bStat;

    if ((f_bmDirection == CbmRequestType_Direction_DtoH)
     && (g_RequestType == CbmRequestType_Type_Class)
     && (g_Recipient == CbmRequestType_Recipient_Interface)) {
        switch(request) {
        case USBD_HID_GET_REPORT:
            /* GET_REPORT */
            bStat = _USBD_HID_GetReport(g_wValue, g_wLength);
            break;
        case USBD_HID_GET_IDLE:
            /* GET_IDLE */
            bStat = _USBD_HID_GetIdle(g_wValue);
            break;
        case USBD_HID_GET_PROTOCOL:
            /* GET_PROTOCOL */
            bStat = _USBD_HID_GetProtocol();
            break;
        default:
            /* Not supported request */
            bStat = false;
            break;
        }
    } else if ((f_bmDirection == CbmRequestType_Direction_HtoD)
     && (g_RequestType == CbmRequestType_Type_Class)
     && (g_Recipient == CbmRequestType_Recipient_Interface)) {
        switch(request) {
        case USBD_HID_SET_REPORT:
            /* SET_REPORT */
            bStat = _USBD_HID_SetReport(g_wValue, g_wLength);
            break;
        case USBD_HID_SET_IDLE:
            /* SET_IDLE */
            bStat = _USBD_HID_SetIdle(g_wValue);
            break;
        case USBD_HID_SET_PROTOCOL:
            /* SET_PROTOCOL */
            bStat = _USBD_HID_SetProtocol(g_wValue);
            break;
        default:
            /* Not supported request */
            bStat = false;
            break;
        }
    } else {
        /* Not supported request */
        bStat = false;
    }

    return bStat;
}

/**
  * @brief  Send data on interrupt IN endpoint
  * @param  ulSize : Send data size.
  * @param  pucData : Send data buffer pointer.
  * @retval bStat : true or false.
  */
bool USBD_HID_SendData(uint32_t ulSize, uint8_t *pucData)
{
    bool bStat;

    /* Send data on EP3 (Interrupt-IN) */
    bStat = usbd_Epx_Send(USBD_EP3, ulSize, pucData, NULL, _USBD_HID_CB_SendData);

    return bStat;
}

/**
  * @brief  Callback for USBD_HID_SendData
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP3.
  * @retval None
  */
static void _USBD_HID_CB_SendData(USBD_EPx epNum)
{
    if (gHID_CB.pfnSendData != NULL) {
        /* Callback */
        (*gHID_CB.pfnSendData)(gEpTransInfo[epNum].ulTransSize, gEpTransInfo[epNum].pucBuf, gEpTransInfo[epNum].usEpStat);
    }
}

/**
  * @brief  Send data on control endpoint
  * @param  ulSize : Send data size.
  * @param  pucData : Send data buffer pointer.
  * @retval bStat : true or false.
  */
bool USBD_HID_CtrlSendData(uint32_t ulSize, uint8_t *pucData)
{
    bool bStat;

    /* Send data on EP0 (Control-IN) */
    bStat = usbd_Epx_Send(USBD_EP0, ulSize, pucData, NULL, _USBD_HID_CB_CtrlSendData);

    return bStat;
}

/**
  * @brief  Callback for USBD_HID_CtrlSendData
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP0.
  * @retval None
  */
static void _USBD_HID_CB_CtrlSendData(USBD_EPx epNum)
{
    if (gHID_CB.pfnCtrlSendData != NULL) {
        /* Callback */
        (*gHID_CB.pfnCtrlSendData)(gEpTransInfo[epNum].ulTransSize, gEpTransInfo[epNum].pucBuf, gEpTransInfo[epNum].usEpStat);
    }
}

/**
  * @brief  Receive data on control endpoint
  * @param  ulSize : Receive data size.
  * @param  pucData : Receive data buffer pointer.
  * @retval bStat : true or false.
  */
bool USBD_HID_CtrlRecvData(uint32_t ulSize, uint8_t *pucData)
{
    bool bStat;

    /* Receive data on EP0 (Control-OUT) */
    bStat = usbd_Epx_Recv(USBD_EP0, ulSize, pucData, NULL, _USBD_HID_CB_CtrlRecvData);

    return bStat;
}

/**
  * @brief  Callback for USBD_HID_CtrlRecvData
  * @param  epNum : Endpoint number.
  *   This parameter is only USBD_EP0.
  * @retval None
  */
static void _USBD_HID_CB_CtrlRecvData(USBD_EPx epNum)
{
    if (gHID_CB.pfnCtrlRecvData != NULL) {
        /* Callback */
        (*gHID_CB.pfnCtrlRecvData)(gEpTransInfo[epNum].ulTransSize, gEpTransInfo[epNum].pucBuf, gEpTransInfo[epNum].usEpStat);
    }
}

/**
  * @brief  Received GET_REPORT
  * @param  usValue : Request value.
  * @param  usLength : Request length.
  * @retval bStat : true or false.
  */
static bool _USBD_HID_GetReport(uint16_t usValue, uint16_t usLength)
{
    bool    bStat = false;
    uint8_t ucReportType;
    uint8_t ucReportID;

    if (g_wIndex == CbInterfaceNumber1_0) {
        /* Get report type */
        ucReportType = (uint8_t)((usValue & 0xFF00U) >> 8U);

        /* Check report type */
        if (USBD_HID_REPORT_TYPE_INPUT != ucReportType) {
            return false;
        }

        /* Get report ID */
        ucReportID = (uint8_t)(usValue & 0x00FFU);

        if (gHID_CB.pfnGetReport != NULL) {
            /* Callback */
            bStat = (*gHID_CB.pfnGetReport)(ucReportID, usLength);
        } else {
            bStat = false;
        }
    }

    return bStat;
}

/**
  * @brief  Received SET_REPORT
  * @param  usValue : Request value.
  * @param  usLength : Request length.
  * @retval bStat : true or false.
  */
static bool _USBD_HID_SetReport(uint16_t usValue, uint16_t usLength)
{
    bool    bStat = false;
    uint8_t ucReportType;
    uint8_t ucReportID;

    if (g_wIndex == CbInterfaceNumber1_0) {
        /* Get report type */
        ucReportType = (uint8_t)((usValue & 0xFF00U) >> 8U);

        /* Check report type */
        if (USBD_HID_REPORT_TYPE_OUTPUT != ucReportType) {
            return false;
        }

        /* Get report ID */
        ucReportID = (uint8_t)(usValue & 0x00FFU);

        if (gHID_CB.pfnSetReport != NULL) {
            /* Callback */
            bStat = (*gHID_CB.pfnSetReport)(ucReportID, usLength);
        } else {
            bStat = false;
        }
    }

    return bStat;
}

/**
  * @brief  Received GET_IDLE
  * @param  usValue : Request value.
  * @retval bStat : true or false.
  */
static bool _USBD_HID_GetIdle(uint16_t usValue)
{
    bool    bStat = false;
    uint8_t ucValue_H;
    uint8_t ucReportID;

    ucValue_H = (uint8_t)((usValue & 0xFF00U) >> 8U);
 
    if ((ucValue_H == USBD_HID_GET_IDLE_VALUE_H)
     && (g_wIndex == CbInterfaceNumber1_0)
     && (g_wLength == USBD_HID_GET_IDLE_LENGTH)) {
        /* Get report ID */
        ucReportID = (uint8_t)(usValue & 0x00FFU);

        if (gHID_CB.pfnGetIdle != NULL) {
            /* Get idle rate (Callback) */
            l_aucIdleRate[ucReportID] = (*gHID_CB.pfnGetIdle)(ucReportID);
        }

        /* Send idle rate on EP0 (Control-IN) */
        bStat = usbd_Epx_Send(USBD_EP0, USBD_HID_GET_IDLE_LENGTH, &l_aucIdleRate[ucReportID], NULL, NULL);
    }

    return bStat;
}

/**
  * @brief  Received SET_IDLE
  * @param  usValue : Request value.
  * @retval bStat : true or false.
  */
static bool _USBD_HID_SetIdle(uint16_t usValue)
{
    bool    bStat = false;
    uint8_t ucReportID;
    uint8_t ucDuration;

    if ((g_wLength == USBD_HID_SET_IDLE_LENGTH)
     && (g_wIndex == CbInterfaceNumber1_0)) {
        /* Get report ID */
        ucReportID = (uint8_t)(usValue & 0x00FFU);

        /* Get duration */
        ucDuration = (uint8_t)((usValue & 0xFF00) >> 8U);

        if (gHID_CB.pfnSetIdle != NULL) {
            /* Callback */
            (*gHID_CB.pfnSetIdle)(ucReportID, ucDuration);
        } else {
            /* Manage the duration in a HID function */
            l_aucIdleRate[ucReportID] = ucDuration;
        }
        
        bStat = true;
    }

    return bStat;
}

/**
  * @brief  Received GET_PROTOCOL
  * @param  None
  * @retval bStat : true or false.
  */
static bool _USBD_HID_GetProtocol(void)
{
    bool bStat = false;

    if ((g_wValue == USBD_HID_GET_PROTOCOL_VALUE)
     && (g_wIndex == CbInterfaceNumber1_0)
     && (g_wLength == USBD_HID_GET_PROTOCOL_LENGTH)) {
        if (gHID_CB.pfnGetProtocol != NULL) {
            /* Get protocol (Callback) */
            *l_pucProtocol = (*gHID_CB.pfnGetProtocol)();
        }

        /* Send protocol on EP0 (Control-IN) */
        bStat = usbd_Epx_Send(USBD_EP0, USBD_HID_GET_PROTOCOL_LENGTH, l_pucProtocol, NULL, NULL);
    }
    
    return bStat;
}

/**
  * @brief  Received SET_PROTOCOL
  * @param  usValue : Request value.
  * @retval bStat : true or false.
  */
static bool _USBD_HID_SetProtocol(uint16_t usValue)
{
    bool bStat = false;
    
    if ((g_wLength == USBD_HID_SET_PROTOCOL_LENGTH)
     && (g_wIndex == CbInterfaceNumber1_0)) {
        if (gHID_CB.pfnSetProtocol != NULL) {
            /* Callback */
            (*gHID_CB.pfnSetProtocol)(usValue);
        } else  {
            /* Manage the protocol in a HID function */
            *l_pucProtocol = (uint8_t)(usValue & 0x00FF);
        }
        
        bStat = true;
    }

    return bStat;
}

/**
  * @brief  USB connect (Call at CONFIGURED state)
  * @param  None
  * @retval None
  */
void USBD_HID_Connect(void)
{
    /* Check local USB status */
    if (l_bUsbStat == false) {
        l_bUsbStat = true;
        
        if (gHID_CB.pfnConnStat != NULL) {
            /* Callback */
            (*gHID_CB.pfnConnStat)(l_bUsbStat);
        }
    }
}

/**
  * @brief  USB disconnect (Call at Non-CONFIGURED State)
  * @param  None
  * @retval None
  */
void USBD_HID_DisConnect(void)
{
    /* Check local USB status */
    if (l_bUsbStat == true) {
        l_bUsbStat = false;
        
        /* Set current protocol */
        *l_pucProtocol = (uint8_t)USBD_HID_REPORT_PROTOCOL;
        
        /* Cancellation to a transmission demand */
        usbd_Epx_Abort(USBD_EP3);

        if (gHID_CB.pfnConnStat != NULL) {
            /* Callback */
            (*gHID_CB.pfnConnStat)(l_bUsbStat);
        }
    }
}

/* EOC */
