/**
 *******************************************************************************
 * @file    usbd_descriptor.h
 * @brief   USB Standard Descriptor Header for TOSHIBA Cortext M0/M3 Device Series
 * @version V1.000
 * @date    2015/09/11
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 *
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

#ifndef _USBD_DESCRIPTOR_H_
#define _USBD_DESCRIPTOR_H_

/* for bLength */
#define CbLength_Device                             0x12U
#define CbLength_DeviceQualifer                     0x0AU
#define CbLength_Configuration                      0x09U
#define CbLength_OtherSpeedConfiguration            0x07U
#define CbLength_Interface                          0x09U
#define CbLength_Endpoint                           0x07U

/* for bDescripterType */
#define CbDescripterType_Device                     0x01U
#define CbDescripterType_Configuration              0x02U
#define CbDescripterType_String                     0x03U
#define CbDescripterType_Interface                  0x04U
#define CbDescripterType_Endpoint                   0x05U
#define CbDescripterType_DeviceQualifer             0x06U
#define CbDescripterType_OtherSpeedConfiguration    0x07U
#define CbDescripterType_InterfacePower             0x08U

/* for bmAttributes.RemoteWakup */
#define CRemoteWakeup_OFF                   0U      /* Bus Power    */
#define CRemoteWakeup_ON                    1U      /* Self Power   */

/* for bmAttributes.Self_powered */
#define CSelf_powered_OFF                   0U      /* Disable RemoteWakeup */
#define CSelf_powered_ON                    1U      /* Enable RemoteWakeup  */

/* for Endpoint Descriptor      */
#define CbEndPointAddress_MASK              0x7FU

/* for Interface Descriptor     */
#define CbConfigurationValue_Init           0x00U

/* Device (full-speed information)
    Standard Device Descriptor  */
typedef struct {
    uint8_t bLength;            /* size of this descriptor                      */
    uint8_t bDescripterType;    /* Device Descriptor Type                       */
    uint16_t bcdUSB;            /* USB Specification Release Number             */
    uint8_t bDeviceClass;       /* Class code                                   */
    uint8_t bDeviceSubClass;    /* Subclass code                                */
    uint8_t bDeviceProtocol;    /* Protocol code                                */
    uint8_t bMaxPacketSize0;    /* Maximum packet size for EP0                  */
    uint16_t idVendor;          /* Vendor ID                                    */
    uint16_t idProduct;         /* Product ID                                   */
    uint16_t bcdDevice;         /* Device release number(BCD)                   */
    uint8_t iManufacture;       /* Index of string descriptor(manufacturer)     */
    uint8_t iProduct;           /* Index of string descriptor(product)          */
    uint8_t iSerialNumber;      /* index of string descriptor(serial numuber)   */
    uint8_t bNumConfiguration;  /* Number of possible configurations            */
} DeviceDescriptor_t;

/* Device_Qualifier (high-speed information)
     Device_Qualifier Descriptor        */
typedef struct {
    uint8_t bLength;            /* size of this descriptor              */
    uint8_t bDescriptorType;    /* Device Qualifier Type(Const)         */
    uint16_t bcdUSB;            /* USB spec. versioon(BCD)              */
    uint8_t bDeviceClass;       /* Class Code                           */
    uint8_t bDeviceSubClass;    /* SubClass Code                        */
    uint8_t bDeviceProtocol;    /* Protocol Code                        */
    uint8_t bMaxPacketSize0;    /* Maximum packet size for other speed  */
    uint8_t bNumConfigurations; /* Number of Other-speed Configurations */
    uint8_t bReserved;          /* Reserved                             */
} DeviceQualifierDescriptor_t;

/* Configuration (full-speed information)
     Standard Configuration Descriptor  */
typedef struct {
    uint8_t bLength;            /* size of this descriptor              */
    uint8_t bDescripterType;    /* CONFIGURATION Descriptor Type(Const) */
    uint16_t wTotalLength;      /* Total length of data returned        */
    uint8_t bNumInterfaces;     /* Nubmer of interface supported        */
    uint8_t bConfigurationValue;        /* Value to use as an argument  */
    uint8_t iConfiguration;     /* Index ot string descriptor(this)     */
    uint8_t bmAttributes;       /* Configuration characteristics        */
    uint8_t bMaxPower;          /* Maximum power consumption            */
} ConfigDescriptor_t;

typedef struct {
    uint8_t Reserved0:5;        /* <4:0>: reset to zero */
    uint8_t RemoteWakeup:1;     /* <5>  :               */
    uint8_t SelfPowered:1;      /* <6>  :               */
    uint8_t Reserved1:1;        /* <7>  : set to one    */
} ConfigDescriptor_bmAttributes_t;

/* Other_Speed_Configuration  (high-speed information)
    Other_Speed_Configuration Descriptor    */
typedef struct {
    uint8_t bLength;            /* size of this descriptor                      */
    uint8_t bDescriptorType;    /* Other_speed_Configuration(Const)             */
    uint16_t wTotalLength;      /* Total length of data returned                */
    uint8_t bNumInterfaces;     /* Number of interfaces supported               */
    uint8_t bConfigurationValue;        /* Value to use to select configuration */
    uint8_t iConfiguraion;      /* index of string descriptor                   */
    uint8_t bmAttributes;       /* Same as Configuration descriptor             */
    uint8_t bMaxPower;          /* Same as Configuration descriptor             */
} OtherSpeedConfigrationDescriptor_t;

/* Interface
    Standard Interface Descriptor   */
typedef struct {
    uint8_t bLength;            /* size of this descriptor              */
    uint8_t bDescripterType;    /* INTERFACE Descriptor Type(Const)     */
    uint8_t bInterfaceNumber;   /* Nubmer of this interface             */
    uint8_t bAlternateSetting;  /* Value used to select this alternate  */
    uint8_t bNumEndPoints;      /* Number of endpoints used             */
    uint8_t bInterfaceClass;    /* Class code                           */
    uint8_t bInterfaceSubClass; /* Sub Class code                       */
    uint8_t bInterfaceProtocol; /* Protocol code                        */
    uint8_t iInterface;         /* Index of string descriptor           */
} InterfaceDescriptor_t;

/* Endpoint
     Standard Endpoint Descriptor   */
typedef struct {
    uint8_t bLength;            /* size of this descriptor             */
    uint8_t bDescripterType;    /* ENDPOINT Descriptor Type(Const)     */
    uint8_t bEndPointAddress;   /* The address of the endpoint         */
    uint8_t bmAttributes;       /* the endopoint's attributes          */
    uint16_t wMaxPacketSize;    /* Maximu packet size this endpoint    */
    uint8_t bInterval;          /* Interval for polling endpoint       */
} EndpointDescriptor_t;

typedef struct {
    uint8_t TransferType:2;     /* <1:0>                */
    uint8_t SynchronizationType:2;      /* <3:2>        */
    uint8_t UsageType:2;        /* <5:4>                */
    uint8_t Reserved:2;         /* <7:6> zero           */
} EndpointDescriptor_bmAttributes_t;

typedef struct {
    uint16_t packet_size:11;    /* <10:0>               */
    uint16_t add_microframe:2;  /* <12:11>              */
    uint16_t f:3;               /* <15:13> zero         */
} EndpointDescriptor_wMaxPacketSize_t;

/* String  */
typedef struct {
    uint8_t bLength;            /* size of this descriptor */
    uint8_t bDescripterType;
} StringDescriptor_t;

#endif  /* _USBD_DESCRIPTOR_H_ */
