/**
 *******************************************************************************
 * @file    usbd_device_request.c
 * @brief   USB Standard Request Code for TOSHIBA 'TMPM066' Device Series
 *          (includes Class Specific Process)
 * @version V0.100
 * @date    2015/09/11
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 *
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */
#include <string.h>
#include "usbd_hw.h"
#include "usbd_var.h"
#include "usbd_device_request.h"
#include "usbd_typedefs.h"
#include "usbd_trans.h"
#include "usbd_descriptor.h"

#ifdef _USBD_HID_
#include "usbd_hid.h"
#include "usbd_descriptor_hid.h"
#endif
#ifdef _USBD_CDC_
#include "usbd_cdc.h"
#include "usbd_descriptor_cdc.h"
#endif
#ifdef _USBD_MSC_
#include "usbd_msc.h"
#include "usbd_descriptor_msc.h"
#endif

/* used at usbd_get_descriptor   */
static uint8_t sConfigDescData[CwTotalLengthConf1];

/* used at usbd_get_status  */
static GetStatus_t sGetStatus;
 /* for GetStatus               */
#define sGetStatusDevice            sGetStatus.Data
#define sGetStatusInterface         sGetStatus.Data
#define sGetStatusEndpoint          sGetStatus.Data
#define fSelfPowered                sGetStatus.Device.SelfPowered
#define fRemoteWakeup               sGetStatus.Device.RemoteWakeup
#define fHalt                       sGetStatus.Endpoint.Halt

#define CsGetStatus_INIT            0x0000U
#define CsGetStatusDevice_INIT      CsGetStatus_INIT
#define CsGetStatusInterface_DATA   CsGetStatus_INIT
#define CsGetStatusEndpoint_INIT    CsGetStatus_INIT

#define CfSelfPowered_OFF           CSelf_powered_OFF
#define CfSelfPowered_ON            CSelf_powered_ON
#define CfRemoteWakeup_OFF          CRemoteWakeup_OFF
#define CfRemoteWakeup_ON           CRemoteWakeup_ON
#define CfHalt_OFF                  CEPxStallFeature_OFF
#define CfHalt_ON                   CEPxStallFeature_ON

/* used at usbd_get_interface               */
static uint32_t sbAlternateSetting;

/* Endpoint Information in usbd_trans.c */
extern EpTransInfo_t gEpTransInfo[CNumEndpointSupport];

/* UDC State parameter */
static uint32_t s_Current_Config;
static uint32_t s_Current_Interface;
volatile static uint32_t s_Current_Alternate;
static uint32_t s_Buf_Current_Config;
static uint32_t s_Buf_Current_Interface;
volatile static uint32_t s_Buf_Current_Alternate;

static bool usbd_clear_feature(void);
static bool usbd_clear_feature_device(void);
static bool usbd_clear_feature_endpoint(void);

static bool usbd_get_configuration(void);

static bool usbd_get_descriptor(void);
static bool usbd_get_descriptor_device(uint8_t index);
static bool usbd_get_descriptor_config(uint8_t index);
static bool usbd_get_descriptor_string(uint8_t index);

static bool usbd_get_interface(void);

static bool usbd_get_status(void);
static bool usbd_get_status_device(void);
static bool usbd_get_status_interface(void);
static bool usbd_get_status_endpoint(void);

static bool usbd_set_address(void);

static bool usbd_set_configuration(void);

static bool usbd_set_feature(void);
static bool usbd_set_feature_normal(void);
static bool usbd_set_feature_device(void);
static bool usbd_set_feature_endpoint(void);

static bool usbd_set_interface(void);
static bool usbd_check_xxx_feature_device(void);
static bool usbd_check_xxx_feature_endpoint(USBD_EPx * endpoint);
static bool usbd_check_xxx_interface(const InterfaceDescriptor_t ** pp_interface);
static void usbd_set_xxx_reset_endpoint(uint8_t num_endpoint);

#ifdef _USBD_HID_
static bool usbd_get_descriptor_HID(uint8_t);
static bool usbd_get_descriptor_HID_report(uint8_t);
#endif

/**
  * @brief  Standard request process after gotten setup package
  * @param  request : Index of standard request.
  * @retval status : true or false.
  */
bool usbd_interrupt_setup_standard(uint32_t request)
{
    bool status = true;

    switch (request) {
    case CbRequest_GET_STATUS:
        status = usbd_get_status();
        break;
    case CbRequest_CLEAR_FEATURE:
        status = usbd_clear_feature();
        break;

    case CbRequest_SET_FEATURE:
        status = usbd_set_feature();
        break;

    case CbRequest_SET_ADDRESS:
        status = usbd_set_address();
        break;

    case CbRequest_GET_DESCRIPTOR:
        status = usbd_get_descriptor();
        break;

    case CbRequest_GET_CONFIGURATION:
        status = usbd_get_configuration();
        break;

    case CbRequest_SET_CONFIGURATION:
        status = usbd_set_configuration();
        break;

    case CbRequest_GET_INTERFACE:
        status = usbd_get_interface();
        break;

    case CbRequest_SET_INTERFACE:
        status = usbd_set_interface();
        break;

    case CbRequest_SYNCH_FRAME:
        break;

    default:
        status = false;
        break;
    }

    return status;
}


/**
  * @brief  Initialize variables used in USBD
  * @param  None
  * @retval None
  */
void usbd_initialize_standard_class_work_data(void)
{
    const ConfigDescriptor_t *config;
    ConfigDescriptor_bmAttributes_t *attribute;

    gUDC2Addr.All = CgUD2ADDR_INIT;
    gUDC2AddrBuf.All = CgUD2ADDR_INIT;

    fEP0StallFeature = CEPxStallFeature_OFF;
    fEP1StallFeature = CEPxStallFeature_OFF;
    fEP2StallFeature = CEPxStallFeature_OFF;
    fEP3StallFeature = CEPxStallFeature_OFF;

    gEP0Payload_Size = CwMaxPacketSize_EP0;
    gEP1Payload_Size = CwMaxPacketSize_EP1_INIT;
    gEP2Payload_Size = CwMaxPacketSize_EP2_INIT;
    gEP3Payload_Size = CwMaxPacketSize_EP3_INIT;

    gEPxConfigArray[USBD_EP1] = 0x88U;   /* EP1 as BULK IN for CDC,MSC */
    gEPxConfigArray[USBD_EP2] = 0x08U;   /* EP2 as BULK OUT for CDC,MSC */
    gEPxConfigArray[USBD_EP3] = 0x8CU;   /* EP3 as INTERRUPT IN for CDC,HID */

    s_Buf_Current_Config = CbConfigurationValue_Init;
    s_Buf_Current_Interface = CbInterfaceNumber_Init;
    s_Buf_Current_Alternate = CbAlternateSetting_Init;

    s_Current_Config = s_Buf_Current_Config;
    s_Current_Interface = s_Buf_Current_Interface;
    s_Current_Alternate = s_Buf_Current_Alternate;

    g_USB_Stage = IDLE_STAGE;

    memset(gEpTransInfo, 0x00U, sizeof(gEpTransInfo));

    /* make status device result    */
    sGetStatusDevice = CsGetStatusDevice_INIT;
    config = gStructConfigDesc[CbConfigurationValue1 - 1].p_config;
    attribute = (ConfigDescriptor_bmAttributes_t *) config->bmAttributes;
    fSelfPowered = attribute->SelfPowered;
    fRemoteWakeup = attribute->RemoteWakeup;

    return;
}

/**
  * @brief  Get_Status request management
  * @param  None
  * @retval status : true or false.
  */
static bool usbd_get_status(void)
{
    bool status;

    /* check common data    */
    if ((f_bmDirection == CbmRequestType_Direction_HtoD) ||
        (g_RequestType >= CbmRequestType_Type_Error) ||
        (g_wValue != CwValue_GetStatus) || (g_wLength != CwLength_GetStatus)) {
        return false;
    } else {
        /* Do nothing */
    }

    g_USB_Stage = DATA_STAGE;

    status = true;
    switch (g_Recipient) {
    case CbmRequestType_Recipient_Device:
        if (g_wIndex != CwIndex_ZERO) {
            status = false;
        } else {
            status = usbd_get_status_device();
        }
        break;

    case CbmRequestType_Recipient_Interface:
        if ((g_wIndex & CwIndex_RESERVED_MASK_INTERFACE) != CwIndex_RESERVED_INTERFACE) {
            status = false;
        } else {
            status = usbd_get_status_interface();
        }
        break;

    case CbmRequestType_Recipient_Endpoint:
        if ((g_wIndex & CwIndex_RESERVED_MASK_ENDPOINT) != CwIndex_RESERVED_ENDPOINT) {
            status = false;
        } else {
            status = usbd_get_status_endpoint();
        }
        break;

    default:
        status = false;
        break;
    }

    if (status == true) {
        status = usbd_Epx_Send(USBD_EP0, g_wLength, (uint8_t *)&sGetStatusDevice, NULL, NULL);
    } else {
        /* Do nothing */
    }

    return status;
}

/**
  * @brief  Get_Status Request management: when Recipient is device
  * @param  None
  * @return true or false.
  */
static bool usbd_get_status_device(void)
{
    const ConfigDescriptor_t *config;
    ConfigDescriptor_bmAttributes_t *attribute;

    switch (gDeviceState) {
    case CDEVICE_STATE_DEFAULT:
        return false;

    case CDEVICE_STATE_ADDRESSED:
        break;

    case CDEVICE_STATE_CONFIGURED:
        break;

    default:
        return false;
    }

    /* make status data     */
    sGetStatusDevice = CsGetStatus_INIT;

    config = gStructConfigDesc[CbConfigurationValue1 - 1].p_config;
    attribute = (ConfigDescriptor_bmAttributes_t *) config->bmAttributes;
    fSelfPowered = attribute->SelfPowered;
    fRemoteWakeup = attribute->RemoteWakeup;

    return true;
}

/**
  * @brief  Get_Status Request management: when Recipient is interface
  * @param  None
  * @return true or false.
  */
static bool usbd_get_status_interface(void)
{
    const ConfigDescriptor_t *p_config;

    wIndex_t *wIndex_work;
    uint8_t interface_number;

    switch (gDeviceState) {
    case CDEVICE_STATE_DEFAULT:
        return false;

    case CDEVICE_STATE_ADDRESSED:
        return false;           /* set EP0 to STALL     */

    case CDEVICE_STATE_CONFIGURED:
        break;

    default:
        return false;
    }

    wIndex_work = (wIndex_t *) & g_wIndex;
    interface_number = (uint8_t) wIndex_work->Interface.Number;

    if (s_Current_Config == 0) {
        return false;
    } else {
        p_config = gUserDescriptor.p_struct_config[s_Current_Config - 1]->p_config;
    }

    if (interface_number >= p_config->bNumInterfaces) {
        return false;
    } else {
        /* Do nothing */
    }

    /* make status data     */
    sGetStatusInterface = CsGetStatusInterface_DATA;

    return true;
}

/**
  * @brief  Get_Status Request management: when Recipient is endpoint
  * @param  None
  * @return true or false.
  */
static bool usbd_get_status_endpoint(void)
{
    const InterfaceDescriptor_t *p_interface;

    wIndex_t *wIndex_work;
    uint8_t endpoint_number;
    uint16_t check_data;

    wIndex_work = (wIndex_t *) & g_wIndex;
    endpoint_number = (uint8_t) wIndex_work->Endpoint.Endpoint;

    switch (gDeviceState) {
    case CDEVICE_STATE_DEFAULT:
        return false;

    case CDEVICE_STATE_ADDRESSED:
        if (endpoint_number == USBD_EP0) {
            /* Do nothing */
        } else {
            return false;       /* set EP0 to STALL     */
        }
        break;

    case CDEVICE_STATE_CONFIGURED:
        break;

    default:
        return false;
    }

    p_interface = gUserDescriptor.p_struct_config[s_Current_Config - 1]->p_struct_interface[s_Current_Interface]->p_interface;
#ifndef _USBD_MSC_
    /* Force Pass on CDC,HID */
#else
    if (endpoint_number > p_interface->bNumEndPoints) {
        return false;
    } else {
        /* Do nothing */
    }
#endif

    /* make status data     */
    sGetStatusEndpoint = CsGetStatusEndpoint_INIT;
    if (endpoint_number == USBD_EP0) {
        fHalt = fEP0StallFeature;
    } else {
        check_data = (uint16_t) (CEPxStallFeature_MASK << endpoint_number);
        if ((gEPxStallFeatureFlag.All & 0xFFFFU & check_data) != CEPxStallFeature_Clear) {
            fHalt = CfHalt_ON;
        } else {
            fHalt = CfHalt_OFF;
        }
    }

    return true;
}

/**
  * @brief  Clear_Feature Request management
  * @param  None
  * @retval status : true or false.
  */
static bool usbd_clear_feature(void)
{
    bool status;

    /* check common data    */
    if ((f_bmDirection != CbmRequestType_Direction_HtoD) ||
        (g_RequestType != CbmRequestType_Type_Standard) || (g_wLength != CwLength_ClearFeature)) {
        return false;
    } else {
        /* Do nothing */
    }

    status = false;
    switch (g_Recipient) {
    case CbmRequestType_Recipient_Device:
        if (g_wIndex != CwIndex_ZERO) {
            status = false;
        } else {
            status = usbd_clear_feature_device();
        }
        break;

    case CbmRequestType_Recipient_Interface:
        status = false;
        break;

    case CbmRequestType_Recipient_Endpoint:
        if ((g_wIndex & CwIndex_RESERVED_MASK_ENDPOINT) != CwIndex_RESERVED_ENDPOINT) {
            status = false;
        } else {
            status = usbd_clear_feature_endpoint();
        }
        break;

    default:
        status = false;
        break;
    }

    if (status == true) {
        g_USB_Stage = STATUS_STAGE;
    } else {
        /* Do nothing */
    }

    return status;
}

/**
  * @brief  Clear_Feature Request management: when Recipient is device
  * @param  None
  * @retval status : true or false.
  */
static bool usbd_clear_feature_device(void)
{
    bool status;

    status = usbd_check_xxx_feature_device();

    if (status == true) {
        fRemoteWakeup = CfRemoteWakeup_OFF;
    } else {
        /* Do nothing */
    }

    return status;
}

/**
  * @brief  Clear_Feature Request management: when Recipient is endpoint
  * @param  None
  * @retval status : true or false.
  */
static bool usbd_clear_feature_endpoint(void)
{
    bool status;
    USBD_EPx EPx;

    status = usbd_check_xxx_feature_endpoint(&EPx);

    if (status != true) {
        return false;
    } else {
        if (EPx != USBD_EP0) {
            usbd_set_epx_feature(EPx, USBD_CMD_EP_RESET);
#ifdef _USBD_MSC_
            /* Callback (direct) */
            USBD_MSC_CB_ClearFeature(EPx, g_wValue);
#endif
        } else {
            /* Do nothing */
        }
        return true;
    }
}

/**
  * @brief  Set_Feature Request management
  * @param  None
  * @retval status : true or false.
  */
static bool usbd_set_feature(void)
{
    bool status;

    /* check common data    */
    if ((f_bmDirection != CbmRequestType_Direction_HtoD) ||
        (g_RequestType != CbmRequestType_Type_Standard) || (g_wLength != CwLength_SetFeature)) {
        return false;
    } else {
        /* Do nothing */
    }

    status = false;
    if (g_wValue == CwValueSelector_TEST_MODE) {
        /* Do nothing */
    } else {
        status = usbd_set_feature_normal();
    }

    if (status == true) {
        g_USB_Stage = STATUS_STAGE;
    } else {
        /* Do nothing */
    }

    return status;
}

/**
  * @brief  Set Feature Request at Normal Mode
  * @param  None
  * @retval status : true or false.
  */
static bool usbd_set_feature_normal(void)
{
    bool status;

    status = false;
    switch (g_Recipient) {
    case CbmRequestType_Recipient_Device:
        if (g_wIndex != CwIndex_ZERO) {
            status = false;
        } else {
            status = usbd_set_feature_device();
        }
        break;

    case CbmRequestType_Recipient_Interface:
        status = false;
        break;

    case CbmRequestType_Recipient_Endpoint:
        if ((g_wIndex & CwIndex_RESERVED_MASK_ENDPOINT) != CwIndex_RESERVED_ENDPOINT) {
            status = false;
        } else {
            status = usbd_set_feature_endpoint();
        }
        break;

    default:
        status = false;
        break;
    }

    return status;
}

/**
  * @brief  Set Feature Request at Normal Mode: when Recipient is device
  * @param  None
  * @retval status : true or false.
  */
static bool usbd_set_feature_device(void)
{
    bool status;

    status = usbd_check_xxx_feature_device();

    if (status == true) {
        fRemoteWakeup = CfRemoteWakeup_ON;
    } else {
        /* Do nothing */
    }

    return status;
}

/**
  * @brief  Set Feature Request at Normal Mode: when Recipient is endpoint
  * @param  None
  * @return true or false.
  */
static bool usbd_set_feature_endpoint(void)
{
    bool status;
    USBD_EPx EPx;

    status = usbd_check_xxx_feature_endpoint(&EPx);

    if (status != true) {
        return false;
    } else {
        usbd_set_epx_feature(EPx, USBD_CMD_EP_STALL);
        return true;
    }

}

/**
  * @brief  Set_Address Request management
  * @param  None
  * @return true or false.
  */
static bool usbd_set_address(void)
{
    /* check common data    */
    if ((f_bmDirection != CbmRequestType_Direction_HtoD) ||
        (g_RequestType != CbmRequestType_Type_Standard) ||
        (g_Recipient != CbmRequestType_Recipient_Device) ||
        (g_wValue > CDeviceAddressMAX) ||
        (g_wIndex != CwIndex_ZERO) || (g_wLength != CwLength_SetAddress)) {
        return false;
    } else {
        /* Do nothing */
    }

    if (fEP0StallFeature == CEPxStallFeature_ON) {
        return false;
    } else {
        /* Do nothing */
    }

    gDeviceAddrBuf = (uint8_t) g_wValue;

    switch (gDeviceState) {
    case CDEVICE_STATE_DEFAULT:
        if (gDeviceAddrBuf == CDEVIECE_ADDRESS_INIT) {
            gDeviceStateBuf = CDEVICE_STATE_DEFAULT;
        } else {
            gDeviceStateBuf = CDEVICE_STATE_ADDRESSED;
        }
        break;

    case CDEVICE_STATE_ADDRESSED:
        if (gDeviceAddrBuf == CDEVIECE_ADDRESS_INIT) {
            gDeviceStateBuf = CDEVICE_STATE_DEFAULT;
        } else {
            gDeviceStateBuf = CDEVICE_STATE_ADDRESSED;
        }
        break;

    case CDEVICE_STATE_CONFIGURED:
        return false;

    default:
        return false;
    }

    g_USB_Stage = STATUS_STAGE;

    return true;
}

/**
  * @brief  Get_Configuration Request management
  * @param  None
  * @retval status : true or false.
  */
static bool usbd_get_configuration(void)
{
    uint16_t data;
    bool status;

    /* check common data    */
    if ((f_bmDirection != CbmRequestType_Direction_DtoH) ||
        (g_RequestType != CbmRequestType_Type_Standard) ||
        (g_Recipient != CbmRequestType_Recipient_Device) ||
        (g_wIndex != CwIndex_ZERO) ||
        (g_wValue != CwValue_GetConfiguration) || (g_wLength != CwLength_GetConfiguration)) {
        return false;
    } else {
        /* Do nothing */
    }

    g_USB_Stage = DATA_STAGE;

    switch (gDeviceState) {
    case CDEVICE_STATE_DEFAULT:
        return false;

    case CDEVICE_STATE_ADDRESSED:
        data = 0U;
        break;

    case CDEVICE_STATE_CONFIGURED:
        data = s_Current_Config;
        break;

    default:
        return false;
    }

    status = usbd_Epx_Send(USBD_EP0, g_wLength, (uint8_t *)&data, NULL, NULL);

    return status;
}

/**
  * @brief  Set_Configuration Request management
  * @param  None
  * @return true or false.
  */
static bool usbd_set_configuration(void)
{
#if 1   /* Force Use All Endpoints (EP1,2,3) */
#else
    const InterfaceDescriptor_t *p_interface;
    const StructInterfaceDesc_t *p_struct_interface;
#endif

    wValue_SetConfiguration_t *wValue_work;
    uint8_t wValue_config;

    /* check common data    */
    if ((f_bmDirection != CbmRequestType_Direction_HtoD) ||
        (g_RequestType != CbmRequestType_Type_Standard) ||
        (g_Recipient != CbmRequestType_Recipient_Device) ||
        (g_wIndex != CwIndex_ZERO) || (g_wLength != CwLength_SetConfiguration)) {
        return false;
    } else {
        /* Do nothing */
    }

    wValue_work = (wValue_SetConfiguration_t *) & g_wValue;
    wValue_config = (uint8_t) wValue_work->Bit.Configuration;
    if (wValue_work->Bit.Reserved != CwValue_Reserved) {
        return false;
    } else {
        /* Do nothing */
    }

    if (wValue_config > gDeviceDescriptor.bNumConfiguration) {
        return false;
    } else {
        /* Do nothing */
    }

    switch (gDeviceState) {
    case CDEVICE_STATE_DEFAULT:
        return false;

    case CDEVICE_STATE_ADDRESSED:
        if (wValue_config == CbConfigurationValue_Init) {
            /* Do nothing */
        } else {
            gDeviceStateBuf = CDEVICE_STATE_CONFIGURED;
#if 1   /* Force Use All Endpoints (EP1,2,3) */
            usbd_set_xxx_reset_endpoint(USBD_EP3);
#ifdef _USBD_HID_
            /* Connect HID */
            USBD_HID_Connect();
#endif
#ifdef _USBD_CDC_
            /* Connect CDC */
            USBD_CDC_Connect();
#endif
#ifdef _USBD_MSC_
            /* Connect MSC */
            USBD_MSC_Connect();
#endif
#else
            p_struct_interface = gUserDescriptor.p_struct_config[wValue_config - 1]->p_struct_interface[s_Current_Interface];
            p_interface = p_struct_interface->p_interface;
            usbd_set_xxx_reset_endpoint(p_interface->bNumEndPoints);
#endif
        }
        break;

    case CDEVICE_STATE_CONFIGURED:
        if (wValue_config == CbConfigurationValue_Init) {
            gDeviceStateBuf = CDEVICE_STATE_ADDRESSED;
            USBD_SetEPCMD(USBD_EP0, USBD_CMD_ALL_EP_INVALID);
#ifdef _USBD_HID_
            /* Disconnect HID */
            USBD_HID_DisConnect();
#endif
#ifdef _USBD_CDC_
            /* Disconnect CDC */
            USBD_CDC_DisConnect();
#endif
#ifdef _USBD_MSC_
            /* Disconnect MSC */
            USBD_MSC_DisConnect();
#endif
        } else {
#if 1   /* Force Use All Endpoints (EP1,2,3) */
            usbd_set_xxx_reset_endpoint(USBD_EP3);
#else
            p_struct_interface = gUserDescriptor.p_struct_config[wValue_config - 1]->p_struct_interface[s_Current_Interface];
            p_interface = p_struct_interface->p_interface;
            usbd_set_xxx_reset_endpoint(p_interface->bNumEndPoints);
#endif
        }
        break;

    default:
        return false;
    }

    s_Buf_Current_Config = wValue_config;
    s_Buf_Current_Interface = CbInterfaceNumber_Init;
    s_Buf_Current_Alternate = CbAlternateSetting_Init;

    g_USB_Stage = STATUS_STAGE;

    return true;
}

/**
  * @brief  Get_Interface Request management
  * @param  None
  * @retval status : true or false.
  */
static bool usbd_get_interface(void)
{
    const InterfaceDescriptor_t *p_interface;
    bool status;

    /* check common data    */
    if ((f_bmDirection != CbmRequestType_Direction_DtoH) ||
        (g_RequestType != CbmRequestType_Type_Standard) ||
        (g_Recipient != CbmRequestType_Recipient_Interface) ||
        (g_wValue != CwValue_GetInterface) || (g_wLength != CwLength_GetInterface)) {
        return false;
    } else {
        /* Do nothing */
    }

    status = usbd_check_xxx_interface(&p_interface);
    if (status != true) {
        return false;
    } else {
        /* Do nothing */
    }
    g_USB_Stage = DATA_STAGE;

    sbAlternateSetting = (uint32_t) (p_interface->bAlternateSetting);

    status = usbd_Epx_Send(USBD_EP0, g_wLength, (uint8_t *)&sbAlternateSetting, NULL, NULL);

    return status;
}

/**
  * @brief  Set_Interface Request management
  * @param  None
  * @return true or false.
  */
static bool usbd_set_interface(void)
{
    const InterfaceDescriptor_t *p_interface;

    wIndex_t *wIndex_work;
    uint8_t interface_number;

    bool status;

    /* check common data    */
    if ((f_bmDirection != CbmRequestType_Direction_HtoD) ||
        (g_RequestType != CbmRequestType_Type_Standard) ||
        (g_Recipient != CbmRequestType_Recipient_Interface) ||
        (g_wLength != CwLength_SetInterface)) {
        return false;
    } else {
        /* Do nothing */
    }

    status = usbd_check_xxx_interface(&p_interface);
    if (status != true) {
        return false;
    } else {
        /* Do nothing */
    }

    wIndex_work = (wIndex_t *) & g_wIndex;
    interface_number = (uint8_t) wIndex_work->Interface.Number;

    s_Buf_Current_Interface = interface_number;
    s_Buf_Current_Alternate = (uint8_t) g_wValue;

#if 1   /* Force Use All Endpoints (EP1,2,3) */
    usbd_set_xxx_reset_endpoint(USBD_EP3);
#else
    usbd_set_xxx_reset_endpoint(p_interface->bNumEndPoints);
#endif

    g_USB_Stage = STATUS_STAGE;

    return true;
}

/**
  * @brief  Get_Descriptor Request management
  * @param  None
  * @return true or false.
  */
static bool usbd_get_descriptor(void)
{
    wValue_GetDescriptor_t *wValue_work;
    bool status;
    uint8_t index;

    /* check common data    */
    if (f_bmDirection != CbmRequestType_Direction_DtoH) {
        return false;
    } else {
        /* Do nothing */
    }

    g_USB_Stage = DATA_STAGE;

    switch (gDeviceState) {
    case CDEVICE_STATE_DEFAULT:
        break;

    case CDEVICE_STATE_ADDRESSED:
        break;

    case CDEVICE_STATE_CONFIGURED:
        break;

    default:
        return false;
    }

    wValue_work = (wValue_GetDescriptor_t *) & g_wValue;
    index = (uint8_t) wValue_work->Bit.Index;
    switch (wValue_work->Bit.Type) {
    case CwValue_TypeDevice:
        status = usbd_get_descriptor_device(index);
        break;

    case CwValue_TypeConfig:
        status = usbd_get_descriptor_config(index);
        break;

    case CwValue_TypeString:
        status = usbd_get_descriptor_string(index);
        break;

    case CwValue_TypeDeviceQualifier:
        /* Not Supported on FS Device */
        status = false;
        break;

    case CwValue_TypeOtherSpeed:
        /* Not Supported on FS Device */
        status = false;
        break;

#ifdef _USBD_HID_
    case CwValue_TypeHIDDescriptor:
        status = usbd_get_descriptor_HID(index);
        break;

    case CwValue_TypeHIDReport:
        status = usbd_get_descriptor_HID_report(index);
        break;
#endif

    default:
        status = false;
        break;
    }

    if (status == false) {
        return false;
    } else {
    }

    return true;
}

/**
  * @brief  Get_Descriptor Request management: for device
  * @param  index :  Descriptor index number.
  * @retval status : true or false.
  */
static bool usbd_get_descriptor_device(uint8_t index)
{
    uint16_t size;
    bool status;

    if ((index != CwValue_GetDescriptor_IndexDevice) || (g_wIndex != CwIndex_GetDescriptor_Device)) {
        return false;
    } else {
        /* Do nothing */
    }

    /* write EP0 FIFO       */
    if (g_wLength <= CbLength_Device) {
        size = g_wLength;
    } else {
        size = CbLength_Device;
    }

    status = usbd_Epx_Send(USBD_EP0, size, (uint8_t *)&gDeviceDescriptor, NULL, NULL);

    return status;
}

/**
  * @brief  Get_Descriptor Request management: for configuration
  * @param  index :  Descriptor index number.
  * @retval status : true or false.
  */
static bool usbd_get_descriptor_config(uint8_t index)
{
    uint8_t *p_dst;
    const uint8_t *p_src;
    uint8_t i, j, k;
    uint8_t max_config;
    uint8_t max_interface;
    uint8_t max_endpoint;
    uint16_t length;
    uint8_t ep_index;
    uint16_t size;
    bool status;

    const ConfigDescriptor_t *p_config;
    const InterfaceDescriptor_t *p_interface;
    const StructInterfaceDesc_t *p_struct_interface;
    const EndpointDescriptor_t **pp_endpoint;

    EndpointDescriptor_t endpoint_work;

    /* check data */
    max_config = gUserDescriptor.p_device->bNumConfiguration;
    if ((index > max_config) || (g_wIndex != CwIndex_GetDescriptor_Config)) {
        return false;
    } else {
        /* Do nothing */
    }

    /* set configuration descriptor         */
    p_config = gUserDescriptor.p_struct_config[index]->p_config;
    p_dst = sConfigDescData;

    /* set Configuration[index] configuration descriptor    */
    p_src = (const uint8_t *) p_config;
    length = p_config->bLength;
    for (k = 0U; k < length; k++) {
        *p_dst++ = *p_src++;
    }

    /* set Configuration[index] interface/endpoint descriptor       */
    max_interface = p_config->bNumInterfaces;
    for (i = 0U; i < max_interface; i++) {

        p_struct_interface = gUserDescriptor.p_struct_config[index]->p_struct_interface[i];
        p_interface = p_struct_interface->p_interface;

        /* set interface[i] descriptor  */
        p_src = (const uint8_t *) p_interface;
        length = p_interface->bLength;
        for (k = 0U; k < length; k++) {
            *p_dst++ = *p_src++;
        }

#ifdef _USBD_HID_
        /* HID descriptor */
        p_src = (uint8_t *) &gHIDDescriptor;
        for (k = 0U; k < CbLength_HIDDescriptor; k++) {
            *p_dst++ = *p_src++;
        }
#endif
#ifdef _USBD_CDC_
        switch (i) {
        case 0:     /* interface0 */
            /* Header Functional Descriptor (config:1, interface:0) */
            p_src = (uint8_t *) & gHeaderFunctionalDescriptor1_0;
            for (k = 0U; k < gHeaderFunctionalDescriptor1_0.bLength; k++) {
                *p_dst++ = *p_src++;
            }
            /* Abstract Control Management Functional Descriptor (config:1, interface:0) */
            p_src = (uint8_t *) & gAbstractControlManagementFunctionalDescriptor1_0;
            for (k = 0U; k < gAbstractControlManagementFunctionalDescriptor1_0.bLength; k++) {
                *p_dst++ = *p_src++;
            }
            /* Union Functional Descriptor (config:1, interface:0) */
            p_src = (uint8_t *) & gUnionFunctionalDescriptor1_0;
            for (k = 0U; k < gUnionFunctionalDescriptor1_0.bLength; k++) {
                *p_dst++ = *p_src++;
            }
            /* Call Management Functional Descriptor (config:1, interface:0) */
            p_src = (uint8_t *) & gCallManagementFunctionalDescriptor1_0;
            for (k = 0U; k < gCallManagementFunctionalDescriptor1_0.bLength; k++) {
                *p_dst++ = *p_src++;
            }
            break;

        case 1:     /* interface1 */
            /* Header Functional Descriptor (config:1, interface:1) */
            p_src = (uint8_t *) & gHeaderFunctionalDescriptor1_1;
            for (k = 0U; k < gHeaderFunctionalDescriptor1_1.bLength; k++) {
                *p_dst++ = *p_src++;
            }
            break;

        default:
            break;
        }
#endif
#ifdef _USBD_MSC_
#endif
        /* set endpoint[j] descriptor   */
        pp_endpoint = p_struct_interface->pp_endpoint;
        max_endpoint = p_interface->bNumEndPoints;
        for (j = 0U; j < max_endpoint; j++) {

            /* convert max packet size for Full or High Speed       */
            endpoint_work = *pp_endpoint[j];
            ep_index = (endpoint_work.bEndPointAddress & CbEndPointAddress_MASK);
            endpoint_work.wMaxPacketSize = gEPxPayload_Size[ep_index];

            p_src = (const uint8_t *) &endpoint_work;
            length = endpoint_work.bLength;
            for (k = 0U; k < length; k++) {
                *p_dst++ = *p_src++;
            }
        }
    }

    /* write EP0 FIFO       */
    length = p_config->wTotalLength;
    if (g_wLength <= length) {
        size = g_wLength;
    } else {
        size = length;
    }

    status = usbd_Epx_Send(USBD_EP0, size, (uint8_t *)sConfigDescData, NULL, NULL);

    return status;
}

/**
  * @brief  Get_Descriptor Request management: for string
  * @param  index :  String descriptor index number.
  * @retval status : true or false.
  */
static bool usbd_get_descriptor_string(uint8_t index)
{
    uint16_t length;
    const StringDescriptor_t *p_string;
    uint16_t size;
    bool status;

    if (index >= CbNumStringDescriptor) {
        return false;
    } else {
        /* Do nothing */
    }

    p_string = (StringDescriptor_t *) gStringDescriptor[index];

    length = p_string->bLength;

    if (g_wLength <= length) {
        size = g_wLength;
    } else {
        size = length;
    }

    status = usbd_Epx_Send(USBD_EP0, size, (uint8_t *)p_string, NULL, NULL);

    return status;
}

/**
  * @brief  Check current device feature. called by "Clear/Set Feature" Request
  * @param  None
  * @retval status : true or false.
  */
static bool usbd_check_xxx_feature_device(void)
{
    bool status;

    switch (gDeviceState) {
    case CDEVICE_STATE_DEFAULT:
        return false;

    case CDEVICE_STATE_ADDRESSED:
        break;

    case CDEVICE_STATE_CONFIGURED:
        break;

    default:
        return false;
    }

    status = true;
    if (g_wValue == CwValueSelector_DEVICE_REMOTE_WAKEUP) {
        status = true;
    } else {
        status = false;
    }

    return status;
}

/**
  * @brief  Check current endpoint feature. called by "Clear/Set Feature" Request
  * @param  *endpoint : Endpoint number to be returned.
  * @return true or false.
  */
static bool usbd_check_xxx_feature_endpoint(USBD_EPx * endpoint)
{
    const StructConfigDesc_t *p_struct_config;

    const InterfaceDescriptor_t *p_interface;
    const StructInterfaceDesc_t *p_struct_interface;

    wIndex_t *wIndex_work;

    wIndex_work = (wIndex_t *) & g_wIndex;
    *endpoint = (USBD_EPx) wIndex_work->Endpoint.Endpoint;

    switch (gDeviceState) {
    case CDEVICE_STATE_DEFAULT:
        return false;

    case CDEVICE_STATE_ADDRESSED:
        if (*endpoint != USBD_EP0) {
            return false;
        } else {
            /* Do nothing */
        }
        break;

    case CDEVICE_STATE_CONFIGURED:
        break;

    default:
        return false;
    }

    if (g_wValue != CwValueSelector_ENDPOINT_HALT) {
        return false;
    } else {
        /* Do nothing */
    }

    p_struct_config = gUserDescriptor.p_struct_config[s_Current_Config - 1];
    p_struct_interface = p_struct_config->p_struct_interface[s_Current_Interface];
    p_interface = p_struct_interface->p_interface;
#ifndef _USBD_MSC_
    /* Force Pass on CDC,HID */
#else
    if (*endpoint > p_interface->bNumEndPoints) {
        return false;
    } else {
        /* Do nothing */
    }
#endif

    return true;
}

/**
  * @brief  Check current interface. called by "Get/Set Interface" Request
  * @param  **pp_interface : Interface descriptor pointer.
  * @return true or false.
  */
static bool usbd_check_xxx_interface(const InterfaceDescriptor_t ** pp_interface)
{
    const ConfigDescriptor_t *p_config;
    const StructConfigDesc_t *p_struct_config;

    const StructInterfaceDesc_t *p_struct_interface;

    wIndex_t *wIndex_work;
    uint8_t interface_number;

    if ((g_wIndex & CwIndex_RESERVED_MASK_INTERFACE) != CwIndex_RESERVED_INTERFACE) {
        return false;
    } else {
        /* Do nothing */
    }

    switch (gDeviceState) {
    case CDEVICE_STATE_DEFAULT:
        return false;

    case CDEVICE_STATE_ADDRESSED:
        return false;           /* EP0 Stall    */

    case CDEVICE_STATE_CONFIGURED:
        break;

    default:
        return false;
    }

    wIndex_work = (wIndex_t *) & g_wIndex;
    interface_number = (uint8_t) wIndex_work->Interface.Number;

    if (s_Current_Config == 0) {
        return false;
    } else {
        p_struct_config = gUserDescriptor.p_struct_config[s_Current_Config - 1];
        p_config = p_struct_config->p_config;
    }

    if (interface_number >= p_config->bNumInterfaces) {
        return false;
    } else {
        /* Do nothing */
    }

    p_struct_interface = p_struct_config->p_struct_interface[s_Current_Interface];
    *pp_interface = p_struct_interface->p_interface;

    return true;
}

/**
  * @brief  Reset Endpoint. called by "Set Interface/Set Configuration" Request
  * @param  num_endpoint : Number of endpoints in interface.
  * @retval None
  */
static void usbd_set_xxx_reset_endpoint(uint8_t num_endpoint)
{
    uint8_t i;
    uint32_t tmpAddr = 0U;

    USBD_SetEPCMD(USBD_EP0, USBD_CMD_ALL_EP_INVALID);

    for (i = USBD_EP1; i <= num_endpoint; i++) {
        tmpAddr = (uint32_t) UDC2_EP1MAXPACKETSIZE + (0x10U * (i - 1U));
        USBD_WriteUDC2Reg(tmpAddr, gEPxPayload_Size[i]);

        tmpAddr = (uint32_t) UDC2_EP1STATUS + (0x10U * (i - 1U));
        USBD_WriteUDC2Reg(tmpAddr, gEPxConfigArray[i]);

        usbd_set_epx_feature((USBD_EPx) i, USBD_CMD_EP_RESET);

        USBD_SetEPCMD((USBD_EPx) i, USBD_CMD_EP_ENABLE);
    }

    return;
}

/**
  * @brief  Set Endpoint feature.
  * @param  EPx : Endpoint number.
  *    This parameter can be one of the following values :
  *      USBD_EP0, USBD_EP1, USBD_EP2, USBD_EP3,
  *      USBD_EP4, 
  * @param  Cmd : Command which be set to EPx.
  *    This parameter can be one of the following values :
  *      USBD_CMD_EP_STALL, USBD_CMD_EP_RESET
  * @retval None
  */
void usbd_set_epx_feature(USBD_EPx EPx, USBD_EPCMD Cmd)
{
    if (Cmd == USBD_CMD_EP_STALL) {
        gEPxStallFeatureFlag.All |= 0x01UL << (uint32_t) (EPx);
        USBD_SetEPCMD(EPx, USBD_CMD_EP_STALL);
    } else {
        gEPxStallFeatureFlag.All &= (uint32_t) (~(0x01UL << (uint32_t) (EPx)));
        gEPxStallFeatureFlag.All &= 0xFFU;      /* only keep low 8bit */
        USBD_SetEPCMD(EPx, USBD_CMD_EP_RESET);
    }

    return;
}

/**
  * @brief  Update current configuration from buffered variables
  * @param  None
  * @retval None
  */
void usbd_device_request_update_current_config(void)
{
    s_Current_Config = s_Buf_Current_Config;
    s_Current_Interface = s_Buf_Current_Interface;
    s_Current_Alternate = s_Buf_Current_Alternate;

    return;
}

#ifdef _USBD_HID_
/**
  * @brief  Send HID descriptor
  * @param  index : Request index (no use).
  * @retval status : true or false.
  */
static bool usbd_get_descriptor_HID(uint8_t index)
{
    bool        status;
    uint16_t    usSize;

    if (g_wLength <= CbLength_HIDDescriptor) {
        usSize = g_wLength;
    } else {
        usSize = CbLength_HIDDescriptor;
    }

    status = usbd_Epx_Send(USBD_EP0, usSize, (uint8_t *)&gHIDDescriptor, NULL, NULL);

    return status;
}

/**
  * @brief  Send HID report descriptor
  * @param  index : Request index (no use).
  * @retval status : true or false.
  */
static bool usbd_get_descriptor_HID_report(uint8_t index)
{
    bool        status;
    uint16_t    usSize;

    if (g_wLength <= CbLength_HIDReportDescriptor) {
        usSize = g_wLength;
    } else {
        usSize = CbLength_HIDReportDescriptor;
    }

    status = usbd_Epx_Send(USBD_EP0, usSize, (uint8_t *)&gHIDReportDescriptor, NULL, NULL);

    return status;
}
#endif

/* EOC */
