/**
 *******************************************************************************
 * @file    usbd_hw_interrupt.c
 * @brief   USB H/W Interrupt Handler Code for TOSHIBA 'TMPM066' Device Series
 * @version V0.100
 * @date    2015/09/11
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 *
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

#include "usbd_hw.h"
//#include "tmpm365_cg.h"         // add
//#include "tmpm365_gpio.h"       // add
#include "usbd_var.h"
#include "usbd_device_request.h"
#include "usbd_typedefs.h"
#include "usbd_trans.h"

#ifdef _USBD_HID_
#include "usbd_hid.h"
#include "usbd_descriptor_hid.h"
#endif
#ifdef _USBD_CDC_
#include "usbd_cdc.h"
#include "usbd_descriptor_cdc.h"
#endif
#ifdef _USBD_MSC_
#include "usbd_scsi.h"
#include "usbd_msc.h"
#include "usbd_descriptor_msc.h"
#endif

static void USB_ISR_Setup(void);
static void USB_ISR_Rx_Zero(void);
static void USB_ISR_Status(void);
static void USB_ISR_Status_NAK(void);
static void USB_ISR_EP0(void);
static void USB_ISR_EPx(void);
static void USB_ISR_NAK(void);
static void USB_ISR_SOF(void);
static void USB_ISR_Suspend_Resume(void);
static void USB_ISR_USB_Reset_End(void);
static void USB_ISR_USB_Reset(void);

/**
  * @brief  VBUS detect Interrupt Service Routine for USBD
  * @param  None
  * @retval None
  */
void INTUSBWKUP_IRQHandler(void)
{
    volatile int dummy;
    volatile int k;
    //TSB_CG_IMCGC_INTAEN = 0U;

    for(k = 0; 1000 >k ; k++);

#if 0       //2015/9/11 16:28 modify it later after CG module is ready
    if (GPIO_ReadDataBit(GPIO_PG, GPIO_BIT_5) == 0) {
        /* VBUS OFF */
        CG_SetSTBYReleaseINTSrc(CG_INT_SRC_USB_PON, CG_INT_ACTIVE_STATE_H, ENABLE);
#ifdef _USBD_HID_
        /* Disconnect HID */
        USBD_HID_DisConnect();
#endif
#ifdef _USBD_CDC_
        /* Disconnect CDC */
        USBD_CDC_DisConnect();
#endif
#ifdef _USBD_MSC_
        /* Disconnect MSC */
        USBD_MSC_DisConnect();
#endif
        usbd_initialize_standard_class_work_data();
    }

    if (GPIO_ReadDataBit(GPIO_PG, GPIO_BIT_5) != 0) {
        /* VBUS ON */
        CG_SetSTBYReleaseINTSrc(CG_INT_SRC_USB_PON, CG_INT_ACTIVE_STATE_L, ENABLE);
    }

    CG_ClearINTReq(CG_INT_SRC_USB_PON);

#endif  //2015/9/11 16:28

    //TSB_CG_IMCGC_INTAEN = 1U;
}

/**
  * @brief  Main Interrupt Service Routine for USBD
  * @param  None
  * @retval None
  */
void INTUSB_IRQHandler(void)
{
    USBD_INTStatus tmp = { 0U };

    tmp = USBD_GetINTStatus();

    /* Select interrupt process */
    if (tmp.Bit.Setup) {
        USB_ISR_Setup();
    } else if (tmp.Bit.Status_NAK) {
        USB_ISR_Status_NAK();
    } else if (tmp.Bit.Status) {
        USB_ISR_Status();
    } else if (tmp.Bit.Rx_Zero) {
        USB_ISR_Rx_Zero();
    } else if (tmp.Bit.SOF) {
        USB_ISR_SOF();
    } else if (tmp.Bit.EP0) {
        USB_ISR_EP0();
    } else if (tmp.Bit.EP) {
        USB_ISR_EPx();
    } else if (tmp.Bit.NAK) {
        USB_ISR_NAK();
    } else if (tmp.Bit.Suspend_Resume) {
        USB_ISR_Suspend_Resume();
    } else if (tmp.Bit.USB_Reset) {
        USB_ISR_USB_Reset();
    } else if (tmp.Bit.USB_Reset_End) {
        USB_ISR_USB_Reset_End();
    } else {
        /* Do noting */
    }

    return;
}

/**
  * @brief  Interrupt Service Routine after get setup interrupt
  * @param  None
  * @retval None
  */
static void USB_ISR_Setup(void)
{
    bool status = true;
    uint32_t tmp = 0U;

    USBD_ClearINT(USBD_INT_SETUP);

    if (g_USB_Stage == IDLE_STAGE) {
        g_USB_Stage = SETUP_STAGE;
    } else {
        status = false;
    }

    /* Get device requests data */
    USBD_ReadUDC2Reg(UDC2_BREQ_BMREQTYPE, &tmp);
    /* bmRequestType */
    g_bmRequestType = (uint8_t) (tmp & 0x00FFU);
    /* bRequest */
    g_bRequest = (uint8_t) ((tmp & 0x0000FF00U) >> 8U);
    /* wValue */
    USBD_ReadUDC2Reg(UDC2_WVALUE, &tmp);
    g_wValue = (uint16_t) tmp;
    /* wIndex */
    USBD_ReadUDC2Reg(UDC2_WINDEX, &tmp);
    g_wIndex = (uint16_t) tmp;
    /* wLength */
    USBD_ReadUDC2Reg(UDC2_WLENGTH, &tmp);
    g_wLength = (uint16_t) tmp;

    /* Recognize setup stage (Received device request) */
    USBD_SetEPCMD(USBD_EP0, USBD_CMD_SETUP_RECEIVED);

    if (status == true) {
        if (g_RequestType == CUD2BRQ_B_ReqType_STANDARD) {
            /* Standard requests */
            status = usbd_interrupt_setup_standard(g_bRequest);
        } else {
#ifdef _USBD_HID_
            /* HID class requests */
            status = usbd_interrupt_HID_req(g_bRequest);
#endif
#ifdef _USBD_CDC_
            /* CDC/ACM class requests */
            status = usbd_interrupt_CDC_req(g_bRequest);
#endif
#ifdef _USBD_MSC_
            /* MSC(BOT) class requests */
            status = usbd_interrupt_MSC_req(g_bRequest);
#endif
        }
    } else {
        /* Do nothing */
    }

    if (status == false) {
        g_USB_Stage = IDLE_STAGE;
        usbd_set_epx_feature(USBD_EP0, USBD_CMD_EP_STALL);
    } else {
        /* Do nothing */
    }

    return;
}

/**
  * @brief  Interrupt Service Routine after get zero length data
  * @param  None
  * @retval None
  */
static void USB_ISR_Rx_Zero(void)
{
    USBD_ClearINT(USBD_INT_RX_ZERO);

    if (g_USB_Stage == DATA_STAGE) {
        if (f_bmDirection == CbmRequestType_Direction_DtoH) {
            usbd_Epx_Write_Fifo(USBD_EP0);
        } else {
            usbd_Epx_Read_Fifo(USBD_EP0);
        }
    } else {
        /* Do nothing */
    }

    return;
}

/**
  * @brief  Additional function for "Set Feature Request" and "Set Address Request"
  * @param  None
  * @retval None
  */
static void USB_ISR_Status(void)
{

    USBD_ClearINT(USBD_INT_STATUS);

    if (g_USB_Stage == STATUS_STAGE) {
        if ((g_bRequest == CbRequest_SET_CONFIGURATION) || (g_bRequest == CbRequest_SET_ADDRESS)) {

            gUDC2Addr = gUDC2AddrBuf;
            USBD_WriteUDC2Reg(UDC2_ADDR, gUDC2Addr.All);
        } else {
            /* Do nothing */
        }

        /* Update Config/Interface/Alternate */
        usbd_device_request_update_current_config();

        /* Stage information initialize */
        g_USB_Stage = IDLE_STAGE;
    } else {
        /* Do nothing */
    }

    return;
}

/**
  * @brief  Additional function for STATUS NAK at status stage of control read/write
  * @param  None
  * @retval None
  */
static void USB_ISR_Status_NAK(void)
{
    USBD_ClearINT(USBD_INT_STATUS_NAK);
    g_USB_Stage = STATUS_STAGE;
    USBD_SetEPCMD(USBD_EP0, USBD_CMD_SETUP_FIN);

    return;
}

/**
  * @brief  Function for received or transmited EP0 ACK
  * @param  None
  * @retval None
  */
static void USB_ISR_EP0(void)
{
    USBD_ClearINT(USBD_INT_EP0);

    if (f_bmDirection == CbmRequestType_Direction_DtoH) {
        usbd_Epx_Write_Fifo(USBD_EP0);
    } else {
        usbd_Epx_Read_Fifo(USBD_EP0);
    }

    return;
}

/**
  * @brief  Function for received or transmited EPx ACK
  * @param  None
  * @retval None
  */
static void USB_ISR_EPx(void)
{
    uint32_t tmp = 0U;
    uint8_t i;

    /* Read EPx ACK interruption */
    USBD_ReadUDC2Reg(UDC2_INTEP, &tmp);
    tmp >>= 1U;

    /* Clear i_ep */
    USBD_ClearINT(USBD_INT_EP);

    /* Read/Write process */
    for (i=1U; i<CNumEndpointSupport; i++) {
        if (tmp & 0x01U) {
            /* EPi ACK interruption */
            if (i & 0x01U) {
                /* IN */
                usbd_Epx_Write_Fifo((USBD_EPx)i);
            } else {
                /* OUT */
                usbd_Epx_Read_Fifo((USBD_EPx)i);
            }
        } else {
            /* Do nothing */
        }
        tmp >>= 1U;
    }

    return;
}

/**
  * @brief  Interrupt Service Routine after NAK is sent to host
  * @param  None
  * @retval None
  */
static void USB_ISR_NAK(void)
{
    USBD_ClearINT(USBD_INT_NAK);

    return;
}

/**
  * @brief  Interrupt Service Routine after SOF packet is received
  * @param  None
  * @retval None
  */
static void USB_ISR_SOF(void)
{
    USBD_ClearINT(USBD_INT_SOF);

    return;
}

/**
  * @brief  Interrupt Service Routine after suspend/resume
  * @param  None
  * @retval None
  */
static void USB_ISR_Suspend_Resume(void)
{
    USBD_ClearINT(USBD_INT_SUSPEND_RESUME);

    return;
}

/**
  * @brief  Interrupt Service Routine after end of usb reset
  * @param  None
  * @retval None
  */
static void USB_ISR_USB_Reset_End(void)
{
    UDC2_ADDR_t udc2addr_work;
    uint32_t tmp = 0U;

    USBD_ClearINT(USBD_INT_USB_RESET_END);
#ifdef _USBD_HID_
    /* Disconnect HID */
    USBD_HID_DisConnect();
#endif
#ifdef _USBD_CDC_
    /* Disconnect CDC */
    USBD_CDC_DisConnect();
#endif
#ifdef _USBD_MSC_
    /* Disconnect MSC */
    USBD_MSC_DisConnect();
#endif
    usbd_initialize_standard_class_work_data();

    /* Current_Speed check */
    USBD_ReadUDC2Reg(UDC2_ADDR, &tmp);
    udc2addr_work.All = tmp;
    if (udc2addr_work.Bit.Current_Speed == CUDC2ADDR_HIGHSPEED) {
        gEP1Payload_Size = CwMaxPacketSize_HS;
        gEP2Payload_Size = CwMaxPacketSize_HS;
    } else {
        gEP1Payload_Size = CwMaxPacketSize_FS;
        gEP2Payload_Size = CwMaxPacketSize_FS;
    }

    return;
}

/**
  * @brief  Interrupt Service Routine after usb reset is detected
  * @param  None
  * @retval None
  */
static void USB_ISR_USB_Reset(void)
{
    USBD_ClearINT(USBD_INT_USB_RESET);

    return;
}
