/**
 *******************************************************************************
 * @file    usbd_trans.c
 * @brief   USB EPx Transfer Code for TOSHIBA 'TMPM066' Device Series
 * @version V0.100
 * @date    2015/09/11
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 *
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

#include "usbd_hw.h"
#include "usbd_var.h"
#include "usbd_trans.h"

/** 
  * @brief Endpoint information
  */
EpTransInfo_t gEpTransInfo[CNumEndpointSupport];

/**
  * @brief  EPx receive API
  * @param  epNum : Endpoint number.
  *    This parameter can be one of the following values :
  *      USBD_EP0, USBD_EP2, USBD_EP4
  * @param  ulSize : Transfer size.
  * @param  pucBuf : Buffer pointer.
  * @param  pvInfo : Extended information.
  * @param  pfnCallback : Callback function.
  * @retval bStat : true or false.
  */
bool usbd_Epx_Recv(USBD_EPx epNum, uint32_t ulSize, uint8_t* pucBuf, void* pvInfo, EpCallbackFunc pfnCallback)
{
    bool bStat = false;
    
    /* Check endpoint (IN) */
    if ((epNum != USBD_EP0) && (epNum != USBD_EP2) && (epNum != USBD_EP4) ) {
        /* Illegal endpoint (not OUT) */
        return false;
    }

    /* Check buffer pointer */
    if (pucBuf == (uint8_t*)NULL) {
        /* Illegal buffer pointer */
        return false;
    }

    /* Check status */
    if (gEpTransInfo[epNum].usEpStat != EPSTAT_BUSY) {
        /* IDLE */
        gEpTransInfo[epNum].usEpStat    = EPSTAT_BUSY;
        gEpTransInfo[epNum].usEpNum     = epNum;
        gEpTransInfo[epNum].ulReqSize   = ulSize;
        gEpTransInfo[epNum].ulTransSize = 0U;
        gEpTransInfo[epNum].pucBuf      = pucBuf;
        gEpTransInfo[epNum].pucCurBuf   = pucBuf;
        gEpTransInfo[epNum].pvInfo      = pvInfo;
        gEpTransInfo[epNum].pfnCallback = pfnCallback;

        /* Enable EPx interrupt */
        USBD_SetINTEPMask(epNum, ENABLE);
        
        bStat = true;
    } else {
        /* BUSY */
        bStat = false;
    }
    return bStat;
}

/**
  * @brief  EPx send API
  * @param  epNum : Endpoint number.
  *    This parameter can be one of the following values :
  *      USBD_EP0, USBD_EP1, USBD_EP3
  * @param  ulSize : Transfer size.
  * @param  pucBuf : Buffer pointer.
  * @param  pvInfo : Extended information.
  * @param  pfnCallback : Callback function.
  * @retval bStat : true or false.
  */
bool usbd_Epx_Send(USBD_EPx epNum, uint32_t ulSize, uint8_t* pucBuf, void* pvInfo, EpCallbackFunc pfnCallback)
{
    bool bStat = false;
    
    /* Check endpoint (OUT) */
    if ((epNum != USBD_EP0) && (epNum != USBD_EP1) && (epNum != USBD_EP3) ) {
        /* Illegal endpoint (not IN) */
        return false;
    }

    /* Check buffer pointer */
    if (pucBuf == (uint8_t*)NULL) {
        /* Illegal buffer pointer */
        return false;
    }

    /* Check status */
    if (gEpTransInfo[epNum].usEpStat != EPSTAT_BUSY) {
        /* IDLE */
        gEpTransInfo[epNum].usEpStat    = EPSTAT_BUSY;
        gEpTransInfo[epNum].usEpNum     = epNum;
        gEpTransInfo[epNum].ulReqSize   = ulSize;
        gEpTransInfo[epNum].ulTransSize = 0U;
        gEpTransInfo[epNum].pucBuf      = pucBuf;
        gEpTransInfo[epNum].pucCurBuf   = pucBuf;
        gEpTransInfo[epNum].pvInfo      = pvInfo;
        gEpTransInfo[epNum].pfnCallback = pfnCallback;

        /* Write first packet to FIFO */
        usbd_Epx_Write_Fifo(epNum);

        /* Enable EPx interrupt */
        USBD_SetINTEPMask(epNum, ENABLE);
        
        bStat = true;
    } else {
        /* BUSY */
        bStat = false;
    }
    return bStat;
}

/**
  * @brief  EPx abort API
  * @param  epNum : Endpoint number.
  *    This parameter can be one of the following values :
  *      USBD_EP1, USBD_EP2, USBD_EP3, USBD_EP4,
  * @retval bStat : true or false.
  */
bool usbd_Epx_Abort(USBD_EPx epNum)
{
    USBD_EPxStatus tmp = { 0U };
    bool           bStat;
    
    /* Check endpoint */
    if (epNum != USBD_EP0) {
        /* Disable EPx interrupt */
        USBD_SetINTEPMask(epNum, DISABLE);

        /* EPx disable */
        USBD_SetEPCMD(epNum, USBD_CMD_EP_DISABLE);

        /* Wait till disable bit(bit8) turn to 1 */
        do {
            /* Get current EPx status */
            tmp = USBD_GetEPxStatus(epNum);
        } while (tmp.Bit.Disable != 1);

        /* EPx FIFO clear */
        USBD_SetEPCMD(epNum, USBD_CMD_EP_FIFO_CLEAR);

        if (gEpTransInfo[epNum].usEpStat == EPSTAT_BUSY) {
            /* Abort */
            gEpTransInfo[epNum].usEpStat = EPSTAT_ABORT;

            /* Callback */
            if (gEpTransInfo[epNum].pfnCallback != NULL) {
                (*(gEpTransInfo[epNum].pfnCallback))(epNum);
            }
        } else {
            /* Idle */
            gEpTransInfo[epNum].usEpStat = EPSTAT_IDLE;
        }

        /* EPx enable */
        USBD_SetEPCMD(epNum, USBD_CMD_EP_ENABLE);
        
        bStat = true;
    } else {
        /* Illegal endpoint */
        bStat = false;
    }
    
    return bStat;
}

/**
  * @brief  Read data from EPx_FIFO (Data transferred from host to device)
  * @param  epNum : Endpoint number.
  *    This parameter can be one of the following values :
  *      USBD_EP0, USBD_EP2, USBD_EP4
  * @retval None
  */
void usbd_Epx_Read_Fifo(USBD_EPx epNum)
{

    uint32_t length;                /* Read data length                */
    uint32_t cntThrow;              /* Read data length for throw away */
    uint32_t tmp32;                 /* Read data buffer from EPx_FIFO  */
    uint16_t *idx16;                /* Data buffer pointer             */
    bool     bShortPacket = false;  /* Short packet flag               */

    idx16 = (uint16_t *) gEpTransInfo[epNum].pucCurBuf;
    /* Get the valid data length that are stored in the EPx_FIFO */
    USBD_ReadUDC2Reg(UDC2_EPxDATASIZE(epNum), &length);

    /* Judgement of remain total length */
    cntThrow = 0U;
    if (gEpTransInfo[epNum].ulReqSize < length) {
        cntThrow = (length - gEpTransInfo[epNum].ulReqSize);
        length   = gEpTransInfo[epNum].ulReqSize;
    } else {
        if (length < gEPxPayload_Size[epNum]) {
            /* Short packet */
            bShortPacket = true;
        } else {
            /* Do nothing */
        }
    }

    /* Read data from EPx_FIFO */
    while (length != 0U) {
        USBD_ReadUDC2Reg(UDC2_EPxFIFO(epNum), &tmp32);
        if (length == 1U) {
            *(uint8_t *) idx16 = (uint8_t) tmp32;
            length = 0U;

            /* Update endpoint information */
            gEpTransInfo[epNum].ulReqSize--;
            gEpTransInfo[epNum].ulTransSize++;
            gEpTransInfo[epNum].pucCurBuf++;
        } else {
            *idx16 = (uint16_t) tmp32;
            idx16++;
            length -= 2U;       /* Size of uint16_t */

            /* Update endpoint information */
            gEpTransInfo[epNum].ulReqSize   -= 2U;
            gEpTransInfo[epNum].ulTransSize += 2U;
            gEpTransInfo[epNum].pucCurBuf   += 2U;
        }
    }

    /* Get exception data(throw away)   */
    while (cntThrow != 0U) {
        USBD_ReadUDC2Reg(UDC2_EPxFIFO(epNum), &tmp32);
        if (cntThrow == 1U) {
            cntThrow = 0U;
        } else {
            cntThrow -= 2U;
        }
    }

    if ((bShortPacket == true) || (gEpTransInfo[epNum].ulReqSize == 0U)) {
        /* Receive completed */
        gEpTransInfo[epNum].usEpStat = EPSTAT_COMPLETE;

        /* Disable EPx interrupt */
        USBD_SetINTEPMask(epNum, DISABLE);

        /* Callback */
        if (gEpTransInfo[epNum].pfnCallback != NULL) {
            (*(gEpTransInfo[epNum].pfnCallback))(epNum);
        }
    } else {
        /* Do nothing */
    }
    return;
}

/**
  * @brief  Write data to EPx_FIFO (Data transferred from device to host)
  * @param  epNum : Endpoint number.
  *    This parameter can be one of the following values :
  *      USBD_EP0, USBD_EP1, USBD_EP3
  * @retval None
  */
void usbd_Epx_Write_Fifo(USBD_EPx epNum)
{
    uint32_t length;                /* Write data length               */
    uint32_t tmp32;                 /* Write data buffer to EPx_FIFO   */
    uint16_t *idx16;                /* Data buffer pointer             */
    uint8_t  chardata;              /* Write fraction data             */

    idx16 = (uint16_t *) gEpTransInfo[epNum].pucCurBuf;

    /* Judgement of remain total length */
    if (gEpTransInfo[epNum].ulReqSize >= gEPxPayload_Size[epNum]) {
        /* Get payload size */
        length = gEPxPayload_Size[epNum];
        /* Write data to EPx_FIFO */
        while (length > 0U) {
            tmp32 = (uint32_t) (*idx16);
            USBD_WriteUDC2Reg(UDC2_EPxFIFO(epNum), tmp32);
            idx16++;
            length -= 2U;       /* Size of uint16 */

            /* Update endpoint information */
            gEpTransInfo[epNum].ulReqSize   -= 2U;
            gEpTransInfo[epNum].ulTransSize += 2U;
            gEpTransInfo[epNum].pucCurBuf   += 2U;
        }

    } else if (gEpTransInfo[epNum].ulReqSize != 0U) {
        /* Get send request size */
        length = gEpTransInfo[epNum].ulReqSize;
        /* Write data to EPx_FIFO */
        while (length != 0U) {
            if (length == 1U) {
                /* Write fraction data */
                chardata = (uint8_t) (*idx16);
                *((uint8_t *) UDC2_EPxFIFO(epNum)) = chardata;
                length = 0U;

                /* Update endpoint information */
                gEpTransInfo[epNum].ulReqSize--;
                gEpTransInfo[epNum].ulTransSize++;
                gEpTransInfo[epNum].pucCurBuf++;
            } else {
                tmp32 = (uint32_t) (*idx16);
                USBD_WriteUDC2Reg(UDC2_EPxFIFO(epNum), tmp32);
                idx16++;
                length -= 2U;   /* Size of uint16 */

                /* Update endpoint information */
                gEpTransInfo[epNum].ulReqSize   -= 2U;
                gEpTransInfo[epNum].ulTransSize += 2U;
                gEpTransInfo[epNum].pucCurBuf   += 2U;
            }
        }
        /* Notification short packet */
        USBD_SetEPCMD(epNum, USBD_CMD_EP_EOP);

    } else {
        /* Send completed */
        gEpTransInfo[epNum].usEpStat = EPSTAT_COMPLETE;

        /* Disable EPx interrupt */
        USBD_SetINTEPMask(epNum, DISABLE);

        /* Callback */
        if (gEpTransInfo[epNum].pfnCallback != NULL) {
            (*(gEpTransInfo[epNum].pfnCallback))(epNum);
        }
    }
    return;
}

/* EOC */
