/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of uDMAC demo for the 
 *          TOSHIBA 'TMPM066' Device Series 
 * @version V0.100
 * @date    2015/09/11
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2015 All rights reserved
 *******************************************************************************
 */

/* This demo is to use uDMAC to transfer RAM data from "src" to "dst" */

#include "tmpm066_udmac.h"
#include "stdbool.h"
#include "tmpm066_cg.h"

/* select DMA mode */
#define  DMA_DEMOMODE_BASIC
/* #define  DMA_DEMOMODE_AUTOMATIC */

/* Reserve 1K RAM area for control data of each DMAC unit. */
/*  each channel use 4 word, 32 channels, primary and alternative */
/*   so, each unit need: 4 x 32 x 2 = 256 word = 1024 bytes */
#if defined (  __ICCARM__  )    /* IAR EWARM */
/* For TMPM066FWQG uDMAC_CFG are defined in file TMPM066FWQG_Flash_For_uDMAC.icf */
uint32_t uDMAC_Control_Data[256U] @ ".uDMAC_CFG";
#elif defined ( __CC_ARM  )     /* Keil MDK */
#define uDMAC_CFG     (0x20000400U)
uint32_t uDMAC_Control_Data[256U] __attribute__((at(uDMAC_CFG)));
#endif

#define  TX_NUMBERS   256U
uint16_t src[TX_NUMBERS];
uint16_t dst[TX_NUMBERS];
uint32_t idx = 0U;

bool isSWTransferFinished(uint8_t Channel);
Result Buffercompare(const uint16_t * Src, const uint16_t * Dst, uint32_t Len);

volatile Result dmaResult = ERROR;

/**
  * @brief  The main function of uDMAC demo
  * @param  None
  * @retval None
  */
int main(void)
{
    DMAC_InitTypeDef init = { 0U };
    uint32_t tmpAddr = 0U;
    bool transferState = true;
    
    /* enable clock supply to DMAC */
    CG_SetFcPeriphA(CG_FC_PERIPH_DMAC, ENABLE);
    
    DMAC_SetTransferType(DMAC_UART0_TX, DMAC_BURST);
    DMAC_SetChannel(DMAC_UART0_TX, ENABLE);
    DMAC_SetMask(DMAC_UART0_TX, ENABLE);
    DMAC_SetPrimaryAlt(DMAC_UART0_TX, DMAC_PRIMARY);
    DMAC_SetChannelPriority(DMAC_UART0_TX, DMAC_PRIOTIRY_NORMAL);

    DMAC_SetPrimaryBaseAddr((uint32_t) & uDMAC_Control_Data);

    for (idx = 0U; idx < TX_NUMBERS; idx++) {
        src[idx] = idx;
    }


    /* set the end address of source and destination */
    tmpAddr = (uint32_t) & src;
    init.SrcEndPointer = tmpAddr + ((TX_NUMBERS - 1U) * sizeof(src[0U]));
    tmpAddr = (uint32_t) & dst;
    init.DstEndPointer = tmpAddr + ((TX_NUMBERS - 1U) * sizeof(dst[0U]));

#if defined(DMA_DEMOMODE_BASIC )
    init.Mode = DMAC_BASIC;
#elif defined(DMA_DEMOMODE_AUTOMATIC)
    init.Mode = DMAC_AUTOMATIC;
#endif

    init.NextUseBurst = DMAC_NEXT_NOT_USE_BURST;
    init.TxNum = TX_NUMBERS;
    init.ArbitrationMoment = DMAC_AFTER_32_TX;

    /* now both src and dst use uint16_t type which is 2bytes long */
    init.SrcWidth = DMAC_HALF_WORD;
    init.SrcInc = DMAC_INC_2B;
    init.DstWidth = DMAC_HALF_WORD;
    init.DstInc = DMAC_INC_2B;

    /* fill to 'control data' area after all members of structure is set */
    DMAC_FillInitData(DMAC_UART0_TX, &init);


    /* enable DMA unit A after all configuration is finished */
    DMAC_Enable();


    do {
        /* because of "init.Mode = DMAC_BASIC" above, here need to trigger it until transfer is finished, */
        /* if DMAC_AUTOMATIC is used, only need to trigger it once */
        DMAC_SetSWReq(DMAC_UART0_TX);

        transferState = isSWTransferFinished(DMAC_UART0_TX);

    } while (transferState == false);

    dmaResult = DMAC_GetBusErrState();

    /* now DMA is finished */
    /* after function Buffercompare() is called, dmaResult == SUCCESS */
    dmaResult = ERROR;
    dmaResult = Buffercompare(src, dst, TX_NUMBERS);

    dmaResult = dmaResult;      /* just to remove compiler's warning */

    while (1U);

}


/**
  * @brief  To compare two half word buffer
  * @param  Src : pointer to point source buffer 
  * @param  Dst : pointer to point destination buffer   
  * @param  Len : how many data will be compared 
  * @retval SUCCESS: two buffer contain same data
            ERROR: two buffer contain different data  
  */
Result Buffercompare(const uint16_t * Src, const uint16_t * Dst, uint32_t Len)
{
    uint32_t tmp = 0U;
    Result result = SUCCESS;

    for (tmp = 0U; tmp < Len; tmp++) {
        if (Src[tmp] != Dst[tmp]) {
            result = ERROR;
            break;
        } else {
            /* Do nothing */
        }
    }

    return result;
}


/**
  * @brief  Check if DMA transfer by software trigger is finished.
  * @param  None
  * @param  Channel: Select channel.
  *   This parameter can be one of the following values:
  *   DMAC_UART0_RX, DMAC_UART0_TX, DMAC_UART1_RX                          
  *   DMAC_UART1_TX, DMAC_I2C0_RX, DMAC_I2C0_TX                            
  *   DMAC_I2C1_RX, DMAC_I2C1_TX, DMAC_TSPI_RX                             
  *   DMAC_TSPI_TX, DMAC_TMRB0_CMP_MATCH, DMAC_TMRB1_CMP_MATCH             
  *   DMAC_TMRB2_CMP_MATCH, DMAC_TMRB3_CMP_MATCH, DMAC_TMRB4_CMP_MATCH     
  *   DMAC_TMRB5_CMP_MATCH, DMAC_TMRB6_CMP_MATCH, DMAC_TMRB7_CMP_MATCH     
  *   DMAC_TMRB0_INPUT_CAP0, DMAC_TMRB1_INPUT_CAP0, DMAC_TMRB2_INPUT_CAP0  
  *   DMAC_TMRB3_INPUT_CAP0, DMAC_TMRB4_INPUT_CAP0, DMAC_TMRB5_INPUT_CAP0  
  *   DMAC_TMRB6_INPUT_CAP0, DMAC_TMRB7_INPUT_CAP0, DMAC_TOP_PRIORITY_ADC 
  *   DMAC_ADC_COMPLETION
  * @retval 
  *   false: transfer is not finished
  *   true: transfer is finished
  */
bool isSWTransferFinished(uint8_t Channel)
{
    DMAC_PrimaryAlt PriAlt = DMAC_PRIMARY;
    uint32_t Addr = 0U;
    uint32_t tmpCtrl = 0U;
    bool ret = false;

    assert_param(IS_DMAC_CHANNEL_ALL(Channel));

    PriAlt = DMAC_GetPrimaryAlt(Channel);

    if (PriAlt == DMAC_PRIMARY) {
        Addr = DMAC_GetBaseAddr(DMAC_PRIMARY);
    } else {
        Addr = DMAC_GetBaseAddr(DMAC_ALTERNATE);
    }

    /* add offset for channel, each channel use 4 words RAM */
    Addr += (16U * (uint32_t) Channel);

    /* get 'control data' */
    tmpCtrl = *((uint32_t *) (Addr + 8U));

    /* low 3bits is 'cycle_ctrl', it becomes 0 when transfer is finished */
    if (tmpCtrl & 0x07U) {
        /* Do nothing */
    } else {
        ret = true;
    }

    return ret;
}


#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1U) {
        __NOP();
    }
}
#endif
